/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/*
 * function __check_sum
 *
 * parameter:
 *  p		pointer to __access, __association or __text
 *
 * returns:
 *  void
 *
 * abstract:
 *   calculate a checksum over the first 4 short's of the data
 *
*/

void
__check_sum (void *p)
{
    unsigned short	*ptr = (unsigned short *)p;
    
    ptr[4] = ptr[0] + ptr[1] + ptr[2] + ptr[3] + 0x14;
} /* __ check_sum */

/*
 * function __text_check_sum
 *
 * parameter:
 *  p		pointer to __access, __association or __text
 *
 * returns:
 *  int		1 ... checksum ok
 *		0 ... checksum not ok
 *
 * abstract:
 *   test the checksum over the first 4 short's of the data
 *
*/

int
__test_check_sum (void *p)
{
    unsigned short	*ptr = (unsigned short *)p;
    unsigned short	chks = ptr[0] + ptr[1] + ptr[2] + ptr[3] + 0x14;
    
    return (ptr[4] == chks);
} /* __test_check_sum */
