/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (connectfail);
EXCEPTION (notassociated);
EXCEPTION (rangefail);
EXCEPTION (spacefail);

/*
 * function set_buffer
 *
*/

static int
set_buffer (short fst_index)
{
    __fs_table_entry	*fse = &__fs_table[fst_index];
    
    return (0);
} /* set_buffer */

/*
 * function move_back_bytes
 *
*/

static int
move_back_bytes (__fs_table_entry *fse, short fst_index, 
		 int move_val)
{
    int		status = 0;
    
    switch (move_val)
    {
    case -2:
	status = set_buffer (fst_index);
	fse->offset_in_buffer--;
	break;
    case -1:
	status = set_buffer (fst_index);
	break;
    default:
	fse->offset_in_buffer -= move_val;
	break;
    }
    return (status);
} /* move_back_bytes */


/*
 * function search_back_record
 *
*/

static int
search_back_record (__fs_table_entry *fse, short fst_index)
{
    int		record_size = 0;
    /* !!!!!!!!!!! */
    return (0);
} /* search_back_record */

/*
 * function move_file_back_form_dyn_rec
 *
*/

static int
move_file_back_form_dyn_rec (short fst_index, int loop)
{
    __fs_table_entry	*fse = &__fs_table[fst_index];
    int			status;
    
    while (loop > 0)
    {
	if (move_back_bytes (fse, fst_index, 1))
	    return (1);
	if (fse->buffer_ptr[fse->offset_in_buffer] != CTRL_Z)
	    status = move_back_bytes (fse, fst_index, 2);
	else
	    status = move_back_bytes (fse, fst_index, 1);
	if (status)
	    return (1);
	if (search_back_record (fse, fst_index))
	    return (1);
	loop--;
    }
    return (0);
} /* move_file_back_form_dyn_rec */


/*
 * function move_file_form_dyn_rec
 *
*/

static int
move_file_form_dyn_rec (short fst_index, int loop)
{
    __fs_table_entry	*fse = &__fs_table[fst_index];
    /* !!!!!!!!!!!!!!!*/
    return (0);
} /* move_file_back_dyn_rec */


/*
 * function move_file_back_rec
 *
*/

static int
move_file_back_rec (short fst_index, int loop)
{
    __fs_table_entry	*fse = &__fs_table[fst_index];
    /* !!!!!!!!!!!!!!*/
    return (0);
} /* move_file_back_rec */


/*
 * function move_file_ptr_dyn_rec
 *
*/

static int
move_file_ptr_rec (short fst_index, int loop)
{
    __fs_table_entry	*fse = &__fs_table[fst_index];
    /* !!!!!!!!!!!!! */
    return (0);
} /* move_file_ptr_rec */


/*
 * function varying_rec_move_value_calc
 *
*/

static int
varying_rec_move_value_calc (short fst_index, int *help_base_index,
			     int is_text, int where_is_last)
{
    __fs_table_entry	*fse = &__fs_table[fst_index];
    __s_block		save_block;
    int			back, status;
    
    __SAVE_VALUES (save_block, *fse);
    if (*help_base_index >= fse->curr_index)
    {
	/* here: help_base_index >= curr_index
	   This branch is reached, when connect called first in a program.
	   number_of_records counts, how many records have to be skipped
	   afterwards. if !buffer_read them read byte into buffer.
	   skipping will be donw forwards. */
	if (!fse->buffer_read)
	{
	    if (__read_buffer (fst_index, 0))
	    {
		__RESTORE_VALUES (*fse, save_block);
		return (1);
	    }
	    fse->offset_in_buffer = 0;
	    fse->buffer_read = 1;
	}
	back = 0;
    }
    else if (((fse->curr_index / 2) < *help_base_index) &&
	     (*help_base_index < fse->curr_index))
    {
	/* here: curr_index/2 < help_base_index < curr_index
	   In this branch the buffer is filled by a former connect call,
	   therefore it is not necessary to check for a possible
	   __check_buffer call. Skipping will be done backwards. */
	if (fse->bit_list_buffer)
	{
	    if (__write_buffer (fst_index))
	    {
		__RESTORE_VALUES (*fse, save_block);
		return (1);
	    }
	}
	back = 1;
    }
    else
    {
	/* here: help_base_index < curr_index
	   skipping will be done in forward direction from the start of the file.
	   Therefore __check_buffer must be called to set the file pointer to
	   the beginning of the file. */
	fse->curr_index = 0;
	if (fse->bit_list_buffer)
	{
	    if (__write_buffer (fst_index))
	    {
		__RESTORE_VALUES (*fse, save_block);
		return (1);
	    }
	}
	if (__read_buffer (fst_index, 0))
	{
	    __RESTORE_VALUES (*fse, save_block);
	    return (1);
	}
	fse->offset_in_buffer = 0;
	back = 0;
    }

    if (__FORMATBITSET (fse->bit_list_ass) || is_text)
    {
	/* formatted */
	if (back)
	    status = move_file_back_form_dyn_rec (fst_index,
						  fse->curr_index - *help_base_index);
	else
	    status = move_file_form_dyn_rec (fst_index,
					     fse->curr_index - *help_base_index);
    }
    else
    {
	/* unformatted */
	if (back)
	    status = move_file_back_rec (fst_index,
					 fse->curr_index - *help_base_index);
	else
	    status = move_file_ptr_rec (fst_index,
					fse->curr_index - *help_base_index);
    }
    if (status)
    {
	__RESTORE_VALUES (*fse, save_block);
    }
    return (status);
} /* varying_rec_move_value_calc */


/*
 * function fixed_rec_move_value_calc
 *
*/

static int
fixed_rec_move_value_calc (short fst_index, int *help_base_index,
			   char where, int varying_mode)
{
    __fs_table_entry	*fse = &__fs_table[fst_index];
    int			record_len;
    
    if (varying_mode && !__DIRECTBITSET (fse->bit_list_ass) &&
	__STREAMBITSET (fse->bit_list_ass))
	/* when the store location is varying and not stream, the whole
	   length of the record is incremented by two. */
	record_len = fse->rec_len_acc + 2;
    else
	record_len = fse->rec_len_acc;
    
    if (where == Last)
    {
	/* if Last specified, calculate the number of records in file */
	*help_base_index = fse->file_length / record_len;
    }

    /* return the new position in the file */
    return (*help_base_index * record_len);
} /* fixed_rec_move_value_calc */

/*
 * function _connect
 *
 * parameters:
 *  ptr			pointer to ACCESS or TEXT-location
 *  ass			pointer to ASSOCIATION location
 *  descr		pointer to a structure, which is created by the compiler
 *			and holds all information about the properties of the
 *			ACCESS or TEXT location.
 *  usage		denotes read- and write access for the ACCESS or TEXT location
 *  where		denotes, where the file should be positioned
 *  index		denotes some sort of BASE INDEX, when file is indexable
 *
 * returns:
 *  void
 *
 * exceptions:
 *  connectfail		property mismatch
 *			error during read- or write_buffer
 *  notassociated	ass is not associated
 *  rangefail		`where' out of range
 *  spacefail		can't get file buffer
 *
 * abstract:
 *  CONNECT is used to change the status of an ASSOCIATION from the socalled
 *  FILE HANDLING STATE to the DATA TRANSFER STATE. The ASSOCIATION denotes
 *  in this case the 'outside world object', the ACCESS denotes all record
 *  properties of the single records of the files, devices -
 *  such as (dynamic) size, random access to the file .....   .
 *
*/

void
_connect (void *ptr, __association *ass, __acc_description *descr,
	  char usage, char where, short index)
{
    short		list_acc, ass_list, save_index_diff;
    int			idx = ass->fileindex, init_values;
    __fs_table_entry	*fse = &__fs_table[idx];
    __access		*acc = (__access *)ptr;
    __text		*txt = (__text *)ptr;
    
    /* make checks */
    if (!__ISASSOCIATED (ass))
	CAUSE (notassociated);

    list_acc = descr->attributes & 0xfc20;
    ass_list = fse->bit_list_ass;
    
    /* check if bit lists of association and access are matching
       file not existing                  ||
       file not old                       ||
       directory                          ||
       no write & WO/RW                   ||
       no read & RO/RW                    ||
       index & dynamic                    ||
       stream && (text || !varying)       ||
       ass !connected & where == Same     ||
       indexed access && !index-expr      ||
       indexed && (stream || device)      ||
       RW && ! (indexed || device != Stdin, Stdout, Stderr)
    */
    if (!__EXISTINGBITSET (ass_list)                                       ||
	__NEWBITSET (ass_list)                                             ||
	__DIRBITSET (ass_list)                                             ||
	(!__WRITEBITSET (ass_list) && usage > ReadOnly)                    ||
	(!__READBITSET (ass_list) && usage != WriteOnly)                   ||
	(__ACDDYNAMIC (list_acc) && __ACDINDEXED (list_acc))               ||
	((!__DIRECTBITSET (ass_list) && __STREAMBITSET (ass_list)) &&
	 (__ACDTEXT (list_acc) || !__ACDVARYING (list_acc)))               ||
	(!fse->ref_access && (where == Same))                              ||
	(__ACDINDEXED (list_acc) && !__ACDVALIDINDEX (list_acc))              ||
	(__ACDINDEXED (list_acc) &&
	 ((!__DIRECTBITSET (ass_list) && __STREAMBITSET (ass_list)) ||
	  (fse->device_script != File)))                                    ||
	((usage == ReadWrite) &&
	 !(__ACDINDEXED (list_acc) || (fse->device_script < Stdin) ||
	   (fse->device_script > Stderr))))
    {
	/* fault */
	fse->status_word = EIO;
	CAUSE (connectfail);
    }

    /* variable attr in association is dynamic attr in acc-descr */
    if (__ACDDYNAMIC (list_acc))
	__SETVARIABLEBIT (ass_list);
    else
	__CLRVARIABLEBIT (ass_list);
    
    if (__ACDINDEXED (list_acc))
    {
	save_index_diff = index - descr->index_low;
	fse->low_index_bound = descr->index_low;
	fse->high_index_bound = descr->index_high;
	__SETDIRECTBIT (ass_list);
    }
    else
    {
	save_index_diff = fse->low_index_bound = fse->high_index_bound = 0;
    }
    
    if (__ACCESSVALID (ptr))
    {
	/* access is connected */
	if ((acc == fse->ref_access) || (txt == fse->ref_text))
	    /* very good -- old connection */
	    init_values = 0;
	else
	{
	    fse->status_word = EIO;
	    CAUSE (connectfail);
	}
    }
    else
    {
	if (fse->ref_access)
	{
	    /* if the association is connected to another access,
	       disconnect it */
	    _disconnect (fse->ref_access);
	}
	fse->rec_len_acc = descr->length_rec_mode;
	init_values = 1;
    }
    /* end make checks */


    if (fse->device_script == File)
    {
	int		help_base_index, move_value;
	
	if (!fse->buffer_ptr && fse->number_buffer_bytes)
	{
	    if ((fse->buffer_ptr = malloc (fse->number_buffer_bytes)) == NULL)
		CAUSE (spacefail);
	}
	/* set indeces */
	switch (where)
	{
	case First:
	    help_base_index = 0;
	    __ACCSETFIRST (list_acc);
	    __ACCCLRSAME (list_acc);
	    if (!fse->buffer_read)
	    {
		/* if it is the first connect for this access */
		if (__read_buffer (ass->fileindex, 0))
		    CAUSE (connectfail);
		fse->offset_in_buffer = 0;
		fse->buffer_read = 1;
	    }
	    else if (!__ACDINDEXED (list_acc))
		/* if not indexed */
		__check_buffer (ass->fileindex, 0, 0);
	    fse->curr_index = 0;
	    break;
	case Same:
	    if (!fse->read_write_access)
		/* when there was no read/write access since the last connect
		   operation, the value in index_diff must be added to
		   curr_index */
		help_base_index = fse->curr_index + fse->index_diff;
	    else
		help_base_index = fse->curr_index;
	    __ACCCLRFIRST (list_acc);
	    __ACCSETSAME (list_acc);
	    break;
	case Last:
	    /* write the highest possible record number into help_base_index */
	    help_base_index = INT_MAX;
	    __ACCSETFIRST (list_acc);
	    __ACCSETSAME (list_acc);
	    break;
	default:
	    fse->status_word = ERANGE;
	    CAUSE (rangefail);
	    break;
	}

	fse->index_diff = save_index_diff;
	fse->read_write_access = 0;
	
	if (help_base_index |= fse->curr_index)
	{
	    /* calculate new file position */
	    if (__ACDDYNAMIC (list_acc))
	    {
		if (varying_rec_move_value_calc (ass->fileindex, &help_base_index,
						 __ACCTEXT (list_acc),
						 where == Last))
		    CAUSE (rangefail);
	    }
	    else
	    {
		/* records with fixed size on the file */
		move_value = fixed_rec_move_value_calc (ass->fileindex,
							&help_base_index,
							where,
							__ACDVARYING (list_acc));
		
		if (!fse->buffer_read)
		{
		    /* if it is the first connect to this access */
		    if (__read_buffer (ass->fileindex, move_value))
			CAUSE (connectfail);
		    fse->offset_in_buffer = move_value % 512;
		    fse->buffer_read = 1;
		}
		else if (!__ACDINDEXED (list_acc))
		    __check_buffer (ass->fileindex, move_value, move_value % 512);
	    }
	    fse->curr_index = help_base_index;
	}

	if (!__DIRECTBITSET (ass_list) &&
	    !__STREAMBITSET (ass_list) &&
	    (usage == WriteOnly))
	{
	    if (fse->file_length >= (fse->abs_file_pos + fse->offset_in_buffer))
	    {
		fse->file_length = fse->abs_file_pos + fse->offset_in_buffer;
		/* set_file_size (fse->file_length, fse->handle); */
		fse->end_of_buffer = fse->offset_in_buffer - 1;
	    }
	}
	fse->base_index = help_base_index;
	/* end set indeces */
    }
    
    switch (usage)
    {
    case ReadOnly:
	__SETREADBIT (list_acc);
	__CLRWRITEBIT (list_acc);
	break;
    case WriteOnly:
	__CLRREADBIT (list_acc);
	__SETWRITEBIT (list_acc);
	break;
    case ReadWrite:
	__SETREADBIT (list_acc);
	__SETWRITEBIT (list_acc);
	break;
    default:
	fse->status_word = ERANGE;
	CAUSE (rangefail);
	break;
    }
    fse->bit_list_acc = list_acc;
    if (init_values)
    {
	if (__ACCTEXT (list_acc))
	{
	    /* acess determines a text location */
	    fse->ref_text = txt;
	    fse->ref_access = txt->ref_access = &txt->text_access;
	    txt->fileindex = ass->fileindex;
	    txt->text_ptr = &txt->text_access.next;
	    txt->actual_index = 0;
	}
	else
	    /* not text */
	    fse->ref_access = acc;
	ass->ref_access = fse->ref_access;
	fse->ref_access->ref_ass = ass;
	fse->ref_access->fileindex = ass->fileindex;
	fse->ref_access->next = 0;
    }

    if (__ACCTEXT (list_acc))
    {
	/* TEXT */
	txt->bitlist = list_acc;
	__check_sum (txt);
	__ACCCLRTEXT (list_acc);
    }
    
    fse->ref_access->bitlist = list_acc;
    __check_sum (fse->ref_access);
    
    /* update bitlist of association */
    fse->bit_list_ass = ass->bitlist = ass_list;
    __check_sum (ass);
    
} /* _connect */
