/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (createfail);
EXCEPTION (notassociated);

/*
 * function _create
 *
 * parameters:
 *  ass		pointer to association location
 *
 * returns:
 *  void
 *
 * exceptions:
 *  createfail		file already exists
 *			file is a device or at the moment a directory
 *			error during open
 *  notassociated	ass is not associated
 *
 * abstract:
 *  This procedure creates a with the ASSOCIATION connected file.
 *
*/

void
_create (__association *ass)
{
    int			idx = ass->fileindex;
    __fs_table_entry	*fse = &__fs_table[idx];
    
    if (!__ISASSOCIATED (ass))
	CAUSE (notassociated);
    
    if (!__NEWBITSET (fse->bit_list_ass))
	CAUSE (createfail);
    
    switch (fse->device_script)
    {
    case File:
	if ((fse->handle = open (fse->file_name,
				 O_WRONLY | O_CREAT | O_TRUNC, 0644)) < 0)
	{
	    /* somethings wrong */
	    fse->status_word = errno;
	    CAUSE (createfail);
	}
	fse->file_attr = 0644;
	fse->file_length = 0;

	/* reset NEW-bit, set OLD-bit and EXISTING-bit */
	__CLRNEWBIT (fse->bit_list_ass);
	__SETOLDBIT (fse->bit_list_ass);
	__SETEXISTINGBIT (fse->bit_list_ass);

	__CLRNEWBIT (ass->bitlist);
	__SETOLDBIT (ass->bitlist);
	__SETEXISTINGBIT (ass->bitlist);
	__check_sum (ass);
	break;
    case Directory:
	fse->status_word = EISDIR;
	CAUSE (createfail);
	break;
    default:
	fse->status_word = EIO;
	CAUSE (createfail);
	break;
    }
} /* _create */
