/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (deletefail);
EXCEPTION (notassociated);

/*
 * function _delete
 *
 * parameters:
 *  ass		pointer to association location
 *
 * returns:
 *  void
 *
 * exceptions:
 *  deletefail		file is a device or at the moment a directory
 *			error during close or unlink
 *			file not existing or not writeable
 *  notassociated	ass is not associated
 *
 * abstract:
 *  This procedure deletes the 'outside world object' of an ASSOCIATION.
 *
*/

void
_delete (__association *ass)
{
    int			idx = ass->fileindex;
    __fs_table_entry	*fse = &__fs_table[idx];
    
    if (!__ISASSOCIATED (ass))
	CAUSE (notassociated);
    
    if (!fse->ref_access &&
	__EXISTINGBITSET (fse->bit_list_ass) &&
	(__WRITEBITSET (fse->bit_list_ass) || __DIRBITSET (fse->bit_list_ass)))
    {
	/* not connected, existing and writeable */
	switch (fse->device_script)
	{
	case File:
	    /* first close the file */
	    if (close (fse->handle) < 0)
	    {
		fse->status_word = errno;
		CAUSE (deletefail);
	    }
	    if (unlink (fse->file_name) < 0)
	    {
		fse->status_word = errno;
		CAUSE (deletefail);
	    }
	    break;
	case Directory:
	    fse->status_word = EISDIR;
	    CAUSE (deletefail);
	    break;
	default:
	    fse->status_word = EIO;
	    CAUSE (deletefail);
	    break;
	}
	/* clear the existing bits */
	__CLREXISTINGBIT (fse->bit_list_ass);
	__CLREXISTINGBIT (ass->bitlist);
	__check_sum (ass);
    }
    else
    {
	fse->status_word = EACCES;
	CAUSE (deletefail);
    }
} /* _delete */
