/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (notconnected);
EXCEPTION (writefail);

/*
 * function _disconnect
 *
 * parameter:
 *  ptr		address of ACCESS or TEXT location
 *
 * returns:
 *  void
 *
 * exceptions:
 *  notconnected
 *  writefail
 *
 * abstract:
 *  DISCONNECT ends the relation between a ACCESS (or TEXT) and a ASSOCIATION
 *  location. The ASSOCIATION location returns from the DATA TRANSFER STATE
 *  to the FILE HANDLING STATE.
 *
*/

void
_disconnect (void *ptr)
{
    __access		*acc = (__access *)ptr;
    __fs_table_entry	*fse = &__fs_table[acc->fileindex];
    
    if (!__ISCONNECTED (ptr))
	CAUSE (notconnected);

    /* check if there is a file buffer and if the buffer values have changed */
    if (fse->buffer_ptr)
    {
	if (__write_buffer (acc->fileindex))
	{
	    fse->status_word = errno;
	    CAUSE (writefail);
	}
	free (fse->buffer_ptr);
	fse->buffer_ptr = NULL;
    }
    
    if (fse->ref_text)
    {
	/* disconnect a text location */
	fse->ref_text->ref_access = NULL;
	fse->ref_text = NULL;
	fse->actual_index = fse->max_act_index = 0;
    }
    
    fse->ref_access->ref_ass = NULL;
    fse->ref_access = NULL;
    fse->bit_list_acc = 0;
    fse->base_index = fse->curr_index = 0;
    fse->rec_len_acc = 0;
    fse->abs_file_pos = 0;
    fse->offset_in_buffer = fse->end_of_buffer = 0;
    fse->buffer_read = 0;
    fse->ref_association->ref_access = NULL;
    __check_sum (fse->ref_association);
} /* _disconnect */
