/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (notassociated);
EXCEPTION (protectionfail);
EXCEPTION (writefail);

/*
 * function _dissociate
 *
 * parameters:
 *  ass		pointer to association location
 *
 * returns:
 *  void
 *
 * exceptions:
 *  notassociated	ass is not associated
 *  protectionfail	error during close
 *  writefail		error during write
 *
 * abstract:
 *  This CHILL - 286 FILE I/O standard routine ends the relation between
 *  an 'outside world object' and an ASSOCIATION.
 *
*/

void
_dissociate (__association *ass)
{
    int			idx = ass->fileindex;
    __fs_table_entry	*fse = &__fs_table[idx];
    
    if (!__ISASSOCIATED (ass))
	CAUSE (notassociated);
    
    if (fse->ref_access)
    {
	/* if still connected (also for devices) */
	if (fse->buffer_ptr)
	{
	    /* if there is a file buffer, check for buffer has to be
	       written to file and return memory afterwards */
	    if (fse->bit_list_buffer)
		/* write the buffer to the file */
		if (__write_buffer (idx))
		{
		    fse->status_word = errno;
		    CAUSE (writefail);
		}
	    free (fse->buffer_ptr);
	}
	
	/* mark association pointer in access as invalid */
	fse->ref_access->ref_ass = NULL;

	/* text mark access pointer as invalid */
	if (__ACCTEXT (fse->bit_list_acc))
	    fse->ref_text->ref_access = NULL;
    }

    if ((fse->device_script == File) &&
	(__EXISTINGBITSET (fse->bit_list_ass)))
    {
	/* close the file */
	if (close (fse->handle) < 0)
	{
	    fse->status_word = errno;
	    CAUSE (protectionfail);
	}
    }
    
    /* mark fs-table entry as free */
    fse->ref_association = NULL;

    /* free the filename */
    free (fse->file_name);
    fse->file_name = NULL;
    
    /* invalidate association location */
    ass->ref_access = NULL;
    ass->checksum = ass->bitlist = ass->fileindex = 0;
} /* _dissociate */
