/* File-oriented definitions for GNU CHILL
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef _FILEIO_H
#define _FILEIO_H

/* some useful definitions */
#ifndef NULL
#define NULL	(void *)0
#endif

/* for association */
#define __ISASSOCIATED(x)	__test_check_sum(x)
#define __DIRBITSET(x)		(x & 0x40)
#define __FORMATBITSET(x)	(x & 0x8000)
#define __SHAREDBITSET(x)	(x & 0x80)
#define __WRITEBITSET(x)	(x & 0x100)
#define __SETWRITEBIT(x)	(x |= 0x100)
#define __CLRWRITEBIT(x)	(x &= ~0x100)
#define __READBITSET(x)		(x & 0x200)
#define __SETREADBIT(x)		(x |= 0x200)
#define __CLRREADBIT(x)		(x &= ~0x200)
#define __NEWBITSET(x)		(x & 0x800)
#define __CLRNEWBIT(x)		(x &= ~0x800)
#define __SETOLDBIT(x)		(x |= 0x400)
#define __SETEXISTINGBIT(x)	((x) |= 0x10)
#define __CLREXISTINGBIT(x)     ((x) &= ~0x10)
#define __EXISTINGBITSET(x)	(x & 0x10)
#define __DIRECTBITSET(x)	(x & 0x2000)
#define __SETDIRECTBIT(x)	(x |= 0x2000)
#define __STREAMBITSET(x)	(x & 0x4000)
#define __VARIABLEBITSET(x)	(x & 0x1000)
#define __SETVARIABLEBIT(x)	(x |= 0x1000)
#define __CLRVARIABLEBIT(x)	(x &= ~0x1000)
#define __WRITTEN(x)		(x & 1)
#define __READWRITTEN(x)	(x & 2)

/* for access and text */
#define __ISCONNECTED(x)	__test_check_sum(x)
#define __ACCTEXT(x)		(x & 0x800)
#define __ACCCLRTEXT(x)		(x &= ~0x800)
#define __ACCESSVALID(x)	__test_check_sum(x)
#define __ACCFIRST(x)		(x & 0x80)
#define __ACCSETFIRST(x)	(x |= 0x80)
#define __ACCCLRFIRST(x)	(x &= ~0x80)
#define __ACCSAME(x)		(x & 0x40)
#define __ACCSETSAME(x)		(x |= 0x40)
#define __ACCCLRSAME(x)		(x &= ~0x40)
#define __ACCWRITE(x)		(x & 0x100)
#define __ACCREAD(x)		(x & 0x200)
#define __ACCSETFF(x)		(x |= 0x8)
#define __ACCCLRFF(x)		(x &= ~0x8)
#define __ACCFF(x)		(x & 0x8)
#define __ACCCLRIOT(x)		(x &= ~0x7)
#define __ACCSETIOT1(x)		(x |= 0x1)
#define __ACCSETIOT2(x)		(x |= 0x2)
#define __ACCSETIOT3(x)		(x |= 0x4)
#define __ACCSETIOT4(x)		(x |= 0x3)
#define __ACCSETIOT5(x)		(x != 0x6)
#define __ACCIOT1(x)		((x & 7) == 1)
#define __ACCIOT2(x)		((x & 7) == 2)
#define __ACCIOT3(x)		((x & 7) == 4)
#define __ACCIOT4(x)		((x & 7) == 3)
#define __ACCIOT5(x)		((x & 7) == 6)
#define __ACCDYNAMIC(x)		(x & 0x8000)
#define __ACCVARYING(x)		(x & 0x1000)
#define __ACCOUTOFF(x)		(x & 0x400)

/* for access descriptor */
#define __ACDINDEXED(x)		(x & 0x4000)
#define __ACDDYNAMIC(x)		(x & 0x8000)
#define __ACDTEXT(x)		(x & 0x800)
#define __ACDVARYING(x)		(x & 0x1000)
#define __ACDVALIDINDEX(x)	(x & 0x20)

typedef struct
{
    long		base_index;
    long		curr_index;
    int			offset_in_buffer;
    unsigned short     	bit_list_buffer;
    long		abs_file_pos;
} __s_block;

#define __SAVE_VALUES(x,y)				\
    (x).base_index = (y).base_index,			\
    (x).curr_index = (y).curr_index,			\
    (x).offset_in_buffer = (y).offset_in_buffer,	\
    (x).bit_list_buffer = (y).bit_list_buffer,		\
    (x).abs_file_pos = (y).abs_file_pos

#define __RESTORE_VALUES(x,y)				\
    (x).base_index = (y).base_index,			\
    (x).curr_index = (y).curr_index,			\
    (x).offset_in_buffer = (y).offset_in_buffer,	\
    (x).bit_list_buffer = (y).bit_list_buffer,		\
    (x).abs_file_pos = (y).abs_file_pos

#define CTRL_Z		0x1a
#define FF		'\f'
#define LF		'\n'
#define CR		'\r'

typedef __format_result (*outfun) (int, ...);

extern void	*__xmalloc_();
extern void	*__xrealloc_();

/* fileno's for stdout, stdin, stderr */
#ifndef STDIN_FILENO
#define STDIN_FILENO	0
#endif
#ifndef STDOUT_FILENO
#define STDOUT_FILENO	1
#endif
#ifndef STDERR_FILENO
#define STDERR_FILENO	2
#endif

extern __fs_table_entry	__fs_table[MAXUSEDFILES];
extern __fs_table_entry	*__fs_ptr;

#endif
