/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (empty);

/*
 * function _find_next_file
 *
 * parameters:
 *  ffbuf		pointer to find_file_buffer
 *
 * returns:
 *  void
 *
 * exceptions:
 *  empty
 *
 * abstract:
 *  return next file found during find-file operation
 *
*/

void
_find_next_file (__find_file_buffer *ffbuf)
{
    __ff_internal_file	*wptr;
    
    if (!ffbuf->ptr)
	CAUSE (empty);
    
    if (ffbuf->ptr->cur_index >= ffbuf->ptr->nr_files)
    {
	/* already all files returned */
	if (ffbuf->ptr->f)
	    free (ffbuf->ptr->f);
	free (ffbuf->ptr);
	ffbuf->ptr = NULL;
	CAUSE (empty);
    }
    
    wptr = ffbuf->ptr->f[ffbuf->ptr->cur_index];
    ffbuf->ptr->f[ffbuf->ptr->cur_index++] = NULL;
    ffbuf->attributes = wptr->attrs;
    ffbuf->time = wptr->time;
    ffbuf->filesize = wptr->size;
    ffbuf->name.len = strlen (wptr->name);
    strncpy (&ffbuf->name.body[0], wptr->name, ffbuf->name.len);
    free (wptr);
} /* _find_next_file */
