/* Implement POWERSET runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#include "config.h"
#include <stdio.h>
#include "powerset.h"

extern void __cause_exception (char *exname, char *file, int lineno);
void __store_out (char *out, unsigned long  outlen,
		  unsigned long value);

/*
 * function __flsetclrpowerset
 *
 * parameters:
 *	ps		powerset
 *	bitlength	length of powerset
 *	out		result
 *	outlen		length of result
 *	minval		minimum value of result
 *
 * returns:
 *	int		0 .. nothing found
 *			1 .. something found
 * exceptions:
 *  none
 *
 * abstract:
 *  Find last bit set in a powerset and return the corresponding value
 *  in *out and clear this bit. Return 0 for no more found, else 1.
 *
 */
int
__flsetclrpowerset (ps, bitlength, out, outlen, minval)
     SET_WORD      *ps;
     unsigned long  bitlength;
     char          *out;
     unsigned long  outlen;
     long           minval;
{
  long bitno;

  if (bitlength <= SET_CHAR_SIZE)
    {
      for (bitno = bitlength - 1; bitno >= 0; bitno--)
	if (GET_BIT_IN_CHAR (*((SET_CHAR *)ps), bitno))
	  break;
      if (bitno < 0)
	return 0;
      else
	{
	  __store_out (out, outlen, bitno + minval);
	  CLEAR_BIT_IN_CHAR (*((SET_CHAR *)ps), bitno);
	  return 1;
	}
    }
  else if (bitlength <= SET_SHORT_SIZE)
    {
      for (bitno = bitlength - 1; bitno >= 0; bitno--)
	if (GET_BIT_IN_SHORT (*((SET_SHORT *)ps), bitno))
	  break;
      if (bitno < 0)
	return 0;
      else
	{
	  __store_out (out, outlen, bitno + minval);
	  CLEAR_BIT_IN_SHORT (*((SET_SHORT *)ps), bitno);
	  return 1;
	}
    }
  else
    {
      SET_WORD	*endp = ps;
      SET_WORD	*p = ps + BITS_TO_WORDS(bitlength) - 1;
      unsigned long cnt = ((bitlength - 1) / SET_WORD_SIZE) * SET_WORD_SIZE;
      SET_WORD	c;
      /* FIXME bitorder problems */    
      do
	{
	  if (c = *p)
	    {
	      /* found a bit set .. calculate which */
	      for (bitno = SET_WORD_SIZE - 1; bitno >= 0; bitno--)
		if (GET_BIT_IN_WORD(c, bitno))
		  break;
	      __store_out (out, outlen, cnt + bitno + minval);
	      CLEAR_BIT_IN_WORD(c, bitno);
	      *p = c;
	      return 1;
	    }
	  cnt -= SET_WORD_SIZE;
	  p--;
	} while (p >= endp);
      return 0;
    }
}
