/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/*
 * function _get_directory
 *
 * parameters:
 *  outdir		location to place current working directory
 *  outlen		length of outdir
 *
 * returns:
 *  void
 *
 * exeptions:
 *  spacefail		outstr to short
 *  fileiofail		getcwd delivered an error
 *
 * abstract:
 *  returns the current working directory
 *
*/

void
_get_directory (TVaryingCharType *outstr, int outlen)
{
    char		*cwd;
    int			cwdlen;
#ifdef VMS
    extern char *getcwd ();
#endif
    if ((cwd = getcwd (NULL, MAXPATHLEN + 1)) == NULL)
	CAUSE (fileiofail);
    cwdlen = strlen (cwd);
    if (cwdlen > outlen)
	CAUSE (spacefail);
    strncpy (&outstr->body[0], cwd, cwdlen);
    outstr->len = cwdlen;
    free (cwd);
} /* _get_directory */
