/*
 * $Id: iomodes.h,v 1.9 1994/10/06 16:41:24 moser Exp $
 * 
 * File:		modes.h
 * Date:		Wed Jun 10 14:02:41 1992
 * Author:		Wilfried Moser (fs_moser)
 * 
 * Abstract:
 *	typedefinitions for CHILL-IO library
 *
 * Modifications:
 *	$Log: iomodes.h,v $
 * Revision 1.9  1994/10/06 16:41:24  moser
 *         * wrformat.c (default_conv): Remove processing of
 *         POWERSET's.
 *
 *         * rdformat.c (rf_parse_iolist): Remove processing of
 *         POWERSET's.
 *
 *         * iomodes.h: Remove definition for POWERSET's.
 *
 * Revision 1.8  1994/06/27 09:31:58  moser
 * 	* iomodes.h: Change layout of __tmp_WIO_list that it is always
 * 	the same, regardless if compiled with -fpack or not.
 *
 * Revision 1.7  1994/05/10 12:22:55  moser
 * 	* iomodes.h: Change names of __access, __association and
 * 	__text to __access_type, __association_type and
 * 	__text_type. __access is already defined in one of the
 * 	linux header files.
 *
 * Revision 1.6  1994/03/30 04:48:44  bothner
 * Merged in changes from Wilfried.
 * 	* Makefile.in (LIBOBJS2): Add new modules rdformat.o rdstring.o
 * 	  rdunixstd.o
 * 	* iomodes.h: Remove unnecessary definitions. Redefine interface to
 * 	  READTEXT/WRITETEXT functions.
 * 	* rdstring.c (_readstring): Rework function.
 * 	* rdformat.c: Rework functionality.
 * 	* rdunixstd.d: New module. This is designed to implement
 * 	   READTEXT (stdin...).
 * 	* wrformat.c (copy_text): Change return value of function. In case
 * 	  of an empty string this function has failed.
 * 	* wrformat.c: Change formating of a NULL pointer to NULL instead
 * 	  of 0.
 * 	* wrformat.c (PUT_CHARS): Change macro according to new return
 * 	  value of function copy_text.
 *
 * Revision 1.5  1994/03/02 18:45:18  bill
 * 	* iomodes.h: Add definition for POWERSET's
 * 	* wrunixstd.c: Change the write call to an fprintf because
 * 	  a system we have doesn't support write.
 * 	* wrformat.c: Add processing of SET's and POWERSET's
 *
 * Revision 1.4  1994/01/26 22:36:19  bill
 * 	* Makefile.in: clean up for cross compilation. Remove a lot of
 * 	  functions (file-io) which are not used at the moment.
 * 	* iomodes.h: clean up the datatypes passed to the writetext
 * 	  functions.
 * 	* unhex.c: Comment out include of gvarargs.h (not needed).
 * 	* rtsdummy.c: Comment out include of gvarargs.h (not needed).
 * 	* wrformat.c: change various procedure arguments according to the
 * 	  new type passed by the compiler. Remove include of fcntl.h and
 * 	  sys/stat.h (not needed).
 * 	* wrformat.c (default_conv): Change formatting of integers in case
 * 	  of %O, %H, %B to unsigned. (PR-4395)
 * 	* wrstring.c: change various procedure arguments according to the
 * 	  new type passed by the compiler. Remove include of fcntl.h and
 * 	  sys/stat.h (not needed).
 * 	* wrtext.c: change various procedure arguments according to the
 * 	  new type passed by the compiler.
 * 	* wrunixstd.c: change various procedure arguments according to the
 * 	  new type passed by the compiler. Remove include of fcntl.h and
 * 	  sys/stat.h (not needed).
 * 	* wrunixstd.c (_writeunixstd): Change the fwrite call to fprintf.
 * 	  This is necessary due to a port to a system which does not
 * 	  support fwrite.
 *
 * Revision 1.3  1993/11/15 23:05:46  bill
 * 	* fileio.c (writestring): Changed interface.
 * 	* iomodes.h: New structures for above interface.
 * 	* libchill.c: Delete bunch of powerset-related cruft.
 *           (cause_exception): Empty it out.  Just a stub.
 * 	* powerset.c: Call correct exception causer.
 * 	* string.c: Rewrite string comparisons from scratch.  They
 * 	  were bad...
 *
 * Revision 1.2  1993/10/30  21:43:23  bill
 * 	Copy latest chillrt stuff down here.
 *
 * Revision 1.1  1993/01/02  23:54:55  bothner
 * New directory, with files moved from ../gcc/chillrt.
 *
 * Revision 1.1  1992/12/01  17:51:22  fnf
 * Files moved from gcc/chill to gcc/chillrt.
 *
 * Revision 1.1  1992/11/28  21:43:52  fnf
 * New directory containing chill runtime parts.
 *
 */

#ifndef __modes_h__
#define __modes_h__

#include "rtltypes.h"

#define MAXPATHLEN	1024
#define MAXUSEDFILES	 100

struct __ACCESS_TYPE;

typedef struct
{
  struct __ACCESS_TYPE  *ref_access;
  unsigned short	bitlist;
  short 		fileindex;
  short 		checksum;
} __association_type;

typedef struct __ACCESS_TYPE
{
    __association_type	*ref_ass;
    unsigned short	bitlist;
    short		fileindex;
    short		checksum;
    long		next;
    char		s_loc[4];
} __access_type;

typedef struct
{
    __access_type	*ref_access;
    unsigned short	bitlist;
    short		fileindex;
    short		checksum;
    void		*text_ptr;
    long		actual_index;
    __access_type	text_access;
} __text_type;

typedef enum
{
    File,
    Directory,
    Stdin,
    Stdout,
    Stderr,
} __device;

typedef struct
{
    __association_type	*ref_association;
    __access_type	*ref_access;
    __text_type		*ref_text;
    unsigned short	bit_list_ass;
    unsigned short	bit_list_acc;
    long		base_index;
    long		curr_index;
    short		index_diff;
    short		low_index_bound;
    short		high_index_bound;
    short		rec_len_acc;
    int			actual_index;
    int			max_act_index;
    short		status_word;
    char		*file_name;
    unsigned char	device_script;
    int			file_attr;
    int			handle;
    long		abs_file_pos;
    long		file_length;
    char		*buffer_ptr;
    int			offset_in_buffer;
    int			end_of_buffer;
    unsigned short	bit_list_buffer;
    long		number_buffer_bytes;
    char		read_write_access;
    char		buffer_read;
} __fs_table_entry;

typedef enum
{
    LF_rec_CR,
    FF_rec_CR,
    rec_CR,
    LF_rec,
    FF,
    rec,
} __io_names;

typedef struct
{
    unsigned short	attributes;
    short		length_rec_mode;
    short		index_low;
    short		index_high;
} __acc_description;

typedef union
{
    short	len;
    struct
    {
	short	len;
	char	body[MAXPATHLEN];
    }		new_name;
} __modify_struct;

typedef enum
{
    First,
    Same,
    Last,
} __where;

typedef enum
{
    ReadOnly,
    WriteOnly,
    ReadWrite,
} __usage;

/* return type of __write_format */
typedef enum
{
    end_of_formatting,
    range_fail,
    too_many_repet_levels,
    format_spec_fail,
    qualifierfail,
    overflow_in_int,
    unexp_end_of_form_strg,
    dez_digit_expected,
    qual_not_allowed_here,
    real_clause_width_expected,
    source_txt_ind_out_of_range,
    source_txt_ind_less_than_zero,
    dest_txt_ind_out_of_range,
    dest_txt_ind_less_than_zero,
    conversion_fail,
    unexp_end_of_io_list,
    io_list_int_value_expected,
    repet_factor_zero_not_allowed,
    field_width_zero_not_allowed,
    io_list_too_long,
    io_list_last_byte_not_valid,
    unallowed_io_cntrl_char,
    io_cntrl_not_allowed,
    edit_fail,
    name_not_in_name_table,
    invalid_chill_name,
    set_value_out_of_range,
    pad_char_not_allowed,
    overfl_in_formatstrg,
    undefined_boolean,
    range_length_error,
    char_varying_too_long,
    fieldwidth_not_matching,
    carr_ret_char_not_allowed,
    form_strg_number_expected,
    field_or_padchar_not_matching,
    real_number_expected,
    real_format_not_matching,
    not_implemented,
} __format_result;

typedef enum
{
    Defconv,
    Binary,
    Octal,
    HexUpcase,
    HexLowcase,
    Real,
    LongReal,
} __howtoconvert;

typedef enum
{
    io_1,
    io_2,
    io_3,
    io_4,
    io_5,
} __io_type;

/* here starts the types for the actual implementation of WRITETEXT
   and READTEXT */
typedef enum
{
    _IO_UNUSED,
    _IO_Byte,
    _IO_UByte,
    _IO_Int,
    _IO_UInt,
    _IO_Long,
    _IO_ULong,
    _IO_Bool,
    _IO_Char,
    _IO_CharRange,
    _IO_CharStr,
    _IO_CharVarying,
    _IO_BitStr,
#if 0
    _IO_Powerset,
#endif
    _IO_IntRange,
    _IO_UIntRange,
    _IO_Set,
    _IO_SetRange,
    _IO_Real,
    _IO_LongReal,
    _IO_Pointer,
    _IO_Instance,
} __tmp_IO_enum;

typedef struct
{
    long	value;
    char	*name;
} __tmp_IO_enum_table_type;

typedef struct
{
    char   	*string;
    long	string_length;
} __tmp_IO_forcharstring;

typedef struct
{
    long			value;
    __tmp_IO_enum_table_type	*name_table;
} __tmp_WIO_forset;

typedef struct
{
    unsigned short	ptype;
    unsigned short	pcopy;
} __tmp_IO_instance;

#if 0
typedef struct
{
    void			*data;
    long			bitlength;
    long			lowbound;
    __tmp_IO_enum_table_type	*name_table;
    __tmp_IO_enum		type;
    long                        __dummy2;  /* align to multiple of 64bits */
} __tmp_IO_powerset;
#endif

typedef union
{
    __tmp_IO_forcharstring	__forcharstring;
    __tmp_WIO_forset		__forset;
#if 0
    __tmp_IO_powerset		__forpowerset;
#endif
    char			__forbyte;
    unsigned char		__forubyte;
    short			__forint;
    unsigned short		__foruint;
    long			__forlong;
    unsigned long		__forulong;
    char			__forbool;
    char			__forchar;
    float			__forreal;
    double			__forlongreal;
    void			*__forpointer;
    __tmp_IO_instance		__forinstance;
} __tmp_WIO_union;

typedef struct
{
    __tmp_IO_enum	__descr;
    long                __dummy1; /* force to 64bit alignment */
    __tmp_WIO_union	__t;
} __tmp_WIO_list;

/* here we go for READTEXT */
typedef struct
{
    char	*ptr;
    long	lowbound;
    long	highbound;
} __tmp_IO_charrange;

typedef struct
{
    void	*ptr;
    long     	length;
    long	lowbound;
    long	highbound;
} __tmp_IO_intrange;

typedef struct
{
    void		*ptr;
    long     		length;
    unsigned long	lowbound;
    unsigned long	highbound;
} __tmp_IO_uintrange;

typedef struct
{
    void			*ptr;
    long			length;
    __tmp_IO_enum_table_type	*name_table;
    long			lowbound;
    long			highbound;
} __tmp_IO_forsetrange;

typedef struct
{
    void			*ptr;
    long			length;
    __tmp_IO_enum_table_type	*name_table;
} __tmp_RIO_forset;

typedef union
{
    __tmp_IO_forcharstring	__forcharstring;
    __tmp_RIO_forset		__forset;
#if 0
    __tmp_IO_powerset		__forpowerset;
#endif
    __tmp_IO_charrange		__forcharrange;
    __tmp_IO_intrange		__forintrange;
    __tmp_IO_uintrange		__foruintrange;
    __tmp_IO_forsetrange	__forsetrange;
    char			*__forbyte;
    unsigned char		*__forubyte;
    short			*__forint;
    unsigned short		*__foruint;
    long			*__forlong;
    unsigned long		*__forulong;
    char			*__forbool;
    char			*__forchar;
    float			*__forreal;
    double			*__forlongreal;
    void			**__forpointer;
    __tmp_IO_instance		*__forinstance;
} __tmp_RIO_union;


typedef struct
{
    __tmp_IO_enum	__descr;
    __tmp_RIO_union	__t;
} __tmp_RIO_list;

#endif /* __modes_h__ */
