/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#if 0
#include <fcntl.h>
#include <sys/stat.h>
#endif
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

#define INSTR (instr+*dest_index)
#define INSTRPLUS(n) (*dest_index)+=n, (*act_max_index)+=n
    
typedef char *(*fetch_number) (char *string);
extern fetch_number __RTS_FETCH_NUMBERS__;

/* we need an own strtol cause the normal one does not detect overflow
   conditions */

static __format_result
mystrtol (char *instr, char **endp, int base,
	  int convlength, long *result)
{
    long	val = 0, prefval;
    int		sign = 0, parsed_chars = 0, loop;
    
    /* assume that we don't have a length of 0 */
    if (!convlength)
	return (conversion_fail);
    
    /* skip whitespace */
    while (isspace (*instr))
    {
	if (convlength != -1)
	{
	    convlength--;
	    if (!convlength)
		return (conversion_fail);
	}
	instr++;
    }
    
    /* check for sign */
    if (*instr == '-')
    {
	sign = 1;
	instr++;
    }
    else if (*instr == '+')
	instr++;
    
    loop = 1;
    while (loop)
    {
	char	c = *instr;
	
	prefval = val;
	switch (base)
	{
	case 2:
	    c -= '0';
	    if (c < 0 || c > 1)
	    {
		loop = 0;
		break;
	    }
	    val = (val * 2) + c;
	    break;
	case 8:
	    c -= '0';
	    if (c < 0 || c > 7)
	    {
		loop = 0;
		break;
	    }
	    val = (val * 8) + c;
	    break;
	case 10:
	    c -= '0';
	    if (c < 0 || c > 9)
	    {
		loop = 0;
		break;
	    }
	    val = (val * 10) + c;
	    break;
	case 16:
	    if (!isxdigit (c))
	    {
		loop = 0;
		break;
	    }
	    if (c >= 'a')
		c -= 0x20;
	    c -= '0';
	    if (c > 9)
		c -= 7;
	    val = (val * 16) + c;
	    break;
	default:
	    return (conversion_fail);
	}
	
	/* check against overflow */
	if (val >= 0 && prefval < 0)
	    return (conversion_fail);
	
	if (loop)
	{
	    instr++;
	    parsed_chars++;
	    if (convlength != -1)
	    {
		convlength--;
		if (!convlength)
		    loop =0;
	    }
	}
    }

    /* look for characters processed */
    if (!parsed_chars)
	return (conversion_fail);
    
    /* check for all converted */
    if (convlength != -1)
    {
	if (convlength != 0)
	    return (conversion_fail);
    }
    
    /* now return the values */
    if (endp)
	*endp = instr;
    if (sign)
	val = 0 - val;
    *result = val;
    return (end_of_formatting);
}

static __format_result
rf_getint (char *instr, int *dest_index, int *act_max_index,
	   int base, int whatint, __howtoconvert howtoconvert,
	   int min, int max, int left_justified,
	   int *field_width, long *longptr)
{
    __format_result	result;
    char		*nextchar;
    int			num_chars;
    
    result = mystrtol (INSTR, &nextchar, base,
		       left_justified ? -1 : *field_width,
		       longptr);
    if (result != end_of_formatting)
	return (result);

    /* check the range */
    if (*longptr < min || *longptr > max)
	return (range_fail);
    /* goto character after converted */
    num_chars = nextchar - (INSTR);
    INSTRPLUS (num_chars);
    if (*field_width != -1)
	*field_width -= num_chars;
    return (end_of_formatting);
} /* rf_getint */

static __format_result rf_getbool (char *instr, int *dest_index,
				   int *act_max_index, int *field_width,
				   long *result)
/* returncode:
 *  0 false
 *  1 true
 */
{
    if (*INSTR == '0' || *INSTR == '1' || *INSTR == 't' || *INSTR=='T' ||
	*INSTR == 'f' || *INSTR=='F')
    {
	if (*INSTR == '0' || *INSTR == 'f' || *INSTR == 'F')
	    *result = 0;
	else
	    *result = 1;
	INSTRPLUS (1);
	if (*field_width != -1)
	    (*field_width)--;
	if (*result && strlen (INSTR) > 2 && (*INSTR == 'r' || *INSTR == 'R') &&
	    (INSTR[1] == 'u' || INSTR[1] == 'U') &&
	    (INSTR[2] == 'e' || INSTR[2] == 'E'))
	{
	    INSTRPLUS (3);
	    if ((*field_width) != -1)
		if (*field_width > 2)
		    (*field_width) -= 3;
		else
		    return (conversion_fail);
	}
	if (!*result && strlen (INSTR) > 3 && (*INSTR == 'a' || *INSTR == 'A') && 
	    (INSTR[1] == 'l' || INSTR[1] == 'L') && 
	    (INSTR[2] == 's' || INSTR[2] == 'S') && 
	    (INSTR[3] == 'e' || INSTR[3] == 'E'))
	{
	    INSTRPLUS (4);
	    if (*field_width != -1)
		if (*field_width > 3)
		    (*field_width) -= 4;
		else
		    return (conversion_fail);
	}
    }
    else
	return (conversion_fail);
    return (end_of_formatting);  
} /* rf_getbool */

static __format_result
rf_getchar(char *instr,int *dest_index,int *act_max_index, long *result)
{
    int field_width = -1;
    __format_result errorcode = end_of_formatting;
    
    if (*INSTR == '\'')
    {
	INSTRPLUS (1);
	if (*INSTR == '\\')
	{
	    INSTRPLUS (1);
	    switch(*INSTR)
	    {
	    case 'b':
	    case 'B':
		*result = 8;
		break;
	    case 't':
	    case 'T':
		*result = 9;
		break;
	    case 'n':
	    case 'N':
		*result = 10;
		break;
	    case 'v':
	    case 'V':
		*result = 11;
		break;
	    case 'f':
	    case 'F':
		*result = 12;
		break;
	    case 'r':
	    case 'R':
		*result = 13;
		break;
	    default:
		return (-1);
	    }
	}
	else
	    if (*INSTR)
		*result = *INSTR;
	    else
		return (conversion_fail);
	INSTRPLUS (1);
	if (*INSTR != '\'')
	    return (conversion_fail);
	INSTRPLUS (1);
    }
    else
	errorcode = rf_getint (instr, dest_index, act_max_index,
			       10, _IO_UByte, Defconv,
			       0, 255, 0,
			       &field_width, result);
    return (errorcode);
} /* rf_getchar */

static __format_result
rf_parse_iolist (char *instr,
		 __tmp_RIO_list *iol,
		 int left_justified, int field_width,
		 char pad_char,
		 __howtoconvert howtoconvert,
		 int *dest_index, int *act_max_index)
{
    char ts[256];
    char c;
    char *nextchar;
    long min,max,result = 0;
    __format_result errorcode;
    int base;
    double double_value;
    __tmp_IO_instance *instance;
    void *ptr;
    long length;
    TVaryingCharType	*varyingptr;
    
    switch (iol->__descr) /* set ranges */
    {
    case _IO_Byte:
	min = CHAR_MIN;
	max = CHAR_MAX;
	break;
    case _IO_UByte:
	min = 0;
	max = UCHAR_MAX;
	break;
    case _IO_Int:
	min = SHRT_MIN;
	max = SHRT_MAX;
	break;
    case _IO_UInt:
    case _IO_Instance:
	min = 0;
	max = USHRT_MAX;
	break;
    case _IO_Long:
    case _IO_ULong:
	min = LONG_MIN;
	max = LONG_MAX;
	break;
    case _IO_CharRange:
	min = iol->__t.__forcharrange.lowbound;
	max = iol->__t.__forcharrange. highbound;
	break;
    case _IO_UIntRange:
	min = iol->__t.__foruintrange.lowbound;
	max = iol->__t.__foruintrange.highbound;
	break;
    case _IO_IntRange:
	min = iol->__t.__forintrange.lowbound;
	max = iol->__t.__forintrange.highbound;
	break;
#if 0
    case _IO_Powerset:
	min = iol->__t.__forpowerset.lowbound;
	max = min + iol->__t.__forpowerset.bitlength - 1;
	break;
#endif
    }
    switch (howtoconvert) /* set conversion for integer types */
    {
    case Binary:
	base = 2;
	break;
    case Octal:
	base = 8;
	break;
    case HexLowcase:
    case HexUpcase:
	base = 16;
	break;
    case Defconv:
	base = 10;
	break;
    }
    if (field_width != -1)
    {
	if (strlen (INSTR) < field_width)
	    return (conversion_fail);
	if (!left_justified)
	    while (instr[*dest_index] == pad_char) 
	    {
		INSTRPLUS (1);
		if (--field_width < 1)
		    return (conversion_fail);
	    }
    }
    switch (iol->__descr) /* parse instr */
    {
    case _IO_Byte:
    case _IO_UByte:
    case _IO_Int:
    case _IO_UInt:
    case _IO_Long:
    case _IO_ULong:
    case _IO_CharRange:
    case _IO_IntRange:
    case _IO_UIntRange:
    case _IO_Pointer:
	if ((errorcode = rf_getint (instr, dest_index, act_max_index,
				    base, iol->__descr, howtoconvert,
				    min, max, left_justified,
				    &field_width, &result)) != end_of_formatting)
	    return (errorcode);
	break;
    case _IO_Bool:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	if ((errorcode = rf_getbool (instr, dest_index, act_max_index,
				     &field_width, &result)) != end_of_formatting)
	    return (errorcode);
	break;
    case _IO_Char:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	result = *INSTR;
	INSTRPLUS (1);
	if (field_width != -1)
	    if (--field_width < 0)
		return (conversion_fail);
	break;
    case _IO_CharVarying:
      {
	unsigned short l;

	if (howtoconvert != Defconv)
	    return (conversion_fail);
	result = strlen (INSTR);
	if (result > iol->__t.__forcharstring.string_length)
	    result = iol->__t.__forcharstring.string_length;
	varyingptr = (TVaryingCharType *)iol->__t.__forcharstring.string;
	strncpy (varyingptr->body, INSTR, result);
	l = result;
	memcpy (&varyingptr->len, &l, sizeof (unsigned short));
	INSTRPLUS (result);
	break;
      }
    case _IO_CharStr:
    case _IO_BitStr:
	if (howtoconvert != Defconv)
	  return (conversion_fail);
	if (strlen (INSTR) < iol->__t.__forcharstring.string_length)
	  return (conversion_fail);
	if (iol->__descr == _IO_BitStr)
	  {
	    for (result = 0; result < iol->__t.__forcharstring.string_length; result++)
	      {
                extern void __setbitpowerset ();
                
		if (instr[*dest_index + result] != '0' && instr[*dest_index + result] != '1')
		    return (conversion_fail);
                __setbitpowerset (iol->__t.__forcharstring.string,
                                  iol->__t.__forcharstring.string_length,
                                  0,
                                  result,
                                  instr[*dest_index + result] - '0',
                                  NULL, 0);
#if 0
		else if (instr[*dest_index + result] == '1')
		    iol->__t.__forcharstring.string[result / 8 ] |= 1 << (7 - result % 8);
		else
		    iol->__t.__forcharstring.string[result / 8] &= ~(1 << (7 - result % 8));
#endif
	      }
	  }
        
	if (field_width != -1)
	{
	    field_width -= iol->__t.__forcharstring.string_length;
	    if (field_width < 0)
		return (conversion_fail);
	}
	if (iol->__descr == _IO_CharStr)
	    strncpy (iol->__t.__forcharstring.string, INSTR, iol->__t.__forcharstring.string_length);
	INSTRPLUS (iol->__t.__forcharstring.string_length);
	break;
    case _IO_Set:
    case _IO_SetRange:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	if (!iol->__t.__forset.name_table)
	{
	    result = strtol (INSTR, &nextchar, base);
	    if (INSTR == nextchar)
	       return (conversion_fail);
	}
	else
	{
	    int		nametab_len = 0, namelen;
	    __tmp_IO_enum_table_type	*nt = iol->__t.__forset.name_table;
	    
	    /* search for name in table an INSTR */
	    while (nt[nametab_len].name)
	    {
		namelen = strlen (nt[nametab_len].name);
		if (!strncmp (INSTR, nt[nametab_len].name, namelen))
		    break;
		nametab_len++;
	    }
	    if (!nt[nametab_len].name)
		return (name_not_in_name_table);
	    result = nt[nametab_len].value;
	    INSTRPLUS (namelen);
	}

	/* if we have a range, change agains it */
	if (iol->__descr == _IO_SetRange)
	{
	    /* FIXME: is it allowed to have a range of a noncontiguous enum */
	    if (result < iol->__t.__forsetrange.lowbound ||
		result > iol->__t.__forsetrange.highbound)
		return (range_fail);
	}
	
	break;
    case _IO_Real:
    case _IO_LongReal:
	if (howtoconvert != Real && howtoconvert != LongReal)
	    return (conversion_fail);
	double_value = strtod (INSTR, &nextchar);
	if (nextchar == INSTR)
	    return (conversion_fail);
	(*dest_index) += (nextchar - INSTR);
	break;
    case _IO_Instance:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	if (*INSTR != '[')
	    return (conversion_fail);
	INSTRPLUS (1);
	instance = iol->__t.__forinstance;
	if (strlen(INSTR) > 4 && (*INSTR == 'n' || *INSTR == 'N') && 
	    (INSTR[1] == 'u' || INSTR[1] == 'U') && 
	    (INSTR[2] == 'l' || INSTR[2] == 'L') && 
	    (INSTR[3] == 'l' || INSTR[3] == 'L') && INSTR[4] == ']')
	{
	    INSTRPLUS (5);
	    memset (instance, 0, sizeof (__tmp_IO_instance));
	}
	else
	{
	    unsigned short l;

	    if (isdigit (*INSTR) || *INSTR == '+')
	    {
		if (*INSTR == '+')
		    INSTRPLUS (1);
		if ((errorcode = rf_getint (instr, dest_index, act_max_index,
					    base, iol->__descr, howtoconvert,
					    min, max, 0,
					    &field_width, &result)) != end_of_formatting)
		    return (errorcode);
		if (*INSTR != ';')
		    return (conversion_fail);
		INSTRPLUS (1);
		l = result;
		memcpy (&instance->ptype, &l, sizeof (unsigned short));
	    }
	    else
	    {
		result = 0;
		while (isalnum (*INSTR) || *INSTR == '_')
		{
		    ts[result++] = *INSTR;
		    INSTRPLUS (1);
		}
		if (!result)
		    return (conversion_fail);
		if (*INSTR != ';')
		    return (conversion_fail);
		INSTRPLUS (1);
		ts[result] = 0;
		if (!__RTS_FETCH_NUMBERS__)
		    return (conversion_fail);
		else if ((result = (long)(*__RTS_FETCH_NUMBERS__)(ts)) == -1)
		    return (conversion_fail);
		l = result;
		memcpy (&instance->ptype, &l, sizeof (unsigned short));
	    }
	    if ((errorcode = rf_getint (instr, dest_index, act_max_index,
					base, iol->__descr, howtoconvert,
					min, max, 0,
					&field_width, &result)) != end_of_formatting)
		return (errorcode);
	    if (*INSTR != ']')
		return (conversion_fail);
	    INSTRPLUS (1);
	    l = result;
	    memcpy (&instance->pcopy, &l, sizeof (unsigned short));
	}
	break;
#if 0
    case _IO_Powerset:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	if (*INSTR != '[')
	    return (conversion_fail);
	INSTRPLUS (1);
	while (*INSTR != ']')
	{
	    switch (iol->__t.__forpowerset.type)
	    {
	    case _IO_Bool:
		if ((errorcode = rf_getbool (instr, dest_index, act_max_index,
					     &field_width, &result)) != end_of_formatting)
		    return (errorcode);
#if 0
		SETPOWERBIT (result);
#endif
		break;
	    case _IO_Byte:
	    case _IO_UByte:
	    case _IO_Int:
	    case _IO_UInt:
	    case _IO_Long:
	    case _IO_ULong:
	    case _IO_CharRange:
	    case _IO_IntRange:
	    case _IO_UIntRange:
		if ((errorcode = rf_getint (instr, dest_index, act_max_index,
					    base, iol->__descr, howtoconvert,
					    min, max, 0,
					    &field_width, &result)) != end_of_formatting)
		    return (errorcode);
#if 0
		SETPOWERBIT (result);
#endif
		break;
	    case _IO_Char:
		if ((errorcode = rf_getchar (instr, dest_index,
					     act_max_index, &result)) != end_of_formatting)
		    return (errorcode);
#if 0
		SETPOWERBIT (result);
#endif
		break;
	    case _IO_Set:
	    case _IO_SetRange:
#if 0
		if (!iol->t.powerset.name_table)
		{
		    result = strtol (INSTR, &nextchar, base);
		    if (INSTR == nextchar)
			return (conversion_fail);
		    if (result < 0 || result > USHRT_MAX) /* ?????? */
			return (range_fail);
		}
		else
		{
		    int		nametab_len = 0;
		    
		    result = 0;
		    while (isalnum (*INSTR) || (*INSTR) == '_')
		    {
			ts[result++] = *INSTR;
			INSTRPLUS (1);
		    }
		    if (!result)
			return (conversion_fail);
		    ts[result] = 0;
		    result = 0;
		    while (iol->t.powerset.name_table[nametab_len])
			nametab_len++;
		    while(result < nametab_len &&
			  strcmp (iol->t.powerset.name_table[result], ts))
			result++;
		    if (result >= nametab_len)
			return (name_not_in_name_table);
		}
		if ((iol->t.powerset.mode == SetRange) &&
		    (result < min || result > max))
		    return (range_fail);
		SETPOWERBIT (result);
#endif
		break;
	    }
	    if ((*INSTR != ']') && (*INSTR != ','))
		return (conversion_fail);
	    INSTRPLUS (1);
	}
	INSTRPLUS (1);
	break;
#endif
    }
    if (left_justified)
	while (*INSTR == pad_char)
	{
	    INSTRPLUS (1);
	    if (--field_width < 0)
		return (conversion_fail);
	}
    else if (field_width > 0)
	return (conversion_fail);
    switch (iol->__descr) /* assign values */
    {
    case _IO_Byte:
	*iol->__t.__forbyte = (char)result;
	break;
    case _IO_UByte:
	*iol->__t.__forubyte = (unsigned char)result;
	break;
    case _IO_Int:
      {
	short l = result;
	memcpy (iol->__t.__forint, &l, sizeof (short));
	break;
      }
    case _IO_UInt:
      {
	unsigned short l = result;
	memcpy (iol->__t.__foruint, &l, sizeof (unsigned short));
	break;
      }
    case _IO_Long:
	memcpy (iol->__t.__forlong, &result, sizeof (long));
	break;
    case _IO_ULong:
	memcpy (iol->__t.__forulong, &result, sizeof (unsigned long));
	break;
    case _IO_CharRange:
	*iol->__t.__forcharrange.ptr = (char)result;
	break;
    case _IO_IntRange:
	switch (iol->__t.__forintrange.length)
	{
	case 1:
	    *(char *)(iol->__t.__forintrange.ptr) = (char)result;
	    break;
	case 2:
	  {
	    short l = result;
	    memcpy (iol->__t.__forintrange.ptr, &l, sizeof (short));
	    break;
	  }
	case 4:
	    memcpy (iol->__t.__forintrange.ptr, &result, sizeof (long));
	    break;
	default:
	    return (range_fail);
	    break;
	}
	break;
    case _IO_UIntRange:
	switch (iol->__t.__forintrange.length)
	{
	case 1:
	    *(unsigned char *)(iol->__t.__forintrange.ptr) = (unsigned char)result;
	    break;
	case 2:
	  {
	    unsigned short l = result;
	    memcpy (iol->__t.__forintrange.ptr, &l, sizeof (unsigned short));
	    break;
	  }
	case 4:
	    memcpy (iol->__t.__forintrange.ptr, &result, sizeof (unsigned long));
	    break;
	default:
	    return (range_fail);
	    break;
	}
	break;
    case _IO_Bool:
	*iol->__t.__forbool = (char)result;
	break;
    case _IO_Char:
	*iol->__t.__forchar = (char)result;
	break;
    case _IO_Set:
    case _IO_SetRange:
	if (iol->__descr == _IO_Set)
	{
	    ptr = iol->__t.__forset.ptr;
	    length = iol->__t.__forset.length;
	}
	else
	{
	    ptr = iol->__t.__forsetrange.ptr;
	    length = iol->__t.__forsetrange.length;
	}
	switch (length)
	{
	case 1:
	    *(unsigned char *)ptr = (unsigned char)result;
	    break;
	case 2:
	  {
	    unsigned short l = result;
	    memcpy (ptr, &l, sizeof (unsigned short));
	    break;
	  }
	case 4:
	    memcpy (ptr, &result, sizeof (unsigned long));
	    break;
	default:
	    return (range_fail);
	    break;
	}
	break;
    case _IO_Real:
      {
	float l = double_value;
	memcpy (iol->__t.__forreal, &l, sizeof (float));
	break;
      }
    case _IO_LongReal:
	memcpy (iol->__t.__forlongreal, &double_value, sizeof (double));
	break;
    }
    return (end_of_formatting);
} /* rf_parser_iolist */

/*
 * function __read_format
 *
 * parameters:
 *  formatstr		format string
 *  formatstrlen	length of format string
 *  iolist		io-list
 *  iolistlen		length of io-list (maximum index)
 *  iol_index		working index for io-list (for recursion)
 *  instr		input string
 *  instrlen		input string length
 *  dest_index		destination index
 *  act_max_index	actual maximum index
 *  io_cntl_allowed	allow io-control
 *  h_text_ptr \	  arguments for
 *  h_index    /	  input function
 *  in_function		input function
 *
 * returns:
 *  __format_result
 *
 * exceptions:
 *  none
 *
 * abstract:
 *  do formatting for _readtext and _readstring
 *
 */

__format_result
__read_format(char *formatstr,int formatstrlen,
	      __tmp_RIO_list *iolist,int iolistlen,int *iol_index,
	      char *instr,int instrlen,
	      int *dest_index,int *act_max_index,
	      int io_cntl_allowed,
	      int h_text_ptr,int h_index,outfun in_function)
{
    char *fstr = alloca (formatstrlen + 1);
    char *tmp,*parstr;
    int rep_count;
    int left_justified;
    int field_width,real_width;
    int result;
    int brackets;
    __tmp_RIO_list *iol;
    __howtoconvert howtoconvert;
    char pad_char;
    
    strncpy (fstr, formatstr, formatstrlen);
    fstr[formatstrlen] = 0;
    while (*fstr)
    {
	if (*fstr == '%')
	{
	    fstr++;
	    switch (*fstr)
	    {
	    case '/':
	    case '-':
	    case '+':
	    case '?':
	    case '!':
	    case '=': /* i/o control */
		if (++(*iol_index) == iolistlen)
		    return (unexp_end_of_io_list);
		break;
	    case 'x':
	    case 'X':
	    case '<':
	    case '>':
	    case 't':
	    case 'T': /* editing */
		if (*fstr == 't' || *fstr == 'T')
		    field_width=0; 
		else
		    field_width = 1;
		if (isdigit (*fstr)) /* clause width */
		{
		    field_width = *fstr++ - '0';
		    while (*fstr && isdigit (*fstr))
			field_width=(10*field_width) + (*fstr++ - '0');
		}
		if (*fstr == '<')
		{
		    if (*dest_index - field_width < 0)
			return (dest_txt_ind_less_than_zero);
		    (*dest_index) -= field_width;
		}
		else if (*fstr == '>')
		{
		    if (*dest_index+field_width >= strlen (INSTR))
			return (dest_txt_ind_out_of_range);
		    (*dest_index) += field_width;
		}
		else if (*fstr == 't' || *fstr == 'T')
		{
		    if (field_width < 0)
			return (dest_txt_ind_less_than_zero);
		    if (field_width >= strlen (instr))
			return (dest_txt_ind_out_of_range);
		    (*dest_index) = field_width;
		}
		else
		{
		    if (field_width < 0)
			return (format_spec_fail);
		    if (strlen (INSTR) < field_width)
			return (dest_txt_ind_out_of_range);
		    while (field_width--)
			if (isspace (*INSTR))
			{
			    INSTRPLUS (1);
			}
			else
			    return (conversion_fail);
		}
		break;
	    default: /* conversion */
		rep_count = 1;
		if (isdigit (*fstr)) /* repetition factor */
		{
		    rep_count = *fstr++ - '0';
		    while (*fstr && isdigit (*fstr))
			rep_count = (rep_count * 10) + (*fstr++ - '0');
		    if (!*fstr)
			return (unexp_end_of_form_strg);
		    if (!rep_count)
			return (repet_factor_zero_not_allowed);
		}
		if (*fstr == '(') /* paranthesed expression */
		{
		    fstr++;
		    tmp = fstr;
		    brackets = 0;
		    while (tmp != NULL)
		    {
			if (!strncmp (tmp, "%(", 2))
			    brackets++;
			else if (!strncmp (tmp, "%)", 2))
			    if (brackets)
				brackets--;
			    else
				break;
			tmp++;
		    }
		    if (tmp == NULL)
			return (format_spec_fail); /* none found */
		    parstr = alloca ((int)(tmp - fstr + 1));
		    strncpy (parstr, fstr, (int)(tmp - fstr)); /* extract new string */
		    parstr[tmp-fstr] = 0;
		    while (rep_count)
		    {
			result = __read_format (parstr, strlen (parstr),
						iolist, iolistlen, iol_index,
						instr, strlen (instr),
						dest_index, act_max_index,
						io_cntl_allowed,
						h_text_ptr, h_index, in_function);
			if (result != end_of_formatting)
			    return (result);
			rep_count--;
		    }
		    fstr= tmp + 2; /* skip over %) */
		    if (*fstr == '%' && *(fstr+1) == '.')
			fstr += 2; /* optional termination */
		}
		else
		{
		    char	*tmp = fstr;
		    
		    while(rep_count)
		    {
			fstr = tmp;
			if (*fstr == 'b' || *fstr == 'B')
			    howtoconvert = Binary;
			else if (*fstr == 'h')
			    howtoconvert = HexLowcase;
			else if (*fstr == 'H')
			    howtoconvert=HexUpcase;
			else if (*fstr == 'o' || *fstr == 'O')
			    howtoconvert = Octal;
			else if (*fstr == 'c' || *fstr == 'C')
			    howtoconvert = Defconv;
			else if (*fstr == 'r' || *fstr == 'R')
			    howtoconvert = Real;
			else if (*fstr == 'f' || *fstr == 'F')
			    howtoconvert = LongReal;
			else
			    return (format_spec_fail);
			pad_char = ' ';
			left_justified = 0;
			field_width = -1;
			fstr++;
			if (*iol_index >= iolistlen)
			    return (unexp_end_of_io_list);
			iol = &iolist[(*iol_index)++];
			while (1) /* conversion qualifiers */
			{
			    if (*fstr == 'l' || *fstr == 'L')
			    {
				left_justified = 1;
				fstr++;
			    }
			    else if (*fstr == 'e' || *fstr == 'E')
				fstr++;
			    else if (*fstr == 'p' || *fstr == 'P')
			    {
				fstr++;
				if (!*fstr)
				    return (unexp_end_of_form_strg);
				pad_char = *fstr++;
			    }
			    else break;
			}
			if (isdigit (*fstr)) /* clause width */
			{
			    field_width = *fstr++ - '0';
			    while (*fstr && isdigit (*fstr))
				field_width = (10 * field_width) + (*fstr++ - '0');
			    if (!field_width)
				return (field_width_zero_not_allowed);
			    if (howtoconvert == Real || howtoconvert == LongReal)
				if (*fstr == ':')
				{
				    fstr++;
				    if (isdigit (*fstr))
				    {
					real_width = *fstr++ - '0';
					while(*fstr && isdigit (*fstr))
					    real_width = (10 * real_width) +
						(*fstr++ - '0');
				    }
				    else
					return (real_clause_width_expected);
				}
				else real_width = 0;
			}
			else if (*fstr == 'v' || *fstr == 'V')
			{
			    /* unimplemented */
			    return (not_implemented);
			}
			if (!*(INSTR))
			    return (unexp_end_of_io_list);
			result = rf_parse_iolist (instr,
						  iol,
						  left_justified, field_width, pad_char,
						  howtoconvert,
						  dest_index, act_max_index);
			if (result != end_of_formatting)
			    return (result);
			rep_count--;
		    }
		    if (*fstr == '%' && *(fstr + 1) == '.')
			fstr += 2; /* optional termination */
		}
		break;
	    }
	}
	else
	{
	    if (!*INSTR)
		return (conversion_fail);
	    if (*fstr != *INSTR)
		return (conversion_fail);
	    fstr++;
	    INSTRPLUS (1);
	} 
    }
    return (end_of_formatting);
} /* __read_format */
