/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (notconnected);
EXCEPTION (protectionfail);
EXCEPTION (rangefail);
EXCEPTION (readfail);
EXCEPTION (spacefail);
EXCEPTION (writefail);

/*
 * function _readrecord
 *
 * parameters:
 *  acc			pointer to access location
 *  idx			index, if specified
 *  store_loc_ptr	address of store location
 *
 * returns:
 *  void*		address of store location
 *
 * exceptions:
 *  notconnected
 *  protectionfail
 *  rangefail
 *  readfail
 *  spacefail
 *  writefail
 *
 * abstract:
 *  read one record from specified access
 *
*/

void *
_readrecord (__access *acc, int idx, void *store_loc_ptr)
{
    __fs_table_entry	*fse = &__fs_table[acc->fileindex];
    int			read_bytes;
    
    if (!__ISCONNECTED (acc))
	CAUSE (notconnected);

    if (!__ACCREAD (fse->bit_list_acc))
	CAUSE (readfail);

    /* read_bytes is passed to all procedures to indicate the maximum number
       of bytes to read */
    if (!__DIRECTBITSET (fse->bit_list_ass) &&
	__STREAMBITSET (fse->bit_list_ass))
	read_bytes = fse->rec_len_acc;
    else
    {
	if (!store_loc_ptr || (*(long *)store_loc_ptr > fse->rec_len_acc))
	    CAUSE (readfail);
	else
	    read_bytes = *(long *)store_loc_ptr;
    }
    
    CAUSE (readfail);
} /* _readrecord */
