/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#if 0
#include <fcntl.h>
#include <sys/stat.h>
#endif
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (spacefail);
EXCEPTION (textfail);

/*
 * function _readstring
 *
 * parameter:
 *  string			pointer to string to scan
 *  string_length		length of scan-string
 *  format_string		format string
 *  format_string_length	length of format string
 *  io_list			pointer to io-list
 *  io_list_length		length of io-list in bytes
 *
 * returns:
 *  void
 *
 * exceptions:
 *  spacefail
 *  textfail
 *
 * abstract:
 *  perform same function as `sscanf' in C for CHILL
 *
 */

void
_readstring (char *string, int string_length,
	     char *format_string, int format_string_length,
	     __tmp_RIO_list *io_list, int io_list_len,
	     char *filename, int lineno)
{
    __format_result	res;
    int			dest_index = 0, act_max_index = 0, iol_index = 0;
    char		*instr;
    
    _allocate_memory (&instr, string_length + 1, filename, lineno);
    strncpy (instr, string, string_length);
    instr[string_length] = 0;
    res = __read_format (format_string, format_string_length,
			 io_list, io_list_len / sizeof (__tmp_RIO_list), &iol_index,
			 instr, string_length,
			 &dest_index, &act_max_index,
			 0, 0, 0, 0);

    _return_memory (instr, filename, lineno);
    if (res != end_of_formatting)
	__cause_exception ("textfail", filename, lineno);
} /* _readstring */
