/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (spacefail);
EXCEPTION (textfail);

static __format_result
readtext_io (__text *txt, int idx, int *new_length)
{
    __fs_table_entry	*fse = &__fs_table[txt->fileindex];

#if 1
    abort();
#else
    TRY ((6, EXNAME(protectionfail), EXNAME(notconnected), EXNAME(spacefail),
	  EXNAME(rangefail), EXNAME(readfail), EXNAME(writefail)))
	_readrecord (fse->ref_access, idx, NULL);
    ON (protectionfail)
	fse->index_diff = 1;
    ON (notconnected)
	fse->index_diff = 2;
    ON (spacefail)
	fse->index_diff = 3;
    ON (rangefail)
	fse->index_diff = 4;
    ON (readfail)
	fse->index_diff = 5;
    ON (writefail)
	fse->index_diff = 6;
    ENDON;
#endif

    *new_length = fse->ref_access->next;
    return (end_of_formatting);
} /* readtext_io */

/*
 * function _readtext
 *
 * parameter:
 *  txt				pointer to text-location
 *  idx				index, if specified
 *  format_string		pointer to format string
 *  form_string_length		length of format string
 *  io_list			pointer to io-list
 *  io_list_length		length of io-list in bytes
 *
 * returns:
 *  void
 *
 * exceptions:
 *  spacefail
 *  textfail
 *
 * abstract:
 *  perform same function as `(f)scanf' in C for CHILL
 *
*/

void
_readtext (__text *txt, int idx,
	   char *format_string, int format_string_length,
	   __io_list *io_list, int io_list_len)
{
    __fs_table_entry	*fse = &__fs_table[txt->fileindex];
    __format_result	res;
    int			iol_index = 0;
    
    if (!__ISCONNECTED (txt))
	CAUSE (notconnected);

    if (!__ACCREAD (fse->bit_list_acc))
	CAUSE (readfail);

    fse->index_diff = 0;
    res = __read_format (format_string, format_string_length,
			 io_list, io_list_len / sizeof (__io_list), &iol_index,
			 txt->text_ptr + 4, *(long *)(txt->text_ptr),
			 &fse->actual_index, &fse->max_act_index,
			 1, txt, idx, readtext_io);
    txt->actual_index = fse->actual_index;
    if (res != end_of_formatting)
	CAUSE (textfail);
    switch (fse->index_diff)
    {
    case 0:
	break;
    case 1:
	CAUSE (protectionfail);
	break;
    case 2:
	CAUSE (notconnected);
	break;
    case 3:
	CAUSE (spacefail);
	break;
    case 4:
	CAUSE (rangefail);
	break;
    case 5:
	CAUSE (readfail);
	break;
    case 6:
	CAUSE (writefail);
	break;
    default:
	fprintf (stderr, "_readtext: index_diff out of range (%d)\n", fse->index_diff);
	abort ();
    }
} /* _readtext */
