/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (notassociated);

/*
 * function _sequencible
 *
 * parameters:
 *  ass		pointer to association location
 *
 * returns:
 *  char	1 ... true, 0 ... false
 *
 * exceptions:
 *  notassociated	ass is not associated
 *
 * abstract:
 *  This routine delivers, if a, with an ASSOCIATION location connected
 *  'outside world object', shall be used for sequentiell access only.
 *  Though the (logically) exact attribute is set in a CONNECT operation,
 *  it has to be checked, if the ASSOCIATION is connected to an ACCESS
 *  location, because the default values of the bit's are already set in
 *  the ASSOCIATION operation.
 *
*/

char
_sequencible (__association *ass)
{
    if (__ISASSOCIATED (ass))
	return ((!__DIRECTBITSET (ass->bitlist) &&
		 !__STREAMBITSET (ass->bitlist)) ? 1 : 0);
    else
	CAUSE (notassociated);
} /* _sequencible */
