/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (empty);
EXCEPTION (fileiofail);
EXCEPTION (protectionfail);

#ifndef VMS
#include <pwd.h>
#endif

/*
 * function newstring
 *
*/

static char *
newstring (char *str)
{
    char	*tmp = __xmalloc_ (strlen (str) + 1);
    
    strcpy (tmp, str);
    return (tmp);
} /* newstring */

#ifndef VMS
/*
 * function tilde_expand
 *
*/

static char *
tilde_expand (char *filename)
{
    char	*dirname = filename ? newstring (filename) : NULL;
    
    if (dirname && *dirname == '~')
    {
	char	*temp_name;
	
	if (!dirname[1] || dirname[1] == '/')
	{
	    /* prepend $HOME to the rest of the string */
	    char	*temp_home = getenv ("HOME");
	    
	    temp_name = alloca (1 + strlen (&dirname[1]) +
				(temp_home ? strlen (temp_home) : 0));
	    temp_name[0] = '\0';
	    if (temp_home)
		strcpy (temp_name, temp_home);
	    strcat (temp_name, &dirname[1]);
	    free (dirname);
	    dirname = newstring (temp_name);
	}
	else
	{
	    struct passwd	*user_entry;
	    char		*username = alloca(257);
	    int			i, c;
	    
	    for (i = 1; c = dirname[i]; i++)
	    {
		if (c == '/')
		    break;
		else
		    username[i - 1] = c;
	    }
	    username[i - 1] = '\0';

	    if (!(user_entry = getpwnam (username)))
	    {
		free (dirname);
		return (NULL);
	    }
	    
	    temp_name = alloca (1 + strlen (user_entry->pw_dir) +
				strlen (&dirname[i]));
	    strcpy (temp_name, user_entry->pw_dir);
	    strcat (temp_name, &dirname[i]);
	    free (dirname);
	    dirname = newstring (temp_name);
	}
    }
    return (dirname);
} /* tilde_expand */
#endif

/*
 * function _set_directory
 *
 * parameters:
 *  newdir		new directory
 *
 * returns:
 *  void
 *
 * exceptions:
 *  empty
 *  fileiofail
 *  protectionfail
 *
 * abstract:
 *  change current working directory
 *
*/

void
_set_directory (TVaryingCharType *newdir)
{
    char	*filename = alloca (newdir->len + 1);
    char	*dirname;
    int		chdir_result;
    
    if (!newdir->len)
	CAUSE (empty);
    strncpy (filename, &newdir->body[0], newdir->len);
    filename[newdir->len] = '\0';

#ifndef VMS
    dirname = tilde_expand (filename);
#else
    dirname = newstring (filename);
#endif
    if (!dirname)
	/* can't expand username -- protectionfail */
	CAUSE (protectionfail);
    
    chdir_result = chdir (dirname);
    free (dirname);
    if (chdir_result == -1)
	CAUSE (fileiofail);
} /* _set_directory */
