/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (isconnected);
EXCEPTION (notconnected);

/*
 * function _settextaccess
 *
 * parameters:
 *  txt		pointer to text location
 *  acc		pointer to new access location
 *
 * returns:
 *  void
 *
 * exceptions:
 *  isconnected
 *  notconnected
 *
 * abstract:
 *  change the access location of the text location. The text-location
 *  must be connected, the access location must not be connected.
 *
*/

void
_settextaccess (__text *txt, __access *acc)
{
    __fs_table_entry	*fse = &__fs_table[txt->fileindex];
    
    if (!__ISCONNECTED (txt))
	CAUSE (notconnected);
    
    if (__ISCONNECTED (acc))
	CAUSE (isconnected);

    txt->ref_access->ref_ass = NULL;
    txt->ref_access = acc;
    txt->text_ptr = &acc->next;
    
    acc->ref_ass = fse->ref_association;
    acc->bitlist = fse->ref_access->bitlist;
    acc->fileindex = txt->fileindex;
    fse->ref_association->ref_access = acc;
    fse->ref_access = acc;
    __check_sum (fse->ref_association);
    __check_sum (acc);
    __check_sum (txt);
} /* _settextaccess */
