/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (notconnected);
EXCEPTION (textfail);

/*
 * function _settextindex
 *
 * parameters:
 *  txt		pointer to text-location
 *  new_index	new actual index of text-location
 *
 * returns:
 *  void
 *
 * exceptions:
 *  notconnected
 *  textfail
 *
 * abstract:
 *  change actual index of text-location to new_index
 *
*/

void
_settextindex (__text *txt, long new_index)
{
    if (!__ISCONNECTED (txt))
	CAUSE (notconnected);
    
    if ((new_index < 0) || (new_index > *(long *)(txt->text_ptr)) ||
	(!__WRITTEN (txt->text_access.ref_ass->bitlist) &&
	 (txt->actual_index < new_index)))
	/* new_index not in range 0:(actual)max_length of string
	   OR last_operation == read AND actual index in text loc < new_index */
	CAUSE (textfail);

    txt->actual_index = __fs_table[txt->fileindex].actual_index = new_index;
} /* _settextindex */
