/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#if 0
#include <fcntl.h>
#include <sys/stat.h>
#endif
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

typedef char *(*fetch_names) (int number);
extern fetch_names	__RTS_FETCH_NAMES__;

#define MAX_WORK_LEN	1024

/*
 * function copy_text
 *
*/

static int
copy_text (char *src, char *dest, int destlen,
	   int *dest_index, int *act_max_index, int nr_char)
{
    int		to_copy;
    
    if (((*dest_index) + nr_char) >= destlen)
	to_copy = destlen - *dest_index;
    else
	to_copy = nr_char;
    strncpy (dest + *dest_index, src, to_copy);
    (*dest_index) += to_copy;
    if (*dest_index > *act_max_index)
	*act_max_index = *dest_index;
    return ((nr_char == to_copy) ? nr_char : -1);
} /* copy_text */

/*
 * function get_field_width
 *
*/

static __format_result
get_field_width (__tmp_WIO_list *iol, int *fw, int zero_allowed)
{
    switch (iol->__descr)
    {
    case _IO_Byte:
	*fw = iol->__t.__forbyte;
	break;
    case _IO_UByte:
	*fw = iol->__t.__forubyte;
	break;
    case _IO_Int:
	*fw = iol->__t.__forint;
	break;
    case _IO_UInt:
	*fw = iol->__t.__foruint;
	break;
    case _IO_Long:
	*fw = iol->__t.__forlong;
	break;
    case _IO_ULong:
	*fw = iol->__t.__forulong;
	break;
#if 0
    case _IO_IntRange:
	*fw = iol->__t.__forintrange.value;
	if ((*fw < iol->__t.__forintrange.lower_bound) ||
	    (*fw > iol->__t.__forintrange.upper_bound))
	    return (range_fail);
	break;
    case UIntRange:
	switch (iol->t.intrange.range_length)
	{
	case 1:
	    *fw = *(unsigned char *)(iol->t.intrange.ptr);
	    break;
	case 2:
	    *fw = *(unsigned short *)(iol->t.intrange.ptr);
	    break;
	case 4:
	    *fw = *(unsigned int *)(iol->t.intrange.ptr);
	    break;
	default:
	    return (range_length_error);
	    break;
	}
	/* do the range check */
	if (((unsigned)(*fw) < (unsigned)iol->t.intrange.lowbound) ||
	    ((unsigned)(*fw) > (unsigned)iol->t.intrange.upbound))
	    return (range_fail);
	break;
#endif
    default:
	return (io_list_int_value_expected);
    }
    if (!zero_allowed)
    {
	if (*fw <= 0)
	    return (field_width_zero_not_allowed);
    }
    else if (zero_allowed == 1)
    {
	if (*fw < 0)
	    return (field_width_zero_not_allowed);
    }
    return (end_of_formatting);
} /* get_field_width */

/*
 * function skip_upwards
 *
*/

static __format_result
skip_upwards (char *outstr, int outstrlen, int *dest_index,
	      int *act_max_index, int field_width)
{
    if ((*dest_index + field_width) <= *act_max_index)
    {
	if (field_width >= -(*dest_index))
	    (*dest_index) += field_width;
	else
	    return (dest_txt_ind_out_of_range);
    }
    else
    {
	(*dest_index) += field_width;
	if (*dest_index < outstrlen)
	{
	    char	*p = outstr + *act_max_index;
	    
	    while (*dest_index != *act_max_index)
	    {
		*p++ = ' ';
		(*act_max_index)++;
	    }
	}
	else
	{
	    *dest_index = outstrlen;
	    return (dest_txt_ind_out_of_range);
	}
    }
    return (end_of_formatting);
} /* skip_upwards */

/*
 * function default_conv
 *
 */
static __format_result
default_conv (char *out,
	      __tmp_WIO_list *iol,
	      int left_justified, 
	      int mark_overflow,
	      char pad_char, 
	      int field_width, 
	      int exp_width,
	      __howtoconvert howtoconvert)
{
    int			intval, reslen, i, j, vallen, nametable_length;
    char		*p = out, *f, *psstr;
    char		*tmp;
    char		*fmtstring;
    __tmp_IO_instance	*instptr;
    TVaryingCharType	*varyingptr;
    int			intval_mask = 0xffffffff;
    static char		fmtbuf[250], realfmtstring[80];
    
    switch (iol->__descr)
    {
    case _IO_Pointer:
	intval = (int)iol->__t.__forpointer;
	if (intval == 0)
	{
	    tmp = "NULL";
	    reslen = strlen (tmp);
	    break;
	}
	vallen = 31;
	fmtstring = "H'%X";
	goto do_int;
#if 0
    case _IO_IntRange:
	intval = iol->__t.__forintrange.value;
	switch (iol->__t.__forintrange.range_length)
	{
	case 1:
	    vallen = 7;
	    break;
	case 2:
	    vallen = 15;
	    break;
	case 4:
	    vallen = 31;
	    break;
	default:
	    return (range_length_error);
	    break;
	}
	/* do the range check */
	if ((intval < iol->__t.__forintrange.lower_bound) ||
	    (intval > iol->__t.__forintrange.upper_bound))
	    return (range_fail);
	fmtstring = "%d";
	goto do_int;
    case UIntRange:
	switch (iol->t.intrange.range_length)
	{
	case 1:
	    intval = *(unsigned char *)(iol->t.intrange.ptr);
	    vallen = 7;
	    break;
	case 2:
	    intval = *(unsigned short *)(iol->t.intrange.ptr);
	    vallen = 15;
	    break;
	case 4:
	    intval = *(unsigned int *)(iol->t.intrange.ptr);
	    vallen = 31;
	    break;
	default:
	    return (range_length_error);
	    break;
	}
	/* do the range check */
	if (((unsigned)intval < (unsigned)iol->t.intrange.lowbound) ||
	    ((unsigned)intval > (unsigned)iol->t.intrange.upbound))
	    return (range_fail);
	fmtstring = "%u";
	goto do_int;
#endif
    case _IO_Byte:
	fmtstring = "%d";
	vallen = 7;
	intval = iol->__t.__forbyte;
	intval_mask = 0xff;
	goto do_int;
    case _IO_UByte:
	fmtstring = "%u";
	vallen = 7;
	intval = iol->__t.__forubyte;
	intval &= 0xff;
	goto do_int;
    case _IO_Int:
	fmtstring = "%d";
	vallen = 15;
	intval = iol->__t.__forint;
	intval_mask = 0xffff;
	goto do_int;
    case _IO_UInt:
	fmtstring = "%u";
	vallen = 15;
	intval = iol->__t.__foruint;
	intval &= 0xffff;
	goto do_int;
    case _IO_Long:
	fmtstring = "%d";
	vallen = 31;
	intval = iol->__t.__forlong;
	goto do_int;
    case _IO_ULong:
	fmtstring = "%u";
	vallen = 31;
	intval = iol->__t.__forulong;
#if 0
	goto do_int;
    case Value:
	intval = iol->t.val;
	vallen = 31;
	fmtstring = "%d";
#endif
do_int: ;
	switch (howtoconvert)
	{
	case Octal:
	    fmtstring = "%o";
	    intval &= intval_mask;
	    break;
	case HexUpcase:
	    fmtstring = "%X";
	    intval &= intval_mask;
	    break;
	case HexLowcase:
	    fmtstring = "%x";
	    intval &= intval_mask;
	    break;
	case Binary:
	    intval &= intval_mask;
	    tmp = alloca (vallen + 1);
	    f = tmp;
	    for (i = vallen; i >= 0; i--)
	    {
		if (intval & (1 << i))
		    *f++ = '1';
		else
		    *f++ = '0';
	    }
	    *f = '\0';
	    while ((tmp[0] == '0') && (strlen (tmp) != 1))
		strcpy (tmp, tmp + 1);
	    fmtstring = "%s";
	    intval = (long)(&tmp[0]);
	    break;
	case Defconv:
	    break;
	default:
	    return (not_implemented);
	    break;
	}
	sprintf (fmtbuf, fmtstring, intval);
	tmp = fmtbuf;
	reslen = strlen(tmp);
	break;
    case _IO_Char:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
#if 0
	intval = iol->__t.__forchar;
	intval &= 0xff;
	if (intval >= ' ' && intval <= '~')
	{
	    fmtbuf[0] = intval;
	    fmtbuf[1] = '\0';
	}
	else if (intval < ' ')
	{
	    fmtbuf[0] = '^';
	    fmtbuf[1] = intval | 0x40;
	    fmtbuf[2] = '\0';
	}
	else
	    sprintf (fmtbuf, "C'%02X'", intval);

	tmp = fmtbuf;
	reslen = strlen (tmp);
#else
	/* FIXME: c'00' looses here */
	fmtbuf[0] = iol->__t.__forchar;
	fmtbuf[1] = 0;;
	reslen = 1;
	tmp = fmtbuf;
#endif
	break;
#if 0
    case _IO_CharRange:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	if (iol->__t.__forintrange.range_length != 1)
	    return (range_length_error);
	intval = iol->__t.__forintrange.value;
	if ((intval < iol->__t.__forintrange.lower_bound) ||
	    (intval > iol->__t.__forintrange.upper_bound))
	    return (range_fail);
	intval &= 0xff;
	if (intval >= ' ' && intval <= '~')
	{
	    fmtbuf[0] = intval;
	    fmtbuf[1] = '\0';
	}
	else
	    sprintf (fmtbuf, "C'%02X'", intval);
	tmp = fmtbuf;
	reslen = strlen (tmp);
	break;
#endif
    case _IO_CharVarying:
    {
	TVaryingCharType	*vstr = (TVaryingCharType *)iol->__t.__forcharstring.string;
	union {
	  unsigned short        l;
	  unsigned char		c[sizeof(unsigned short)];
	} u;

	if (howtoconvert != Defconv)
	    return (conversion_fail);
	/* Note that vstr->len may be unaligned if compiled with -fpack.
	   Using memcpy does not help, because gcc optimizes to do
	   a short mode.  */
	for (i = sizeof (unsigned short);  --i >= 0; )
	  u.c[i] = ((unsigned char*)vstr)[i];
	reslen = u.l;
	reslen &= 0xffff;
	tmp = alloca (reslen + 1);
	memcpy (tmp, &vstr->body[0], reslen);
	tmp[reslen] = '\0';
    }
	break;
    case _IO_CharStr:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	tmp = alloca (iol->__t.__forcharstring.string_length + 1);
	memcpy (tmp, iol->__t.__forcharstring.string, iol->__t.__forcharstring.string_length);
	tmp[reslen = iol->__t.__forcharstring.string_length] = '\0';
	break;
    case _IO_Bool:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	if (iol->__t.__forbool == 0)
	    strcpy (fmtbuf, "FALSE");
	else if (iol->__t.__forbool == 1)
	    strcpy (fmtbuf, "TRUE");
	else
	    return (range_fail);
	tmp = fmtbuf;
	reslen = strlen (tmp);
	break;
    case _IO_Set:
#if 0
    case _IO_SetRange:
#endif
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	intval = iol->__t.__forset.value;
#if 0
	if (iol->__descr == _IO_SetRange)
	{
	    /* do the range check */
	    if ((intval < iol->__t.__forsetrange.lower_bound) ||
		(intval > iol->__t.__forsetrange.upper_bound))
		return (range_fail);
	}
	
	/* get length of nametable */
	nametable_length = 0;
	if (iol->__t.__forset.name_table)
	{
	    while (iol->__t.__forset.name_table[nametable_length].name)
		nametable_length++;
	    if (intval >= nametable_length)
		return (range_fail);
	}
	
	if (!iol->__t.__forset.name_table)
	{
	    /* we have no nametable -- print unsigned decimal */
	    vallen = 7;
	    fmtstring = "%u";
	    goto do_int;
	}
	tmp = iol->__t.__forset.name_table[intval];
#endif
	tmp = 0;
	nametable_length = 0;
	if (iol->__t.__forset.name_table)
	{
	    while (iol->__t.__forset.name_table[nametable_length].name)
	    {
		if (iol->__t.__forset.name_table[nametable_length].value == intval)
		{
		    tmp = iol->__t.__forset.name_table[nametable_length].name;
		    break;
		}
		nametable_length++;
	    }
	}
	if (!tmp)
	{
	    /* value out of range or no name_table available */
	    vallen = 7;
	    fmtstring = "%u";
	    goto do_int;
	}
	reslen = strlen (tmp);
	break;
    case _IO_BitStr:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	tmp = alloca (iol->__t.__forcharstring.string_length + 4);
	f = iol->__t.__forcharstring.string;
	strcpy (tmp, "B'");
	for (i = 0; i < iol->__t.__forcharstring.string_length; i++)
	{
	    extern 	int __inpowerset();
	    
	    int		isset;
	    
	    isset = __inpowerset (i, f, iol->__t.__forcharstring.string_length, 0);
	    tmp[i + 2] = isset ? '1' : '0';
	}
	tmp[i + 2] = '\'';
	tmp[i + 3] = '\0';
	reslen = iol->__t.__forcharstring.string_length + 3;
	break;
    case _IO_Instance:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	instptr = &iol->__t.__forinstance;
	if (instptr->ptype == 0 && instptr->pcopy == 0)
	    tmp = "NULL";
	else
	{
	    if (!__RTS_FETCH_NAMES__)
		f = NULL;
	    else
		f = (*__RTS_FETCH_NAMES__) (instptr->ptype);
	    if (f)
	    {
		tmp = alloca (strlen (f) + 20);
		sprintf (tmp, "[%s;%u]", f, instptr->pcopy);
	    }
	    else
	    {
		sprintf (fmtbuf, "[%u;%u]", instptr->ptype, instptr->pcopy);
		tmp = fmtbuf;
	    }
	}
	reslen = strlen (tmp);
	break;

#if 0
    case _IO_Powerset:
	if (howtoconvert != Defconv)
	    return (conversion_fail);
	psstr = __xmalloc_ (4);
	strcpy (psstr, "[ ");
	f = iol->__t.__forpowerset.data;
	for (i = 0; i < iol->__t.__forpowerset.bitlength; i++)
	{
	    extern	int __inpowerset();
	    
	    if (__inpowerset (i, f, iol->__t.__forpowerset.bitlength, 0))
	    {
		intval = i + iol->__t.__forpowerset.lowbound;
		tmp = fmtbuf;
		switch (iol->__t.__forpowerset.type)
		{
		case _IO_Int:
		    sprintf (tmp, "%d ", intval);
		    break;
		case _IO_Bool:
		    if (intval == 0)
			strcpy (tmp, "FALSE ");
		    else if (intval == 1)
			strcpy (tmp, "TRUE ");
		    else
			sprintf (tmp, "BOOL(%d) ", intval);
		    break;
		case _IO_Set:
		    if (!iol->__t.__forpowerset.name_table)
			sprintf (tmp, "%d ", intval);
		    else
		    {
			nametable_length = 0;
			while (iol->__t.__forpowerset.name_table[nametable_length].name)
			{
			    if (iol->__t.__forpowerset.name_table[nametable_length].value == intval)
				break;
			    nametable_length++;
			}
			if (iol->__t.__forpowerset.name_table[nametable_length].name)
			    sprintf (tmp, "%s ",
				     iol->__t.__forpowerset.name_table[nametable_length].name);
			else
			    sprintf (tmp, "%d ", intval);
		    }
		    break;
		case _IO_Char:
		    if ((intval >= ' ') && (intval <= '~'))
			sprintf (tmp, "'%c' ", intval);
		    else if (intval < ' ')
			sprintf (tmp, "'^%c' ", intval + 0x40);
		    else
			sprintf (tmp, "C'%02X' ", intval);
		    break;
		default:
		    free (psstr);
		    return (conversion_fail);
		    break;
		}
		psstr = __xrealloc_ (psstr, strlen (psstr) + strlen (tmp) + 2);
		strcat (psstr, tmp);
	    }
	}
	strcat (psstr, "]");
	tmp = alloca ((reslen = strlen (psstr)) + 1);
	strcpy (tmp, psstr);
	free (psstr);
	break;
#endif

    case _IO_Real:
    case _IO_LongReal:
	if (!((howtoconvert == Real) || (howtoconvert == LongReal)))
	    return (conversion_fail);
	
	/* build real-format-string */
	f = realfmtstring;
	*f++ = '%';
	if (left_justified)
	    *f++ = '-';
	if (field_width != -1)
	{
	    sprintf (f, "%d", field_width);
	    f += strlen (f);
	}
	if (exp_width != -1)
	{
	    sprintf (f, ".%d", exp_width);
	    f += strlen (f);
	}
	if (howtoconvert == Real)
	    strcpy (f, "f");
	else
	    strcpy (f, "E");
	strcat (f, "%n");
	if (iol->__descr == _IO_Real)
	    sprintf (fmtbuf, realfmtstring, iol->__t.__forreal, &i);
	else
	    sprintf (fmtbuf, realfmtstring, iol->__t.__forlongreal, &i);
	assert (i < sizeof (fmtbuf));

	/* change blanks to pad_char */
	i = 0;
	if (pad_char != ' ')
	    while (fmtbuf[i])
	    {
		if (fmtbuf[i] == ' ')
		    fmtbuf[i] = pad_char;
		i++;
	    }
	strcpy (out, fmtbuf);
	return (end_of_formatting);
	break;
    default:
	return (conversion_fail);
	break;
    }

    /* fill output buffer */
    if ((field_width == -1) || (reslen == field_width))
	strcpy (out, tmp);
    else if (reslen > field_width)
    {
	if (mark_overflow)
	{
	    for (i = 0; i < field_width; i++)
		out[i] = '*';
	    out[i] = '\0';
	}
	else if (left_justified)
	{
	    for (i = 0; i < field_width; i++)
		out[i] = tmp[i];
	    out[i] = '\0';
	}
	else
	    strcpy (out, tmp + reslen - field_width);
    }
    else if (left_justified)
    {
	strcpy (out, tmp);
	f = out + reslen;
	for (i = reslen; i < field_width; i++)
	    *f++ = pad_char;
	*f = '\0';
    }
    else
    {
	for (i = 0; i < (field_width - reslen); i++)
	    out[i] = pad_char;
	out[i] = '\0';
	strcat (out, tmp);
    }
    
    return (end_of_formatting);
} /* default_conv */

/*
 * function __write_format
 *
 * parameters:
 *  formatstr		format string
 *  formatstrlen	length of format string
 *  iolist		io-list
 *  iolistlen		length of iolist
 *  iol_index		working index of io_list for recursion
 *  outstr		output string
 *  outstrlen		length of output string
 *  dest_index		destination index
 *  act_max_index	actual maximum index
 *  io_cntl_allowed \
 *  formfeed         \	arguments for
 *  h_text_ptr       /	output routine
 *  h_index         /
 *  out_function	output routine
 *
 * returns:
 *  __format_result
 *
 * exceptions:
 *  none
 *
 * abstract:
 *  do formatting for _writestring and _writetext
 *
*/

#define PUT_CHARS(STRING, LENGTH) {\
    if ((chr_copied = copy_text (STRING, outstr, outstrlen,\
				 dest_index, act_max_index,\
				 LENGTH)) == -1)\
	return (dest_txt_ind_out_of_range);}
#define PUT_CHAR(c) { char tmp_c = (c); PUT_CHARS(&tmp_c, 1); }

__format_result
__write_format (char *formatstr, 
		int formatstrlen,
		__tmp_WIO_list *iolist,
		int iolistlen, 
		int *iol_index,
		char *outstr,
		int outstrlen,
		int *dest_index,
		int *act_max_index,
		int io_cntl_allowed,
		int *formfeed,
		int h_text_ptr,
		int h_index,
		outfun out_function)
{
    register char	*fstr = formatstr;
    char		*save_fstr, *tmp;
    int			chr_copied, rep_count, field_width, mark_overflow;
    char		*flimit = fstr + formatstrlen;
    int			left_justified, exp_width, have_real;
    char		pad_char;
    __tmp_WIO_list	*iol;
    char		*wrkstr = alloca (MAX_WORK_LEN + 1);
    __format_result	res;
    __howtoconvert	howtoconvert;
    __io_type		iotype;

    while (fstr < flimit)
    {
	if (*fstr == '%')
	{
	    rep_count = 1;
	    fstr++;
	    if (fstr >= flimit)
	      return (unexp_end_of_form_strg);
	    if (isdigit (*fstr))
	    {
		/* we have a repetition factor */
		rep_count = *fstr++ - '0';
		while (fstr < flimit && isdigit (*fstr))
		    rep_count = (rep_count * 10) + (*fstr++ - '0');
		if (fstr >= flimit)
		    return (unexp_end_of_form_strg);
		if (!rep_count)
		    return (repet_factor_zero_not_allowed);
	    }
	    save_fstr = fstr;

	    while (rep_count)
	    {
		have_real = 0;
		fstr = save_fstr;
		switch (*fstr)
		{
		case '.':
		    fstr++;
		    /* Do nothing. */
		    break;
		case 'r':
		case 'R':
		    have_real = 1;
		    howtoconvert = Real;
		    goto do_c;
		case 'f':
		case 'F':
		    have_real = 1;
		    howtoconvert = LongReal;
		    goto do_c;
		case 'o':
		case 'O':
		    howtoconvert = Octal;
		    goto do_c;
		case 'h':
		    howtoconvert = HexLowcase;
		    goto do_c;
		case 'H':
		    howtoconvert = HexUpcase;
		    goto do_c;
		case 'b':
		case 'B':
		    howtoconvert = Binary;
		    goto do_c;
		case 'c':
		case 'C':
		    howtoconvert = Defconv;
		do_c: ;
		    pad_char = ' ';
		    field_width = exp_width = -1;
		    left_justified = mark_overflow = 0;
		    fstr++;
		    
		    /* check for iolist entry available */
		    if (*iol_index >= iolistlen)
			return (unexp_end_of_io_list);
		    iol = &iolist[(*iol_index)++];
		    
		    /* check conversion qualifier and clause width */
		    while (fstr < flimit)
		    {
			if (isdigit (*fstr))
			{
			    field_width = *fstr++ - '0';
			    while (fstr < flimit && isdigit (*fstr))
				field_width = (field_width * 10) + (*fstr++ - '0');
			    if (!field_width)
				return (field_width_zero_not_allowed);
			}
			else if ((*fstr == 'l') || (*fstr == 'L'))
			{
			    fstr++;
			    left_justified = 1;
			}
			else if ((*fstr == 'p') || (*fstr == 'P'))
			{
			    fstr++;
			    if (fstr >= flimit)
				return (unexp_end_of_form_strg);
			    pad_char = *fstr++;
			}
			else if ((*fstr == 'v') || (*fstr == 'V'))
			{
			    if (*iol_index >= iolistlen)
				return (unexp_end_of_io_list);
			    if ((res = get_field_width (&iolist[*iol_index],
							&field_width, 0))
				!= end_of_formatting)
				return (res);
			    (*iol_index)++;
			    fstr++;
			}
			else if ((*fstr == 'e') || (*fstr == 'E'))
			{
			    fstr++;
			    mark_overflow = 1;
			}
			else if (*fstr == '%'
				 && fstr + 1 < flimit && fstr[1] == '.')
			{
			    fstr += 2;
			    break;
			}
			else if (have_real && (*fstr == ':'))
			{
			    fstr++;
			    if (fstr >= flimit)
				return (unexp_end_of_form_strg);
			    if (isdigit (*fstr))
			    {
				exp_width = *fstr++ - '0';
				while (fstr < flimit && isdigit (*fstr))
				    exp_width = (exp_width * 10) + (*fstr++ - '0');
			    }
			    else if ((*fstr == 'v') || (*fstr == 'V'))
			    {
				if (*iol_index >= iolistlen)
				    return (unexp_end_of_io_list);
				if ((res = get_field_width (&iolist[*iol_index],
							    &exp_width, 1))
				    != end_of_formatting)
				    return (res);
				(*iol_index)++;
				fstr++;
			    }
			    else
				return (real_clause_width_expected);
			}
			else
			    break;
		    }
		    
		    /* do default conversion */
		    if ((res = default_conv (wrkstr, iol, left_justified,
					     mark_overflow, pad_char,
					     field_width, exp_width,
					     howtoconvert)) != end_of_formatting)
			return (res);
		    PUT_CHARS (wrkstr, strlen(wrkstr));
		    break;
		case 'x':
		case 'X':
		case '<':
		case '>':
		    field_width = 1;
		    goto do_editing;
		case 'T':
		case 't':
		    field_width = 0;
		do_editing: ;		
		    /* editing features */
		    /* save the character */
		    pad_char = *fstr++;
		    
		    /* get editing width */
		    if (fstr >= flimit) ; /* Nothing */
		    else if (isdigit (*fstr) || *fstr == '+' || *fstr == '-')
		    {
			int		negativ = 0;
			
			if ((*fstr == '+') || (*fstr == '-'))
			{
			    negativ = *fstr++ == '-';
			    if (fstr >= flimit || !isdigit (*fstr))
				return (format_spec_fail);
			}
			field_width = *fstr++ - '0';
			while (fstr < flimit && isdigit (*fstr))
			    field_width = (field_width * 10) + (*fstr++ - '0');
			if (negativ)
			    field_width = -field_width;
		    }
		    else if ((*fstr == 'v') || (*fstr == 'V'))
		    {
			if (*iol_index >= iolistlen)
			    return (unexp_end_of_io_list);
			if ((res = get_field_width (&iolist[*iol_index],
						    &field_width,
						    2)) != end_of_formatting)
			    return (res);
			(*iol_index)++;
			fstr++;
		    }
		    
		    if ((pad_char == 'x') || (pad_char == 'X'))
		    {
			int		i;

			if (field_width < 0)
			    return (field_width_zero_not_allowed);

			assert (field_width < MAX_WORK_LEN /* wrkstr to small */);
			for (i = 0; i < field_width; i++)
			    wrkstr[i] = ' ';
			wrkstr[field_width] = '\0';
			PUT_CHARS (wrkstr, field_width);
		    }
		    else if (pad_char == '<')
		    {
			if ((field_width <= *dest_index) &&
			    (field_width >= 0))
			    (*dest_index) -= field_width;
			else if (field_width < 0)
			{
			    field_width = -field_width;
			    if ((res = skip_upwards (outstr, outstrlen, dest_index,
						     act_max_index,
						     field_width)) != end_of_formatting)
				return (res);
			}
			else
			    return (dest_txt_ind_less_than_zero);
		    }
		    else if (pad_char == '>')
		    {
			if ((res = skip_upwards (outstr, outstrlen, dest_index,
						 act_max_index,
						 field_width)) != end_of_formatting)
			    return (res);
		    }
		    else if ((pad_char == 'T') || (pad_char == 't'))
		    {
			if (field_width < 0)
			    return (dest_txt_ind_out_of_range);
			if (field_width <= *act_max_index)
			    *dest_index = field_width;
			else if (field_width < outstrlen)
			{
			    char	*p = outstr + *act_max_index;
			    
			    while (*act_max_index != field_width)
			    {
				*p++ = ' ';
				(*act_max_index)++;
			    }
			    *dest_index = field_width;
			}
			else
			{
			    *dest_index = outstrlen;
			    return (dest_txt_ind_out_of_range);
			}
		    }
		    else
			return (not_implemented);
		    break;
		case '%':
		    PUT_CHAR ('%');
		    fstr += chr_copied;
		    break;
		case '(':
		    /* paranthesed expression.
		       search for closing parenthesis from the end of string.
		       FIXME: mismatch is detected very late. */
		    fstr++;
		    tmp = flimit;
		    while (tmp >= fstr)
		    {
			if (tmp[-2] == '%' && tmp[-1] == ')')
			    break;
			tmp--;
		    }
		    if (tmp <= fstr)
			return (format_spec_fail);
		    
		    /* now format the partial string */
		    if ((res = __write_format (fstr, tmp - fstr - 2,
					       iolist, iolistlen, iol_index,
					       outstr, outstrlen,
					       dest_index, act_max_index,
					       io_cntl_allowed, formfeed,
					       h_text_ptr, h_index,
					       out_function)) != end_of_formatting)
			return (res);

		    /* skip over "%) */
		    fstr = tmp;
		    break;
		case '/':
		    if (!io_cntl_allowed)
		      {
			PUT_CHAR('\n');
			fstr++;
			break;
		      }
		    iotype = io_1;
		    goto do_iocntl;
		case '+':
		    iotype = io_2;
		    goto do_iocntl;
		case '-':
		    iotype = io_3;
		    goto do_iocntl;
		case '?':
		    iotype = io_4;
		    goto do_iocntl;
		case '!':
		    iotype = io_5;
do_iocntl: ;		    
		    if (!io_cntl_allowed)
			return (io_cntrl_not_allowed);
		    if ((res = (*out_function) (h_text_ptr, h_index, formfeed,
						act_max_index,
						iotype)) != end_of_formatting)
			return (end_of_formatting);
		    fstr++;
		    *act_max_index = *dest_index = 0;
		    break;
		case '=':
		    if (!io_cntl_allowed)
			return (io_cntrl_not_allowed);
		    *formfeed = 1;
		    fstr++;
		    break;
		default:
		    return (format_spec_fail);
		    break;
		}
		rep_count--;
	    }
	}
	else
	{
	    /* copy the character to output */
	    save_fstr = fstr;
	    do { fstr++; } while (fstr < flimit && *fstr != '%');
	    PUT_CHARS(save_fstr, fstr - save_fstr);
	    fstr = save_fstr + chr_copied;
	}
    }
    return (end_of_formatting);
} /* __write_format */

#undef MAX_WORK_LEN

