/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (notconnected);
EXCEPTION (protectionfail);
EXCEPTION (rangefail);
EXCEPTION (readfail);
EXCEPTION (spacefail);
EXCEPTION (writefail);

/*
 * function write_control_char_begin
 *
*/

static void
write_control_char_begin (short fidx)
{
    __fs_table_entry	*fse = &__fs_table[fidx];
    char		cntl_char = 0;
    int			i;
    
    if (__ACCFF (fse->bit_list_acc))
    {
	__ACCCLRFF (fse->bit_list_acc);
	cntl_char = '\f';
    }
    else
    {
	if (__ACCIOT2 (fse->bit_list_acc))
	    cntl_char = '\f';
	else if (__ACCIOT3 (fse->bit_list_acc))
	    cntl_char = '\r';
    }
    
    if (cntl_char)
    {
	if ((i = write (fse->handle, &cntl_char, 1)) != 1)
	{
	    if (i == -1)
		fse->status_word = errno;
	    else
		fse->status_word = EIO;
	    CAUSE (writefail);
	}
    }
} /* write_control_char_begin */

/*
 * function write_to_device
 *
*/

static void
write_to_device (short fidx, void *store_loc_ptr)
{
    __fs_table_entry	*fse = &__fs_table[fidx];
    char		*wptr;
    int			bytes_to_write, bytes_written;
    
    if (__ACCVARYING (fse->bit_list_acc))
    {
	/* varying record mode */
	if (*(long *)store_loc_ptr > fse->rec_len_acc)
	    CAUSE (spacefail);

	if (__FORMATBITSET (fse->bit_list_ass) || __ACCTEXT (fse->bit_list_acc))
	{
	    /* format or text */
	    bytes_to_write = *(long *)store_loc_ptr;
	    wptr = (char *)store_loc_ptr + 4;
	    if (__ACCTEXT (fse->bit_list_acc))
		write_control_char_begin (fidx);
	}
	else
	{
	    /* not formatted, text */
	    bytes_to_write = *(long *)store_loc_ptr + 4;
	    wptr = store_loc_ptr;
	}
    }
    else
    {
	if (__FORMATBITSET (fse->bit_list_ass))
	{
	    bytes_to_write = fse->rec_len_acc;
	    wptr = store_loc_ptr;
	}
	else
	{
	    fse->status_word = EIO;
	    CAUSE (writefail);
	}
    }

    /* write bytes of store location to device */
    if ((bytes_written = write (fse->handle, wptr, bytes_to_write)) != bytes_to_write)
    {
	if (bytes_written == -1)
	    fse->status_word = errno;
	else
	    fse->status_word = EIO;
	CAUSE (writefail);
    }

    if (__ACCIOT1 (fse->bit_list_acc) || __FORMATBITSET (fse->bit_list_ass))
    {
	char	c = '\n';
	int	i = write (fse->handle, &c, 1);
	if (i != 1)
	{
	    if (i == -1)
		fse->status_word = errno;
	    else
		fse->status_word = EIO;
	    CAUSE (writefail);
	}
    }
    
} /* write_to_device */

/*
 * function _writerecord
 *
 * arguments:
 *  acc			pointer to access location
 *  idx			index of record to be written, if indexed
 *  store_loc_ptr	address of store location
 *
 * returns:
 *  void
 *
 * exceptions:
 *  notconnected
 *  protectionfail
 *  rangefail
 *  readfail
 *  spacefail
 *  writefail
 *
 * abstract:
 *  write a record to the file or device
 *
*/

void
_writerecord (__access *acc, int idx, void *store_loc_ptr)
{
    __fs_table_entry	*fse = &__fs_table[acc->fileindex];

    if (!__ISCONNECTED (acc))
	CAUSE (notconnected);
    
    if (!__ACCWRITE (fse->bit_list_acc))
    {
	fse->status_word = EINVAL;
	CAUSE (writefail);
    }
    
    
    if (fse->device_script |= File)
	write_to_device (acc->fileindex, store_loc_ptr);
    else
    {
	fse->status_word = EIO;
	CAUSE (writefail);
    }
    
} /* _writerecord */
