/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#if 0
#include <fcntl.h>
#include <sys/stat.h>
#endif
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

/* define all needed exceptions */
EXCEPTION (spacefail);
EXCEPTION (textfail);

/*
 * function _writestring
 *
 * parameters:
 *  outstr		pointer to varying character string
 *  outstrlen		length of body of outstr
 *  formatstr		pointer to format string
 *  formatstrlen	length of format string
 *  iolist		pointer to io-list
 *  iolistlen		length of io-list in bytes
 *
 * returns:
 *  void
 *
 * exceptions:
 *  spacefail		output string to short
 *  textfail		any error during formatting
 *
 * abstract:
 *  perform same function as `sprintf' in C for CHILL
 *
*/

void
_writestring (TVaryingCharType *outstr, int outstrlen,
	      char *formatstr, int formatstrlen,
	      __tmp_WIO_list *iolist, int iolistlen,
	      char *filename, int lineno)
{
    __format_result	fresult;
    int			dest_index = 0, act_max_index = 0, iol_index = 0;

    /* call formatting function */
    fresult = __write_format (formatstr, formatstrlen,
			      iolist, iolistlen / sizeof (__tmp_WIO_list), &iol_index,
			      &outstr->body[0], outstrlen,
			      &dest_index, &act_max_index,
			      0, 0, 0, 0, 0);

    /* set length of output string */
    outstr->len = act_max_index;
    
    /* if formatting fails -- exception */
    if (fresult != end_of_formatting)
	__cause_exception ("textfail", filename, lineno);
} /* _writestring */
