/* Implement Input/Output runtime actions for CHILL.
   Copyright (C) 1992,1993 Free Software Foundation, Inc.
   Author: Wilfried Moser, et al

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define __CHILL_LIB__

#define DEBUG

#include <stdio.h>
#include <sys/types.h>
#if 0
#include <fcntl.h>
#include <sys/stat.h>
#endif
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#ifdef sun	/* FIXME */
#include <floatingpoint.h>
#endif
#ifndef VMS
#include <unistd.h>
#endif

#include "iomodes.h"
#include "fileio.h"

#define OUTBUFLEN	10000

/*
 * function _writeunixstd
 *
 * parameters:
 *  outfile		pointer to a FILE stucture
 *  outstrlen		always 0
 *  formatstr		pointer to format string
 *  formatstrlen	length of format string
 *  iolist		pointer to io-list
 *  iolistlen		length of io-list in bytes
 *
 * returns:
 *  void
 *
 * exceptions:
 *  spacefail		output string to short
 *  textfail		any error during formatting
 *
 * abstract:
 *  perform same function as `fprintf' in C for CHILL
 *
*/

extern void _allocate_memory ();
extern void _return_memory ();

void
_writeunixstd (long outfile, int outlen_dummy,
	       char *formatstr, int formatstrlen,
	       __tmp_WIO_list *iolist, int iolistlen,
	       char *filename, int lineno)
{
    TVaryingCharType	*outp;
    FILE *file;

    _allocate_memory (&outp, sizeof (TVaryingCharType) + OUTBUFLEN + 1,
		      filename, lineno);
    _writestring (outp, OUTBUFLEN, formatstr, formatstrlen,
		  iolist, iolistlen, filename, lineno);
    switch (outfile)
      {
      case 0:
	file = stdin;
	break;
      case 1:
	file = stdout;
	break;
      case 2:
	file = stderr;
	break;
      default:
#if 0
	write (outfile, &outp->body[0], outp->len);
#else
	/* if we come here we have an error in the compiler */
	fprintf (stderr, "LibChill: invalid file handle %d.\n", outfile);
	abort ();
#endif
	return;
      }
#if 0
    fwrite (&outp->body[0], 1, outp->len, file);
#else
    /* Necessary due to porting to a system have only fprintf */
    outp->body[outp->len] = '\0';
    fprintf (file, "%s", outp->body);
#endif
    _return_memory (outp, filename, lineno);
}
