dates: MODULE

-- diffdays computes the number of days between two
-- dates given by day, month, and year.

NEWMODE m_date = STRUCT (
  year  UINT,
  month UBYTE(1:12),
  day   UBYTE(1:31));

DCL datea, dateb m_date;

daysofmonth: PROC( year UINT,
                   month UINT(1:12) ) RETURNS (UINT);
  CASE month OF
  (1, 3, 5, 7, 8, 10, 12):
    RETURN 31;
  (4, 6, 9, 11):
    RETURN 30;
  (2):
    RETURN
    IF year MOD 4 = 0 OR year MOD 100 = 0 AND year MOD 400 /= 0
    THEN 29
    ELSE 28 FI;
  ESAC;
END daysofmonth;

isbefore: PROC( d1, d2 m_date IN) RETURNS (BOOL);
  RETURN d1.year < d2.year ORIF
     d1.year = d2.year AND d1.month < d2.month ORIF
     d1.year = d2.year AND d1.month = d2.month AND d1.day <d2.day;
END isbefore;
     

diffdays: PROC (d1, d2 m_date IN) RETURNS (LONG);
DCL  i, h, sig INT;
DCL ddif LONG;
DCL hd m_date;

IF isbefore( d1, d2 ) THEN
  sig := 1;
ELSE
  sig := -1;
  hd := d1; d1 := d2; d2 := hd;
FI;

/* Compute number of days in the years y1...y2,
 * including leap days.
 */
h := d1.year - 1;
ddif := (d2.year - h)*365
        + d2.year / 4 - d2.year / 100 + d2.year / 400
        - (h / 4 - h / 100 + h / 400);

/* days in first and last month not to be counted */
ddif -:= d1.day + (daysofmonth(d2.year,d2.month) - d2.day);

/* months in first year not to be counted */
DO FOR i := 1 TO d1.month-1;
   ddif -:= daysofmonth(d1.year,i);
OD;

/* months in last year not to be counted */
DO FOR i := d2.month+1 TO 12;
   ddif -:= daysofmonth(d2.year,i);
OD;

RETURN sig*ddif;

END diffdays;

printdate: PROC( d m_date );
  WRITETEXT( stdout, "%C.%C.%C", d.day, d.month,d.year );
END printdate;

datea := m_date[1901, 1, 1];
dateb := m_date[1994, 1, 24];

printdate( datea );
WRITETEXT( stdout, " - " );
printdate( dateb );
WRITETEXT( stdout, " %C days%/", diffdays(datea, dateb) );

datea := m_date[1994, 1, 1];
dateb := m_date[1994, 12, 31];

printdate( datea );
WRITETEXT( stdout, " - " );
printdate( dateb );
WRITETEXT( stdout, " %C days%/", diffdays(datea, dateb) );

END dates;
