chilllib: MODULE

--
-- This module tries to perform a check for all functions 
-- in libchill.a (except those where definitive tests are done)
--

--
-- we get an exception name in here (by overwriting the function cause_exception)
--
SYNMODE m_exname = CHARS (100) VARYING;
DCL __exname, __exfile m_exname;

--
-- struct where powerset is aligned to check if length of each
-- operation is handled properly
--
SYNMODE ps4b_T = POWERSET LONG (0:31);
SYNMODE d_T = STRUCT (
        ps ps4b_T,
        l long);
--
-- a, b are used for all ps operations
--
DCL a, b d_T;

SYNMODE str_M = STRUCT (
        s41 CHAR(4),
        s42 CHAR(4),
        s8 CHAR(8));
DCL sa, sb str_M;

--
-- print ps4b_T
--
print_ps4b_t: PROC (x ps4b_t)

  DCL i LONG;

  WRITETEXT (stdout, "[ ");
  DO FOR i IN x;
    WRITETEXT (stdout, "%C ", i);
  OD;
  WRITETEXT (stdout, "]");
END print_ps4b_T;

--
-- overwrite cause_exception
--
cause_exception: PROC (name CHARS (200) LOC,
                       file CHARS (200) LOC)

   DCL i, j LONG;

   i := 0;
   DO WHILE name(i) /= C'00';
      i +:= 1;
   OD;
   __exname := name(0 UP i);

   i := 0;
   DO WHILE file(i) /= C'00';
      i +:= 1;
   OD;

--
-- remove path from filename
--
   j := i - 1;
   DO WHILE file(j) /= '/';
      j -:= 1;
   OD;
   __exfile := file(j + 1 : i - 1);
END cause_exception;

GRANT cause_exception;

--
-- this function prints the name of raised exception
-- and check agains expected.
--
print_exception: PROC (func     m_exname LOC,
                       expected m_exname LOC)

   DCL res BOOL;

   res := expected = __exname;
   WRITETEXT (stdout, "%C: Exception ""%C"" raised in file ""%C"" --> %C.%/",
              func, __exname, __exfile, res);
END print_exception;

--
-- function dumps a powerset with given length as bitstring
--
dump_bitstring: PROC (str m_exname LOC,
                      x PTR IN,
                      len LONG IN)

   SYNMODE pbit8  = REF BOOLS(8);
   SYNMODE pbit16 = REF BOOLS(16);
   SYNMODE pbit24 = REF BOOLS(24);
   SYNMODE pbit32 = REF BOOLS(32);
   SYNMODE pbit40 = REF BOOLS(40);
   SYNMODE pbit48 = REF BOOLS(48);

   WRITETEXT (stdout, "%C", str);
   CASE len OF
      (1): WRITETEXT (stdout, "%C", pbit8(x)->);
      (2): WRITETEXT (stdout, "%C", pbit16(x)->);
      (3): WRITETEXT (stdout, "%C", pbit24(x)->);
      (4): WRITETEXT (stdout, "%C", pbit32(x)->);
      (5): WRITETEXT (stdout, "%C", pbit40(x)->);
      (6): WRITETEXT (stdout, "%C", pbit48(x)->);
      ELSE
         WRITETEXT (stdout, "Cannot process LEN %C", len);
   ESAC;
   WRITETEXT (stdout, "%/");

END dump_bitstring;

--
-- allgmem.c    _allocate_global_memory
--
allgmem: PROC ()

   DCL p PTR;

   ALLOCATE_GLOBAL_MEMORY (p, 100)
   ON
      ELSE
         print_exception ("allgmem", "notimplemented");
   END;

END allgmem;

--
-- allmem.c     _allocate_memory
--
allmem: PROC ()

   DCL p REF LONG;
   DCL len LONG := -100;

   ALLOCATE_MEMORY (p, len)
   ON
      ELSE
         print_exception ("allmem", "rangefail");
   END;
   ALLOCATE_MEMORY (p, SIZE(p->));
   p-> := 1111;
   WRITETEXT (stdout, "allmem: p-> := %C%/", p->);
   RETURN_MEMORY (p);
END allmem;

--
-- andps.c      __andpowerset
--
andps: PROC ()

   SYNMODE m_ps3 = POWERSET LONG (0:3);
   SYNMODE m_ps8 = POWERSET LONG (0:7);
   SYNMODE m_ps13 = POWERSET LONG (0:12);
   SYNMODE m_ps48 = POWERSET LONG (0:47);

   DCL ps3_1 m_ps3 := [ 0:3 ];
   DCL ps3_2 m_ps3 := [ 1:2 ];
   DCL ps3_3 m_ps3;

   DCL ps8_1 m_ps8 := [ 0:7 ];
   DCL ps8_2 m_ps8 := [ 0, 1, 6, 7 ];
   DCL ps8_3 m_ps8;

   DCL ps13_1 m_ps13 := [ 0:12 ];
   DCL ps13_2 m_ps13 := [ 0, 2, 4, 6, 8, 10, 12 ];
   DCL ps13_3 m_ps13;

   DCL ps48_1 m_ps48 := [ 0:47 ];
   DCL ps48_2 m_ps48 := [ 0:1, 46:47 ];
   DCL ps48_3 m_ps48;

   print_ps3: PROC (x m_ps3)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps3;

   print_ps8: PROC (x m_ps8)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps8;

   print_ps13: PROC (x m_ps13)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps13;

   print_ps48: PROC (x m_ps48)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps48;

   ps3_3 := ps3_1 AND ps3_2;
--   WRITETEXT (stdout, "andps: %C AND %C := %C%/", ps3_1, ps3_2, ps3_3);
   WRITETEXT (stdout, "andps: ");
   print_ps3 (ps3_1);
   WRITETEXT (stdout, " AND ");
   print_ps3 (ps3_2);
   WRITETEXT (stdout, " := ");
   print_ps3 (ps3_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("andps: dump of result = ", ADDR (ps3_3), SIZE (ps3_3));

   ps8_3 := ps8_1 AND ps8_2;
--   WRITETEXT (stdout, "andps: %C AND %C := %C%/", ps8_1, ps8_2, ps8_3);
   WRITETEXT (stdout, "andps: ");
   print_ps8 (ps8_1);
   WRITETEXT (stdout, " AND ");
   print_ps8 (ps8_2);
   WRITETEXT (stdout, " := ");
   print_ps8 (ps8_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("andps: dump of result = ", ADDR (ps8_3), SIZE (ps8_3));

   ps13_3 := ps13_1 AND ps13_2;
--   WRITETEXT (stdout, "andps: %C AND %C := %C%/", ps13_1, ps13_2, ps13_3);
   WRITETEXT (stdout, "andps: ");
   print_ps13 (ps13_1);
   WRITETEXT (stdout, " AND ");
   print_ps13 (ps13_2);
   WRITETEXT (stdout, " := ");
   print_ps13 (ps13_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("andps: dump of result = ", ADDR (ps13_3), SIZE (ps13_3));

   ps48_3 := ps48_1 AND ps48_2;
--   WRITETEXT (stdout, "andps: %C AND %C := %C%/", ps48_1, ps48_2, ps48_3);
   WRITETEXT (stdout, "andps: ");
   print_ps48 (ps48_1);
   WRITETEXT (stdout, " AND ");
   print_ps48 (ps48_2);
   WRITETEXT (stdout, " := ");
   print_ps48 (ps48_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("andps: dump of result = ", ADDR (ps48_3), SIZE (ps48_3));

END andps;

--
-- cardps.c     __cardpowerset
--
cardps: PROC ()

   SYNMODE m_ps3 = POWERSET LONG (0:3);
   SYNMODE m_ps13 = POWERSET LONG (0:13);
   SYNMODE m_ps32 = POWERSET LONG (0:31);

   DCL s1 STRUCT (ps3 m_ps3,
                  l   LONG) := [ [ 0, 3 ], -1 ];
   DCL s2 STRUCT (ps13 m_ps13,
                  l    LONG) := [ [ 0, 6, 12 ], -1 ];
   DCL s3 STRUCT (ps32 m_ps32,
                  l    LONG) := [ [ 0, 10, 20, 31 ], -1 ];

   print_ps3: PROC (x m_ps3)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps3;

   print_ps13: PROC (x m_ps13)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps13;

   print_ps32: PROC (x m_ps32)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps32;

--   WRITETEXT (stdout, "cardps: CARD (%C) := %C%/", s1.ps3, CARD (s1.ps3));
   WRITETEXT (stdout, "cardps: CARD (");
   print_ps3 (s1.ps3);
   WRITETEXT (stdout, ") := %C%/", CARD (s1.ps3));
--   WRITETEXT (stdout, "cardps: CARD (%C) := %C%/", s2.ps13, CARD (s2.ps13));
   WRITETEXT (stdout, "cardps: CARD (");
   print_ps13 (s2.ps13);
   WRITETEXT (stdout, ") := %C%/", CARD (s2.ps13));
--   WRITETEXT (stdout, "cardps: CARD (%C) := %C%/", s3.ps32, CARD (s3.ps32));
   WRITETEXT (stdout, "cardps: CARD (");
   print_ps32 (s3.ps32);
   WRITETEXT (stdout, ") := %C%/", CARD (s3.ps32));
END cardps;

--
-- ffsetps.c    __ffsetpowerset
--
ffsetps: PROC ()

   SYNMODE m_ps3 = POWERSET LONG (0:3);
   SYNMODE m_ps13 = POWERSET LONG (0:13);
   SYNMODE m_ps32 = POWERSET LONG (0:31);

   DCL s1 STRUCT (ps3 m_ps3,
                  l   LONG) := [ [ 3 ], -1 ];
   DCL s2 STRUCT (ps13 m_ps13,
                  l    LONG) := [ [ 6, 12 ], -1 ];
   DCL s3 STRUCT (ps32 m_ps32,
                  l    LONG) := [ [ 10, 20, 31 ], -1 ];
   DCL l LONG;

   print_ps3: PROC (x m_ps3)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps3;

   print_ps13: PROC (x m_ps13)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps13;

   print_ps32: PROC (x m_ps32)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps32;

--   WRITETEXT (stdout, "ffsetps: MIN (%C) := %C%/", s1.ps3, MIN (s1.ps3));
   WRITETEXT (stdout, "ffsetps: MIN (");
   print_ps3 (s1.ps3);
   WRITETEXT (stdout, ") := %C%/", MIN (s1.ps3));
--   WRITETEXT (stdout, "ffsetps: MIN (%C) := %C%/", s2.ps13, MIN (s2.ps13));
   WRITETEXT (stdout, "ffsetps: MIN (");
   print_ps13 (s2.ps13);
   WRITETEXT (stdout, ") := %C%/", MIN (s2.ps13));
--   WRITETEXT (stdout, "ffsetps: MIN (%C) := %C%/", s3.ps32, MIN (s3.ps32));
   WRITETEXT (stdout, "ffsetps: MIN (");
   print_ps32 (s3.ps32);
   WRITETEXT (stdout, ") := %C%/", MIN (s3.ps32));

   s1.ps3 := [];
   s2.ps13 := [];
   s3.ps32 := [];
   l := MIN (s1.ps3)
   ON
      ELSE
         print_exception ("ffsetps", "empty");
   END;
   l := MIN (s2.ps13)
   ON
      ELSE
         print_exception ("ffsetps", "empty");
   END;
   l := MIN (s3.ps32)
   ON
      ELSE
         print_exception ("ffsetps", "empty");
   END;
END ffsetps;

--
-- flsetps      __flsetpowerset
--
flsetps: PROC ()


   SYNMODE m_ps3 = POWERSET LONG (0:3);
   SYNMODE m_ps13 = POWERSET LONG (0:13);
   SYNMODE m_ps32 = POWERSET LONG (0:31);

   DCL s1 STRUCT (ps3 m_ps3,
                  l   LONG) := [ [ 3 ], -1 ];
   DCL s2 STRUCT (ps13 m_ps13,
                  l    LONG) := [ [ 6, 12 ], -1 ];
   DCL s3 STRUCT (ps32 m_ps32,
                  l    LONG) := [ [ 10, 20, 31 ], -1 ];
   DCL l LONG;

   print_ps3: PROC (x m_ps3)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps3;

   print_ps13: PROC (x m_ps13)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps13;

   print_ps32: PROC (x m_ps32)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps32;

--   WRITETEXT (stdout, "flsetps: MAX (%C) := %C%/", s1.ps3, MAX (s1.ps3));
   WRITETEXT (stdout, "flsetps: MAX (");
   print_ps3 (s1.ps3);
   WRITETEXT (stdout, ") := %C%/", MAX (s1.ps3));
--   WRITETEXT (stdout, "flsetps: MAX (%C) := %C%/", s2.ps13, MAX (s2.ps13));
   WRITETEXT (stdout, "flsetps: MAX (");
   print_ps13 (s2.ps13);
   WRITETEXT (stdout, ") := %C%/", MAX (s2.ps13));
--   WRITETEXT (stdout, "flsetps: MAX (%C) := %C%/", s3.ps32, MAX (s3.ps32));
   WRITETEXT (stdout, "flsetps: MAX (");
   print_ps32 (s3.ps32);
   WRITETEXT (stdout, ") := %C%/", MAX (s3.ps32));

   s1.ps3 := [];
   s2.ps13 := [];
   s3.ps32 := [];
   l := MAX (s1.ps3)
   ON
      ELSE
         print_exception ("flsetps", "empty");
   END;
   l := MAX (s2.ps13)
   ON
      ELSE
         print_exception ("flsetps", "empty");
   END;
   l := MAX (s3.ps32)
   ON
      ELSE
         print_exception ("flsetps", "empty");
   END;
END flsetps;

--
-- concatps.c   __concatps
--
concatps: PROC ()

   DCL b4_1  BOOLS(4)  := B'1100';
   DCL b4_2  BOOLS(4)  := B'1100';
   DCL b8_1  BOOLS(8)  := B'11110000';
   DCL b8_2  BOOLS(8)  := B'11110000';
   DCL b16_1 BOOLS(16) := B'1111111100000000';
   DCL b16_2 BOOLS(16) := B'1111111100000000';

   DCL s1 STRUCT (b8 BOOLS(8),
                  l  LONG) := [ B'00000000', -1 ];

   DCL s2 STRUCT (b16 BOOLS(16),
                  l   LONG) := [ B'0000000000000000', -1 ];

   DCL s3 STRUCT (b32 BOOLS(32),
                  l   LONG) := [ B'00000000000000000000000000000000', -1 ];

   s1.b8 := b4_1 // b4_2;
   WRITETEXT (stdout, "concatps: %C // %C := [ %C, %H ]%/", b4_1, b4_2,
                      s1.b8, s1.l);
   s2.b16 := b8_1 // b8_2;
   WRITETEXT (stdout, "concatps: %C // %C := [ %C, %H ]%/", b8_1, b8_2,
                      s2.b16, s2.l);
   s3.b32 := b16_1 // b16_2;
   WRITETEXT (stdout, "concatps: %C // %C := [ %C, %H ]%/", b16_1, b16_2,
                      s3.b32, s3.l);
END concatps;

--
-- sliceps.c    __psslice
--
sliceps: PROC ()

   DCL b5 BOOLS(5) := B'11000';

   DCL s1 STRUCT (b8 BOOLS(8),
                  l  LONG) := [ B'11000000', -1 ];

   DCL s2 STRUCT (b16 BOOLS(16),
                  l  LONG) := [ B'1100000011000000', -1 ];

   DCL s3 STRUCT (b32 BOOLS(32),
                  l  LONG) := [ B'11000000110000001100000011000000', -1 ];

   s1.b8(2 UP 5) := b5;
   WRITETEXT (stdout, "sliceps: result = [ %C, %H ]%/", s1.b8, s1.l);
   s2.b16(2 UP 5) := b5;
   s2.b16(10 UP 5) := b5;
   WRITETEXT (stdout, "sliceps: result = [ %C, %H ]%/", s2.b16, s2.l);
   s3.b32(2 UP 5) := b5;
   s3.b32(10 UP 5) := b5;
   s3.b32(18 UP 5) := b5;
   s3.b32(26 UP 5) := b5;
   WRITETEXT (stdout, "sliceps: result = [ %C, %H ]%/", s3.b32, s3.l);
END sliceps;

--
-- diffps.c     __diffpowerset
--
diffps: PROC ()

   SYNMODE m_ps8  = POWERSET LONG (0:7);
   SYNMODE m_ps16 = POWERSET LONG (0:15);
   SYNMODE m_ps32 = POWERSET LONG (0:31);

   DCL ps8_1 m_ps8 := [ 1, 3, 5, 7 ];
   DCL ps8_2 m_ps8 := [ 0 : 6 ];
   DCL s1 STRUCT (ps8 m_ps8,
                  l   LONG) := [ [ ], -1 ];

   DCL ps16_1 m_ps16 := [ 11, 13, 15 ];
   DCL ps16_2 m_ps16 := [ 11 : 14 ];
   DCL s2 STRUCT (ps16 m_ps16,
                  l    LONG) := [ [ ], -1 ];

   DCL ps32_1 m_ps32 := [ 1, 25, 27, 29, 31 ];
   DCL ps32_2 m_ps32 := [ 25 : 30 ];
   DCL s3 STRUCT (ps32 m_ps32,
                  l    LONG) := [ [ ], -1 ];

   print_ps8: PROC (x m_ps8)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps8;

   print_ps16: PROC (x m_ps16)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps16;

   print_ps32: PROC (x m_ps32)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps32;

   s1.ps8 := ps8_1 - ps8_2;
--   WRITETEXT (stdout, "diffps: %C - %C := [ %C, %H ]%/", ps8_1, ps8_2, s1.ps8, s1.l);
   WRITETEXT (stdout, "diffps: ");
   print_ps8 (ps8_1);
   WRITETEXT (stdout, " - ");
   print_ps8 (ps8_2);
   WRITETEXT (stdout, " := [ ");
   print_ps8 (s1.ps8);
   WRITETEXT (stdout, ", %H ]%/", s1.l);
   s2.ps16 := ps16_1 - ps16_2;
--   WRITETEXT (stdout, "diffps: %C - %C := [ %C, %H ]%/", ps16_1, ps16_2, s2.ps16, s2.l);
   WRITETEXT (stdout, "diffps: ");
   print_ps16 (ps16_1);
   WRITETEXT (stdout, " - ");
   print_ps16 (ps16_2);
   WRITETEXT (stdout, " := [ ");
   print_ps16 (s2.ps16);
   WRITETEXT (stdout, ", %H ]%/", s2.l);
   s3.ps32 := ps32_1 - ps32_2;
--   WRITETEXT (stdout, "diffps: %C - %C := [ %C, %H ]%/", ps32_1, ps32_2, s3.ps32, s3.l);
   WRITETEXT (stdout, "diffps: ");
   print_ps32 (ps32_1);
   WRITETEXT (stdout, " - ");
   print_ps32 (ps32_2);
   WRITETEXT (stdout, " := [ ");
   print_ps32 (s3.ps32);
   WRITETEXT (stdout, ", %H ]%/", s3.l);
END diffps;

--
-- eqps.c       __eqpowerset
--
eqps: PROC ()

   SYNMODE m_ps8  = POWERSET LONG (0:7);
   SYNMODE m_ps16 = POWERSET LONG (0:15);
   SYNMODE m_ps32 = POWERSET LONG (0:31);

   DCL s1 ARRAY (0:1)
          STRUCT (ps8 m_ps8,
                  l   LONG) := [ [ [ 3 : 5 ], 0 ], [ [ 3 : 5 ], -1 ] ];
   DCL s2 ARRAY (0:1)
          STRUCT (ps16 m_ps16,
                  l    LONG) := [ [ [ 13 : 15 ], 0 ], [ [ 13 : 15 ], -1 ] ];
   DCL s3 ARRAY (0:1)
          STRUCT (ps32 m_ps32,
                  l    LONG) := [ [ [ 23 : 25 ], 0 ], [ [ 23 : 25 ], -1 ] ];

   print_ps8: PROC (x m_ps8)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps8;

   print_ps16: PROC (x m_ps16)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps16;

   print_ps32: PROC (x m_ps32)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps32;

--   WRITETEXT (stdout, "eqps: %C = %C := %C%/", s1(0).ps8, s1(1).ps8,
--                      s1(0).ps8 = s1(1).ps8);
   WRITETEXT (stdout, "eqps: ");
   print_ps8 (s1(0).ps8);
   WRITETEXT (stdout, " = ");
   print_ps8 (s1(1).ps8);
   WRITETEXT (stdout, " := %C%/", s1(0).ps8 = s1(1).ps8);
   s1(0).ps8 -:= m_ps8 [ 4 ];
--   WRITETEXT (stdout, "eqps: %C = %C := %C%/", s1(0).ps8, s1(1).ps8,
--                      s1(0).ps8 = s1(1).ps8);
   WRITETEXT (stdout, "eqps: ");
   print_ps8 (s1(0).ps8);
   WRITETEXT (stdout, " = ");
   print_ps8 (s1(1).ps8);
   WRITETEXT (stdout, " := %C%/", s1(0).ps8 = s1(1).ps8);

--   WRITETEXT (stdout, "eqps: %C = %C := %C%/", s2(0).ps16, s2(1).ps16,
--                      s2(0).ps16 = s2(1).ps16);
   WRITETEXT (stdout, "eqps: ");
   print_ps16 (s2(0).ps16);
   WRITETEXT (stdout, " = ");
   print_ps16 (s2(1).ps16);
   WRITETEXT (stdout, " := %C%/", s2(0).ps16 = s2(1).ps16);
   s2(0).ps16 -:= m_ps16 [ 14 ];
--   WRITETEXT (stdout, "eqps: %C = %C := %C%/", s2(0).ps16, s2(1).ps16,
--                      s2(0).ps16 = s2(1).ps16);
   WRITETEXT (stdout, "eqps: ");
   print_ps16 (s2(0).ps16);
   WRITETEXT (stdout, " = ");
   print_ps16 (s2(1).ps16);
   WRITETEXT (stdout, " := %C%/", s2(0).ps16 = s2(1).ps16);

--   WRITETEXT (stdout, "eqps: %C = %C := %C%/", s3(0).ps32, s3(1).ps32,
--                      s3(0).ps32 = s3(1).ps32);
   WRITETEXT (stdout, "eqps: ");
   print_ps32 (s3(0).ps32);
   WRITETEXT (stdout, " = ");
   print_ps32 (s3(1).ps32);
   WRITETEXT (stdout, " := %C%/", s3(0).ps32 = s3(1).ps32);
   s3(0).ps32 -:= m_ps32 [ 24 ];
--   WRITETEXT (stdout, "eqps: %C = %C := %C%/", s3(0).ps32, s3(1).ps32,
--                      s3(0).ps32 = s3(1).ps32);
   WRITETEXT (stdout, "eqps: ");
   print_ps32 (s3(0).ps32);
   WRITETEXT (stdout, " = ");
   print_ps32 (s3(1).ps32);
   WRITETEXT (stdout, " := %C%/", s3(0).ps32 = s3(1).ps32);
END eqps;

--
-- neps.c       __nepowerset
--
neps: PROC ()

   SYNMODE m_ps8  = POWERSET LONG (0:7);
   SYNMODE m_ps16 = POWERSET LONG (0:15);
   SYNMODE m_ps32 = POWERSET LONG (0:31);

   DCL s1 ARRAY (0:1)
          STRUCT (ps8 m_ps8,
                  l   LONG) := [ [ [ 3 : 5 ], 0 ], [ [ 3 : 5 ], -1 ] ];
   DCL s2 ARRAY (0:1)
          STRUCT (ps16 m_ps16,
                  l    LONG) := [ [ [ 13 : 15 ], 0 ], [ [ 13 : 15 ], -1 ] ];
   DCL s3 ARRAY (0:1)
          STRUCT (ps32 m_ps32,
                  l    LONG) := [ [ [ 23 : 25 ], 0 ], [ [ 23 : 25 ], -1 ] ];

   print_ps8: PROC (x m_ps8)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps8;

   print_ps16: PROC (x m_ps16)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps16;

   print_ps32: PROC (x m_ps32)

     DCL i LONG;

     WRITETEXT (stdout, "[ ");
     DO FOR i IN x;
       WRITETEXT (stdout, "%C ", i);
     OD;
     WRITETEXT (stdout, "]");
   END print_ps32;

--   WRITETEXT (stdout, "neps: %C /= %C := %C%/", s1(0).ps8, s1(1).ps8,
--                      s1(0).ps8 /= s1(1).ps8);
   WRITETEXT (stdout, "neps: ");
   print_ps8 (s1(0).ps8);
   WRITETEXT (stdout, " /= ");
   print_ps8 (s1(1).ps8);
   WRITETEXT (stdout, " := %C%/", s1(0).ps8 /= s1(1).ps8);
   s1(0).ps8 -:= m_ps8 [ 4 ];
--   WRITETEXT (stdout, "neps: %C /= %C := %C%/", s1(0).ps8, s1(1).ps8,
--                      s1(0).ps8 /= s1(1).ps8);
   WRITETEXT (stdout, "neps: ");
   print_ps8 (s1(0).ps8);
   WRITETEXT (stdout, " /= ");
   print_ps8 (s1(1).ps8);
   WRITETEXT (stdout, " := %C%/", s1(0).ps8 /= s1(1).ps8);

--   WRITETEXT (stdout, "neps: %C /= %C := %C%/", s2(0).ps16, s2(1).ps16,
--                      s2(0).ps16 /= s2(1).ps16);
   WRITETEXT (stdout, "neps: ");
   print_ps16 (s2(0).ps16);
   WRITETEXT (stdout, " /= ");
   print_ps16 (s2(1).ps16);
   WRITETEXT (stdout, " := %C%/", s2(0).ps16/= s2(1).ps16);
   s2(0).ps16 -:= m_ps16 [ 14 ];
--   WRITETEXT (stdout, "neps: %C /= %C := %C%/", s2(0).ps16, s2(1).ps16,
--                      s2(0).ps16 /= s2(1).ps16);
   WRITETEXT (stdout, "neps: ");
   print_ps16 (s2(0).ps16);
   WRITETEXT (stdout, " /= ");
   print_ps16 (s2(1).ps16);
   WRITETEXT (stdout, " := %C%/", s2(0).ps16/= s2(1).ps16);

--   WRITETEXT (stdout, "neps: %C /= %C := %C%/", s3(0).ps32, s3(1).ps32,
--                      s3(0).ps32 /= s3(1).ps32);
   WRITETEXT (stdout, "neps: ");
   print_ps32 (s3(0).ps32);
   WRITETEXT (stdout, " /= ");
   print_ps32 (s3(1).ps32);
   WRITETEXT (stdout, " := %C%/", s3(0).ps32 /= s3(1).ps32);
   s3(0).ps32 -:= m_ps32 [ 24 ];
--   WRITETEXT (stdout, "neps: %C /= %C := %C%/", s3(0).ps32, s3(1).ps32,
--                      s3(0).ps32 /= s3(1).ps32);
   WRITETEXT (stdout, "neps: ");
   print_ps32 (s3(0).ps32);
   WRITETEXT (stdout, " /= ");
   print_ps32 (s3(1).ps32);
   WRITETEXT (stdout, " := %C%/", s3(0).ps32 /= s3(1).ps32);
END neps;

--
-- setbitps
--
setbitps: PROC();
    a.l := 0;
    a.ps := [0:31];

--    WRITETEXT(stdout, "setbitps: long: %C; ps: %C -> %C.%/", a.l, a.ps, (a.l = 0));
    WRITETEXT (stdout, "setbitps: long: %C; ps: ", a.l);
    print_ps4b_T (a.ps);
    WRITETEXT (stdout, " -> %C.%/", (a.l = 0));
    dump_bitstring("setbitps: ", ADDR(a.ps), SIZE(a.ps));
END setbitps;

--
-- ffsetclrps
--
ffsetclrps: PROC();
    SYNMODE ps1_T = POWERSET LONG (0:20);
    DCL ps1, ps2, ps3 ps1_T;
    DCL i LONG;
    
    print_ps_T: PROC (x ps1_T)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
        WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
    END print_ps_T;

    ps1 := [0:10, 12:20];
    ps2 := [];

    DO FOR i IN ps1;
--        WRITETEXT(stdout, "ffsetclrps: %C is in %C.%/", i, ps1);
	WRITETEXT (stdout, "ffsetclrps: %C is in ", i);
	print_ps_T (ps1);
	WRITETEXT (stdout, ".%/");
        ps3 := [i];
        ps2 := ps2 OR ps3;
    OD;
    IF ps1 /= ps2 THEN
--        WRITETEXT(stdout, "ffsetclrps: %C /= %C !!!.%/", ps1, ps2);
	WRITETEXT (stdout, "ffsetclrps: ");
	print_ps_T (ps1);
	WRITETEXT (stdout, " /= ");
	print_ps_T (ps2);
	WRITETEXT (stdout, " !!!.%/");
    ELSE
        WRITETEXT(stdout, "ffsetclrps: OK.%/");
    FI;

END ffsetclrps;

--
-- flsetclrps
--
flsetclrps: PROC();
    SYNMODE ps1_T = POWERSET LONG (0:20);
    DCL ps1, ps2, ps3 ps1_T;
    DCL i LONG;
    
    print_ps_T: PROC (x ps1_T)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
        WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
    END print_ps_T;

    ps1 := [0:10, 12:20];
    ps2 := [];

    DO FOR i DOWN IN ps1;
--        WRITETEXT(stdout, "flsetclrps: %C is in %C.%/", i, ps1);
	WRITETEXT (stdout, "flsetclrps: %C is in ", i);
	print_ps_T (ps1);
	WRITETEXT (stdout, ".%/");
        ps3 := [i];
        ps2 := ps2 OR ps3;
    OD;
    IF ps1 /= ps2 THEN
--        WRITETEXT(stdout, "flsetclrps: %C /= %C !!!.%/", ps1, ps2);
	WRITETEXT (stdout, "flsetclrps: ");
	print_ps_T (ps1);
	WRITETEXT (stdout, " /= ");
	print_ps_T (ps2);
	WRITETEXT (stdout, " !!!.%/");
    ELSE
        WRITETEXT(stdout, "flsetclrps: OK.%/");
    FI;
END flsetclrps;

--
-- inps
--
inps: PROC();
    SYNMODE ps_T = POWERSET LONG (0:20);
    DCL ps1, ps2, ps3, ps4, ps5, ps6 ps_T;
    DCL i LONG;

    print_ps_T: PROC (x ps_T)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
        WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
    END print_ps_T;

    ps1 := [3:8, 10:13, 20];
    ps2 := ps1;
    ps4 := [];
    ps5 := ps4;
    ps6 := [0:20];

    DO FOR i IN ps6;
        IF i IN ps2 THEN
--            WRITETEXT(stdout, "inps: %C is in %C.%/", i, ps2);
	    WRITETEXT (stdout, "inps: %C is in ", i);
	    print_ps_T (ps2);
	    WRITETEXT (stdout, ".%/");
            ps3 := [i];
            ps4 := ps4 OR ps3;
        ELSE
--            WRITETEXT(stdout, "inps: %C is not in %C.%/", i, ps2);
	    WRITETEXT (stdout, "inps: %C is not in ", i);
	    print_ps_T (ps2);
	    WRITETEXT (stdout, ".%/");
            ps3 := [i];
            ps5 := ps5 OR ps3;
        FI;
    OD;
    IF (ps4 = ps1) AND (ps5 = (ps6 AND (NOT ps1))) THEN
        WRITETEXT(stdout, "inps: OK.%/");
    ELSE
        WRITETEXT(stdout, "inps: FAIL.%/");
    FI;
END inps;

--
-- leps
--
leps: PROC();
    SYNMODE ps_T = POWERSET INT (-4:13);
    DCL ps1, ps2, ps3, ps4 ps_T;

    print_ps_T: PROC (x ps_T)

      DCL i INT;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
        WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
    END print_ps_T;

    ps1 := [-4:13];     -- reference
    ps2 := [];          -- nothing
    ps3 := [-4:13];     -- all
    ps4 := [-1:10];     -- sub
    -- checks
--    WRITETEXT(stdout, "leps: %C <= %C -> %C.%/", ps2, ps1, (ps2 <= ps1));
    WRITETEXT (stdout, "leps: ");
    print_ps_T (ps2);
    WRITETEXT (stdout, " <= ");
    print_ps_T (ps1);
    WRITETEXT (stdout, " -> %C.%/", (ps2 <= ps1));
--    WRITETEXT(stdout, "leps: %C <= %C -> %C.%/", ps3, ps1, (ps3 <= ps1));
    WRITETEXT (stdout, "leps: ");
    print_ps_T (ps3);
    WRITETEXT (stdout, " <= ");
    print_ps_T (ps1);
    WRITETEXT (stdout, " -> %C.%/", (ps3 <= ps1));
--    WRITETEXT(stdout, "leps: %C <= %C -> %C.%/", ps4, ps1, (ps4 <= ps1));
    WRITETEXT (stdout, "leps: ");
    print_ps_T (ps4);
    WRITETEXT (stdout, " <= ");
    print_ps_T (ps1);
    WRITETEXT (stdout, " -> %C.%/", (ps4 <= ps1));
    a.l := -1;
    a.ps := [0:14, 17:30];
    b.l := 0;
    b.ps := [0:14, 17:31];
--    WRITETEXT(stdout, "leps: %C <= %C -> %C.%/", a.ps, b.ps, (a.ps <= b.ps));
    WRITETEXT (stdout, "leps: ");
    print_ps4b_T (a.ps);
    WRITETEXT (stdout, " <= ");
    print_ps4b_T (b.ps);
    WRITETEXT (stdout, " -> %C.%/", (a.ps <= b.ps));
    a.l := -1;
    a.ps := [0:14, 17:31];
    b.l := 0;
    b.ps := [0:14, 17:31];
--    WRITETEXT(stdout, "leps: %C <= %C -> %C.%/", a.ps, b.ps, (a.ps <= b.ps));
    WRITETEXT (stdout, "leps: ");
    print_ps4b_T (a.ps);
    WRITETEXT (stdout, " <= ");
    print_ps4b_T (b.ps);
    WRITETEXT (stdout, " -> %C.%/", (a.ps <= b.ps));
END leps;

--
-- ltps
--
ltps: PROC();
    SYNMODE ps_T = POWERSET INT (-4:13);
    DCL ps1, ps2, ps3, ps4 ps_T;

    print_ps_T: PROC (x ps_T)

      DCL i INT;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
        WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
    END print_ps_T;

    ps1 := [-4:13];     -- reference
    ps2 := [];          -- nothing
    ps3 := [-4:13];     -- all
    ps4 := [-1:10];     -- sub

    -- checks
--    WRITETEXT(stdout, "ltps: %C < %C -> %C.%/", ps2, ps1, (ps2 < ps1));
    WRITETEXT (stdout, "ltps: ");
    print_ps_T (ps2);
    WRITETEXT (stdout, " < ");
    print_ps_T (ps1);
    WRITETEXT (stdout, " -> %C.%/", (ps2 < ps1));
--    WRITETEXT(stdout, "ltps: %C < %C -> %C.%/", ps3, ps1, (ps3 < ps1));
    WRITETEXT (stdout, "ltps: ");
    print_ps_T (ps3);
    WRITETEXT (stdout, " < ");
    print_ps_T (ps1);
    WRITETEXT (stdout, " -> %C.%/", (ps3 < ps1));
--    WRITETEXT(stdout, "ltps: %C < %C -> %C.%/", ps4, ps1, (ps4 < ps1));
    WRITETEXT (stdout, "ltps: ");
    print_ps_T (ps4);
    WRITETEXT (stdout, " < ");
    print_ps_T (ps1);
    WRITETEXT (stdout, " -> %C.%/", (ps4 < ps1));
    a.l := -1;
    a.ps := [0:14, 17:30];
    b.l := 0;
    b.ps := [0:14, 17:31];
--    WRITETEXT(stdout, "ltps: %C < %C -> %C.%/", a.ps, b.ps, (a.ps < b.ps));
    WRITETEXT (stdout, "ltps: ");
    print_ps4b_T (a.ps);
    WRITETEXT (stdout, " < ");
    print_ps4b_T (b.ps);
    WRITETEXT (stdout, " -> %C.%/", (a.ps < b.ps));
    a.l := -1;
    a.ps := [0:14, 17:31];
    b.l := 0;
    b.ps := [0:14, 17:31];
--    WRITETEXT(stdout, "ltps: %C < %C -> %C.%/", a.ps, b.ps, (a.ps < b.ps));
    WRITETEXT (stdout, "ltps: ");
    print_ps4b_T (a.ps);
    WRITETEXT (stdout, " < ");
    print_ps4b_T (b.ps);
    WRITETEXT (stdout, " -> %C.%/", (a.ps < b.ps));
END ltps;

--
-- notps
--
notps: PROC();
    SYNMODE ps_T = POWERSET LONG (-5:9);
    DCL ps1, ps2 ps_T;

    print_ps_T: PROC (x ps_T)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
        WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
    END print_ps_T;

    -- sub
    ps1 := [-1:4, 7:8];
    ps2 := [-5:-2, 5:6, 9];
--    WRITETEXT(stdout, "notps: %C = NOT %C -> %C.%/", ps2, ps1, (ps2 = NOT ps1));
    WRITETEXT (stdout, "notps: ");
    print_ps_T (ps2);
    WRITETEXT (stdout, " = NOT ");
    print_ps_T (ps1);
    WRITETEXT (stdout, " -> %C.%/", (ps2 = NOT ps1));
    -- enire
    ps1 := [];
    ps2 := [-5:9];
--    WRITETEXT(stdout, "notps: %C = NOT %C -> %C.%/", ps2, ps1, (ps2 = NOT ps1));
    WRITETEXT (stdout, "notps: ");
    print_ps_T (ps2);
    WRITETEXT (stdout, " = NOT ");
    print_ps_T (ps1);
    WRITETEXT (stdout, " -> %C.%/", (ps2 = NOT ps1));
--    WRITETEXT(stdout, "notps: %C = NOT %C -> %C.%/", ps1, ps2, (ps1 = NOT ps2));
    WRITETEXT (stdout, "notps: ");
    print_ps_T (ps1);
    WRITETEXT (stdout, " = NOT ");
    print_ps_T (ps2);
    WRITETEXT (stdout, " -> %C.%/", (ps1 = NOT ps2));

    -- not a useful testcase...
    a.l := 0;
    a.ps := [];
--    WRITETEXT(stdout, "notps: NOT %C -> %C; l = %C.%/", a.ps, (NOT a.ps), a.l);
    WRITETEXT (stdout, "notps: NOT ");
    print_ps4b_T (a.ps);
    WRITETEXT (stdout, " -> ");
    print_ps4b_T (NOT a.ps);
    WRITETEXT (stdout, "; l = %C.%/", a.l);
END notps;

--
-- orps
--
orps: PROC ()

   SYNMODE m_ps3 = POWERSET LONG (0:3);
   SYNMODE m_ps8 = POWERSET LONG (0:7);
   SYNMODE m_ps13 = POWERSET LONG (0:12);
   SYNMODE m_ps48 = POWERSET LONG (0:47);

   DCL ps3_1 m_ps3 := [ 0:1 ];
   DCL ps3_2 m_ps3 := [ 2:3 ];
   DCL ps3_3 m_ps3;

   DCL ps8_1 m_ps8 := [ 2, 3, 4, 5 ];
   DCL ps8_2 m_ps8 := [ 0, 1, 6, 7 ];
   DCL ps8_3 m_ps8;

   DCL ps13_1 m_ps13 := [ 0:12 ];
   DCL ps13_2 m_ps13 := [ 0, 2, 4, 6, 8, 10, 12 ];
   DCL ps13_3 m_ps13;

   DCL ps48_1 m_ps48 := [ 2:45 ];
   DCL ps48_2 m_ps48 := [ 0:1, 46:47 ];
   DCL ps48_3 m_ps48;

   print_ps3: PROC (x m_ps3)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
         WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
   END print_ps3;

   print_ps8: PROC (x m_ps8)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
         WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
   END print_ps8;

   print_ps13: PROC (x m_ps13)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
         WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
   END print_ps13;

   print_ps48: PROC (x m_ps48)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
         WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
   END print_ps48;

   ps3_3 := ps3_1 OR ps3_2;
---   WRITETEXT (stdout, "orps: %C OR %C := %C%/", ps3_1, ps3_2, ps3_3);
   WRITETEXT (stdout, "orps: ");
   print_ps3 (ps3_1);
   WRITETEXT (stdout, " OR ");
   print_ps3 (ps3_2);
   WRITETEXT (stdout, " := ");
   print_ps3 (ps3_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("orps: dump of result = ", ADDR (ps3_3), SIZE (ps3_3));

   ps8_3 := ps8_1 OR ps8_2;
--   WRITETEXT (stdout, "orps: %C OR %C := %C%/", ps8_1, ps8_2, ps8_3);
   WRITETEXT (stdout, "orps: ");
   print_ps8 (ps8_1);
   WRITETEXT (stdout, " OR ");
   print_ps8 (ps8_2);
   WRITETEXT (stdout, " := ");
   print_ps8 (ps8_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("orps: dump of result = ", ADDR (ps8_3), SIZE (ps8_3));

   ps13_3 := ps13_1 OR ps13_2;
--   WRITETEXT (stdout, "orps: %C OR %C := %C%/", ps13_1, ps13_2, ps13_3);
   WRITETEXT (stdout, "orps: ");
   print_ps13 (ps13_1);
   WRITETEXT (stdout, " OR ");
   print_ps13 (ps13_2);
   WRITETEXT (stdout, " := ");
   print_ps13 (ps13_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("orps: dump of result = ", ADDR (ps13_3), SIZE (ps13_3));
   ps13_1 := [0:12];
   ps13_2 := [];
   ps13_3 := ps13_1 OR ps13_2;
--   WRITETEXT (stdout, "orps: %C OR %C := %C%/", ps13_1, ps13_2, ps13_3);
   WRITETEXT (stdout, "orps: ");
   print_ps13 (ps13_1);
   WRITETEXT (stdout, " OR ");
   print_ps13 (ps13_2);
   WRITETEXT (stdout, " := ");
   print_ps13 (ps13_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("orps: dump of result = ", ADDR (ps13_3), SIZE (ps13_3));

   ps48_3 := ps48_1 OR ps48_2;
--   WRITETEXT (stdout, "orps: %C OR %C := %C%/", ps48_1, ps48_2, ps48_3);
   WRITETEXT (stdout, "orps: ");
   print_ps48 (ps48_1);
   WRITETEXT (stdout, " OR ");
   print_ps48 (ps48_2);
   WRITETEXT (stdout, " := ");
   print_ps48 (ps48_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("orps: dump of result = ", ADDR (ps48_3), SIZE (ps48_3));

   ps48_1 := [0:47];
   ps48_2 := [];
   ps48_3 := ps48_1 OR ps48_2;
--   WRITETEXT (stdout, "orps: %C OR %C := %C%/", ps48_1, ps48_2, ps48_3);
   WRITETEXT (stdout, "orps: ");
   print_ps48 (ps48_1);
   WRITETEXT (stdout, " OR ");
   print_ps48 (ps48_2);
   WRITETEXT (stdout, " := ");
   print_ps48 (ps48_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("orps: dump of result = ", ADDR (ps48_3), SIZE (ps48_3));

END orps;

--
-- xorps
--
xorps: PROC ()

   SYNMODE m_ps3 = POWERSET LONG (0:3);
   SYNMODE m_ps8 = POWERSET LONG (0:7);
   SYNMODE m_ps13 = POWERSET LONG (0:12);
   SYNMODE m_ps48 = POWERSET LONG (0:47);

   DCL ps3_1 m_ps3 := [ 0:1 ];
   DCL ps3_2 m_ps3 := [ 1:3 ];
   DCL ps3_3 m_ps3;

   DCL ps8_1 m_ps8 := [ 2, 3, 4, 5 ];
   DCL ps8_2 m_ps8 := [ 0, 1, 6, 7 ];
   DCL ps8_3 m_ps8;

   DCL ps13_1 m_ps13 := [ 0:12 ];
   DCL ps13_2 m_ps13 := [ 0, 2, 4, 6, 8, 10, 12 ];
   DCL ps13_3 m_ps13;

   DCL ps48_1 m_ps48 := [ 2:45 ];
   DCL ps48_2 m_ps48 := [ 0:1, 46:47 ];
   DCL ps48_3 m_ps48;

   print_ps3: PROC (x m_ps3)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
         WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
   END print_ps3;

   print_ps8: PROC (x m_ps8)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
         WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
   END print_ps8;

   print_ps13: PROC (x m_ps13)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
         WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
   END print_ps13;

   print_ps48: PROC (x m_ps48)

      DCL i LONG;

      WRITETEXT (stdout, "[ ");
      DO FOR i IN x;
         WRITETEXT (stdout, "%C ", i);
      OD;
      WRITETEXT (stdout, "]");
   END print_ps48;

   ps3_3 := ps3_1 XOR ps3_2;
--   WRITETEXT (stdout, "xorps: %C XOR %C := %C%/", ps3_1, ps3_2, ps3_3);
   WRITETEXT (stdout, "xorps: ");
   print_ps3 (ps3_1);
   WRITETEXT (stdout, " XOR ");
   print_ps3 (ps3_2);
   WRITETEXT (stdout, " := ");
   print_ps3 (ps3_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("orps: dump of result = ", ADDR (ps3_3), SIZE (ps3_3));

   ps8_3 := ps8_1 XOR ps8_2;
--   WRITETEXT (stdout, "xorps: %C XOR %C := %C%/", ps8_1, ps8_2, ps8_3);
   WRITETEXT (stdout, "xorps: ");
   print_ps8 (ps8_1);
   WRITETEXT (stdout, " XOR ");
   print_ps8 (ps8_2);
   WRITETEXT (stdout, " := ");
   print_ps8 (ps8_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("xorps: dump of result = ", ADDR (ps8_3), SIZE (ps8_3));

   ps13_3 := ps13_1 XOR ps13_2;
--   WRITETEXT (stdout, "xorps: %C XOR %C := %C%/", ps13_1, ps13_2, ps13_3);
   WRITETEXT (stdout, "xorps: ");
   print_ps13 (ps13_1);
   WRITETEXT (stdout, " XOR ");
   print_ps13 (ps13_2);
   WRITETEXT (stdout, " := ");
   print_ps13 (ps13_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("xorps: dump of result = ", ADDR (ps13_3), SIZE (ps13_3));
   ps13_1 := [0:12];
   ps13_2 := [0:12];
   ps13_3 := ps13_1 XOR ps13_2;
--   WRITETEXT (stdout, "xorps: %C XOR %C := %C%/", ps13_1, ps13_2, ps13_3);
   WRITETEXT (stdout, "xorps: ");
   print_ps13 (ps13_1);
   WRITETEXT (stdout, " XOR ");
   print_ps13 (ps13_2);
   WRITETEXT (stdout, " := ");
   print_ps13 (ps13_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("xorps: dump of result = ", ADDR (ps13_3), SIZE (ps13_3));

   ps48_3 := ps48_1 XOR ps48_2;
--   WRITETEXT (stdout, "xorps: %C XOR %C := %C%/", ps48_1, ps48_2, ps48_3);
   WRITETEXT (stdout, "xorps: ");
   print_ps48 (ps48_1);
   WRITETEXT (stdout, " XOR ");
   print_ps48 (ps48_2);
   WRITETEXT (stdout, " := ");
   print_ps48 (ps48_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("xorps: dump of result = ", ADDR (ps48_3), SIZE (ps48_3));

   ps48_1 := [0:47];
   ps48_2 := [0:47];
   ps48_3 := ps48_1 XOR ps48_2;
--   WRITETEXT (stdout, "xorps: %C XOR %C := %C%/", ps48_1, ps48_2, ps48_3);
   WRITETEXT (stdout, "xorps: ");
   print_ps48 (ps48_1);
   WRITETEXT (stdout, " XOR ");
   print_ps48 (ps48_2);
   WRITETEXT (stdout, " := ");
   print_ps48 (ps48_3);
   WRITETEXT (stdout, "%/");
   dump_bitstring ("xorps: dump of result = ", ADDR (ps48_3), SIZE (ps48_3));

END xorps;

--
-- concatstr
--
concatstr: PROC();
    DCL str5 CHAR(5);
    DCL str2 CHAR(2);
    DCL str0 CHAR(0);
    SYNMODE str_T = STRUCT (
        str4 CHAR(4),
        l long);
    DCL strl str_T;
    strl.l := 0;
    strl.str4 := "ab" // "cd";
    WRITETEXT(stdout, "concatstr: ab // cd = '%C'; l = %C.%/", strl.str4, strl.l);

    str5 := "abcde";
    str2 := "fg";
    str0 := "";
    WRITETEXT(stdout, "concatstr: '%C' // '%C' = '%C'.%/", str5, str2, (str5 // str2));
    WRITETEXT(stdout, "concatstr: '%C' // '%C' = '%C'.%/", str5, str0, (str5 // str0));
    WRITETEXT(stdout, "concatstr: '%C' // '%C' = '%C'.%/", str0, str2, (str0 // str2));
END concatstr;

--
-- eqstr
--
eqstr: PROC();
    DCL str5_1, str5_2 CHAR(5);
    DCL str6 CHAR (6);
    DCL str2 CHAR(2);
    DCL str0_1, str0_2 CHAR(0);

    str5_1 := "zyxwv";
    str5_2 := "zyxwv";
    str6 := "zyxwv ";
    str0_1 := "";
    str0_2 := "";
    str2 := "ab";
    WRITETEXT(stdout, "eqstr: '%C' = '%C' -> %C.%/", str5_1, str5_2, (str5_1 = str5_2));
    WRITETEXT(stdout, "eqstr: '%C' = '%C' -> %C.%/", str0_1, str0_2, (str0_1 = str0_2));
    WRITETEXT(stdout, "eqstr: '%C' = '%C' -> %C.%/", str5_1, str2, (str5_1 = str2));
    WRITETEXT(stdout, "eqstr: '%C' = '%C' -> %C.%/", str5_1, str0_1, (str5_1 = str0_1));
    WRITETEXT(stdout, "eqstr: '%C' = '%C' -> %C.%/", str5_1, str6, (str5_1 = str6));

    sa.s41 := "abcd";
    sa.s42 := "efgh";
    sa.s8 := "abcdefgh";
    WRITETEXT(stdout, "eqstr: '%C' = '%C' -> %C.%/", sa.s41, sa.s8, (sa.s41 = sa.s8));
END eqstr;

--
-- gestr
--
gestr: PROC();
    DCL str5 CHAR (5);
    DCL str3 CHAR (3);
    DCL str0 CHAR (0);

    str5 := "abcde";
    str3 := "bde";
    str0 := "";
    
    WRITETEXT(stdout, "gestr: '%C' >= '%C' -> %C.%/", str5, str3, (str5 >= str3));
    WRITETEXT(stdout, "gestr: '%C' >= '%C' -> %C.%/", str5, str0, (str5 >= str0));
    WRITETEXT(stdout, "gestr: '%C' >= '%C' -> %C.%/", str3, str5, (str3 >= str5));
    WRITETEXT(stdout, "gestr: '%C' >= '%C' -> %C.%/", str3, str3, (str3 >= str3));

    sa.s41 := "abcd";
    sa.s42 := "efgh";
    sa.s8 := "abcdffgh";
    WRITETEXT(stdout, "gestr: '%C' >= '%C' -> %C.%/", sa.s8, sa.s41, (sa.s8 >= sa.s41));
END gestr;

--
-- gtstr
--
gtstr: PROC();
    DCL str5 CHAR (5);
    DCL str3 CHAR (3);
    DCL str0 CHAR (0);

    str5 := "abcde";
    str3 := "bde";
    str0 := "";
    
    WRITETEXT(stdout, "gtstr: '%C' > '%C' -> %C.%/", str5, str3, (str5 > str3));
    WRITETEXT(stdout, "gtstr: '%C' > '%C' -> %C.%/", str5, str0, (str5 > str0));
    WRITETEXT(stdout, "gtstr: '%C' > '%C' -> %C.%/", str3, str5, (str3 > str5));
    WRITETEXT(stdout, "gtstr: '%C' > '%C' -> %C.%/", str3, str3, (str3 > str3));

    sa.s41 := "abcd";
    sa.s42 := "efgh";
    sa.s8 := "abcdefgh";
    WRITETEXT(stdout, "gtstr: '%C' > '%C' -> %C.%/", sa.s8, sa.s41, (sa.s8 > sa.s41));
END gtstr;

--
-- lestr
--
lestr: PROC();
    DCL str5 CHAR (5);
    DCL str3 CHAR (3);
    DCL str0 CHAR (0);

    str5 := "abcde";
    str3 := "bde";
    str0 := "";
    
    WRITETEXT(stdout, "lestr: '%C' <= '%C' -> %C.%/", str5, str3, (str5 <= str3));
    WRITETEXT(stdout, "lestr: '%C' <= '%C' -> %C.%/", str5, str0, (str5 <= str0));
    WRITETEXT(stdout, "lestr: '%C' <= '%C' -> %C.%/", str3, str5, (str3 <= str5));
    WRITETEXT(stdout, "lestr: '%C' <= '%C' -> %C.%/", str3, str3, (str3 <= str3));
    sa.s41 := "abcd";
    sa.s42 := "efgh";
    sa.s8 := "abcdefgh";
    WRITETEXT(stdout, "lestr: '%C' <= '%C' -> %C.%/", sa.s41, sa.s8, (sa.s41 <= sa.s8));

END lestr;

--
-- ltstr
--
ltstr: PROC();
    DCL str5 CHAR (5);
    DCL str3 CHAR (3);
    DCL str0 CHAR (0);

    str5 := "abcde";
    str3 := "bde";
    str0 := "";
    
    WRITETEXT(stdout, "ltstr: '%C' < '%C' -> %C.%/", str5, str3, (str5 < str3));
    WRITETEXT(stdout, "ltstr: '%C' < '%C' -> %C.%/", str5, str0, (str5 < str0));
    WRITETEXT(stdout, "ltstr: '%C' < '%C' -> %C.%/", str3, str5, (str3 < str5));
    WRITETEXT(stdout, "ltstr: '%C' < '%C' -> %C.%/", str3, str3, (str3 < str3));
    sa.s41 := "abcd";
    sa.s42 := "efgh";
    sa.s8 := "abcdefgh";
    WRITETEXT(stdout, "ltstr: '%C' < '%C' -> %C.%/", sa.s41, sa.s8, (sa.s41 < sa
.s8));

END ltstr;

--
-- nestr
--
nestr: PROC();
    DCL str5_1, str5_2 CHAR (5);
    DCL str3 CHAR(3);
    DCL str0 CHAR (0);

    str5_1 := "abcde";
    str5_2 := "opqrs";
    str3 := "abc";
    str0 := "";
    WRITETEXT(stdout, "nestr: '%C' /= '%C' -> %C.%/", str5_1, str5_2, (str5_1 /= str5_2));
    WRITETEXT(stdout, "nestr: '%C' /= '%C' -> %C.%/", str5_1, str3, (str5_1 /= str3));
    WRITETEXT(stdout, "nestr: '%C' /= '%C' -> %C.%/", str3, str3, (str3 /= str3));
    WRITETEXT(stdout, "nestr: '%C' /= '%C' -> %C.%/", str5_1, str0, (str5_1 /= str0));
    sa.s41 := "abcd";
    sa.s42 := "efgh";
    sa.s8 := "abcdefgh";
    WRITETEXT(stdout, "nestr: '%C' /= '%C' -> %C.%/", sa.s41, sa.s8, (sa.s41 /= sa.s8));
END nestr;

--
-- start the tests
--
allgmem ();
allmem ();
andps ();
cardps ();
ffsetps ();
flsetps ();
concatps ();
sliceps ();
diffps ();
eqps ();
neps ();
setbitps();
ffsetclrps();
flsetclrps();
inps();
leps();
ltps();
notps();
orps();
xorps();
concatstr();
eqstr();
gestr();
gtstr();
lestr();
ltstr();
nestr();

END chilllib;
