/* CHILL compiler regression test file
 Copyright (C) 1992, 1993 Free Software Foundation, Inc.
 
 This file is part of GNU CC.

 GNU CC is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 GNU CC is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GNU CC; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

rngfail: MODULE

-- this module is supposed to check all possibilities for
-- raising a rangefail exception during runtime

-- global used variables and functions 
DCL what_test CHAR (100) VARYING;

success: PROC ();
   WRITETEXT (stdout, what_test // " PASSED%/");
END success;

failed: PROC ();
   WRITETEXT (stdout, what_test // " FAILED%/");
END failed;

-- test chapter 4.2.6 string elements
z200_4_2_6: PROC ();
   DCL fstr CHAR (5) := "Moser";
   DCL vstr CHAR (30) VARYING := "Wilfried";
   DCL i INT;
   DCL c CHAR;

   what_test := "4.2.6 String elements fixed-length lower-bound:";
   i := -1;
   c := fstr(i) ON (rangefail): success (); GOTO fixed_upper; END;
   failed ();
fixed_upper:
   what_test := "4.2.6 String elements fixed-length upper-bound:";
   i := SIZE (fstr);
   c := fstr(i) ON (rangefail): success (); GOTO varying_lower; END;
   failed ();
varying_lower:
   what_test := "4.2.6 String elements varying-length lower-bound:";
   i := -1;
   c := vstr(i) ON (rangefail): success (); GOTO varying_upper; END;
   failed ();
varying_upper:
   what_test := "4.2.6 String elements varying-length upper-bound:";
   i := length (vstr) + 1;
   c := vstr (i) ON (rangefail): success (); RETURN; END;
   failed ();
END z200_4_2_6;

-- test chapter 4.2.7 String slices
z200_4_2_7: PROC ();
   DCL fstr CHAR (5) := "Moser";
   DCL vstr CHAR (30) VARYING := "Wilfried";
   DCL i, j INT;
   DCL res CHAR(20);

   what_test := "4.2.7 String slices left element lower-bound:";
   i := -1;
   j := 5;
   res(i:j) := fstr(i:j) ON (rangefail): success (); GOTO left_upper; END;
   failed ();
left_upper:
   what_test := "4.2.7 String slices left element upper-bound:";
   i := 3;
   j := 6;
   res(i:j) := fstr(i:j) ON (rangefail): success (); GOTO right_lower; END;
   failed ();
right_lower:
   what_test := "4.2.7 String slices right element lower-bound:";
   i := 3;
   j := -1;
   res(i:j) := fstr (i:j) ON (rangefail): success (); GOTO right_upper; END;
   failed ();
right_upper:
   what_test := "4.2.7 String slices right element upper-bound:";
   i := 3;
   j := 7;
   res(i:j) := fstr (i:j) ON (rangefail): success (); GOTO start_and_size; END;
   failed ();
start_and_size:
   what_test := "4.2.7 String slices right element start and size:";
   i := 5;
   res (i UP 10) := fstr(i UP 10) ON (rangefail): success (); GOTO fred; END;
   failed ();

fred:
   what_test := "4.2.7 Varying String slices left element lower-bound:";
   i := -1;
   j := 5;
   res(i:j) := vstr(i:j) ON (rangefail): success (); GOTO left_upper_v; END;
   failed ();
left_upper_v:
   what_test := "4.2.7 Varying String slices left element upper-bound:";
   i := 8;
   j := 11;
   res(i:j) := vstr(i:j) ON (rangefail): success (); GOTO right_lower_v; END;
   failed ();
right_lower_v:
   what_test := "4.2.7 Varying String slices right element lower-bound:";
   i := 3;
   j := -1;
   res(i:j) := vstr (i:j) ON (rangefail): success (); GOTO right_upper_v; END;
   failed ();
right_upper_v:
   what_test := "4.2.7 Varying String slices right element upper-bound:";
   i := 3;
   j := 9;
   res(i:j) := vstr (i:j) ON (rangefail): success (); GOTO start_and_size_v; END;
   failed ();
start_and_size_v:
   what_test := "4.2.7 Varying String slices right element start and size:";
   i := 5;
   res (i UP 10) := fstr(i UP 10) ON (rangefail): success (); GOTO end_varying; END;
   failed ();

end_varying:
  i := 2;
END z200_4_2_7;

-- test chapter 4.2.7 Bit String slices
z200_4_2_7_bit: PROC ();
   DCL fstr BIT (5) := b'10100';
   DCL i, j INT;
   DCL res  BIT (20);
/*
   what_test := "4.2.7 Bit String slices left element lower-bound:";
   i := -1;
   j := 5;
   res(i:j) := fstr(i:j) ON (rangefail): success (); GOTO left_upper; END;
   failed ();
left_upper:
   what_test := "4.2.7 Bit String slices left element upper-bound:";
   i := 3;
   j := 6;
   res(i:j) := fstr(i:j) ON (rangefail): success (); GOTO right_lower; END;
   failed ();
right_lower:
   what_test := "4.2.7 Bit String slices right element lower-bound:";
   i := 3;
   j := -1;
   res(i:j) := fstr (i:j) ON (rangefail): success (); GOTO right_upper; END;
   failed ();
right_upper:
   what_test := "4.2.7 Bit String slices right element upper-bound:";
   i := 3;
   j := 7;
   res(i:j) := fstr (i:j) ON (rangefail): success (); GOTO start_and_size; END;
   failed ();
start_and_size:
   what_test := "4.2.7 Bit String slices right element start and size:";
   i := 5;
   res (i UP 2) := fstr(i UP 2) ON (rangefail): success (); RETURN; END;
   failed ();
*/
END z200_4_2_7_bit;

-- test chapter 4.2.8 Array elements
z200_4_2_8: PROC ();
   DCL farr ARRAY (1:5) INT := [ 1, 2, 3, 4, 5];
   DCL varr ARRAY (1:30) VARYING INT := [ 1, 2, 3, 4, 5];
   DCL i INT;
   DCL c INT;

   what_test := "4.2.6 Array elements fixed-length lower-bound:";
   i := -1;
   c := farr(i) ON (rangefail): success (); GOTO fixed_upper; END;
   failed ();
fixed_upper:
   what_test := "4.2.6 Array elements fixed-length upper-bound:";
   i := SIZE (farr);
   c := farr(i) ON (rangefail): success (); GOTO varying_lower; END;
   failed ();
varying_lower:
   what_test := "4.2.6 Array elements varying-length lower-bound:";
   i := -1;
   c := varr(i) ON (rangefail): success (); GOTO varying_upper; END;
   failed ();
varying_upper:
   what_test := "4.2.6 Array elements varying-length upper-bound:";
   i := length (varr) + 1;
   c := varr (i) ON (rangefail): success (); RETURN; END;
   failed ();
END z200_4_2_8;

-- test chapter 4.2.9 Array slices
z200_4_2_9: PROC ();
   DCL farr ARRAY (1:5) INT := [ 1, 2, 3, 4, 5];
   DCL i, j INT;
   DCL res ARRAY (1:20) INT;

   what_test := "4.2.7 Array slices left element lower-bound:";
   i := -1;
   j := 5;
   res(i:j) := farr(i:j) ON (rangefail): success (); GOTO left_upper; END;
   failed ();
left_upper:
   what_test := "4.2.7 Array slices left element upper-bound:";
   i := 3;
   j := 6;
   res(i:j) := farr(i:j) ON (rangefail): success (); GOTO right_lower; END;
   failed ();
right_lower:
   what_test := "4.2.7 Array slices right element lower-bound:";
   i := 3;
   j := -1;
   res(i:j) := farr (i:j) ON (rangefail): success (); GOTO right_upper; END;
   failed ();
right_upper:
   what_test := "4.2.7 Array slices right element upper-bound:";
   i := 3;
   j := 7;
   res(i:j) := farr (i:j) ON (rangefail): success (); GOTO start_and_size; END;
   failed ();
start_and_size:
   what_test := "4.2.7 Array slices right element start and size:";
   i := 5;
   res (i UP 10) := farr(i UP 10) ON (rangefail): success (); RETURN; END;
   failed ();
END z200_4_2_9;

-- test chapter 6.5.2 For control
z200_6_5_2: PROC ();
   DCL i INT := -1;
   DCL j INT := 5;
   DCL k INT := 4;
   DCL l INT;
   DCL a INT := 5;

   what_test := "6.5.2 For Control step value negative:";
   DO FOR l := j BY i TO k;
      a := a + 10;
   OD ON (rangefail): success(); RETURN; END;
   failed ();
END z200_6_5_2;

-- test chapter 6.20.3 chill value builtins
z200_6_20_3: PROC ();
   DCL bl BYTE := LOWER(BYTE);
   DCL bu BYTE := UPPER(BYTE);
   DCL b BYTE;
   DCL il INT := LOWER(INT);
   DCL iu INT := UPPER (INT);
   DCL i INT;
   DCL ll LONG := LOWER(LONG);
   DCL lu LONG := UPPER(LONG);
   DCL l LONG;
   DCL ubl UBYTE := LOWER(UBYTE);
   DCL ubu UBYTE := UPPER(UBYTE);
   DCL ub UBYTE;
   DCL uil UINT := LOWER(UINT);
   DCL uiu UINT := UPPER(UINT);
   DCL ui UINT;
   DCL ull ULONG := LOWER(ULONG);
   DCL ulu ULONG := UPPER(ULONG);
   DCL ul ULONG;
   NEWMODE intrange = INT(10:200);
   DCL riu intrange := UPPER(intrange);
   DCL ril intrange := LOWER(intrange);
   DCL ri intrange;
   NEWMODE ps_mode = POWERSET intrange;
   DCL ps ps_mode := [];
   NEWMODE anenum = SET (ax, bx, cx);
   DCL enumu anenum := UPPER (anenum);
   DCL enuml anenum := LOWER (anenum);
   DCL enum anenum;
   NEWMODE charrange = CHAR('A':'Z');
   DCL cru charrange := UPPER (charrange);
   DCL crl charrange := LOWER (charrange);
   DCL cr charrange;
   DCL bou BOOL := UPPER(BOOL);
   DCL bol BOOL := LOWER(BOOL);
   DCL bo BOOL;

   what_test := "6.20.3 PRED (LOWER (BYTE)):";
   b := PRED (bl) ON (rangefail): success(); GOTO succ_upper_byte; END;
   failed ();
succ_upper_byte:
   what_test := "6.20.3 SUCC (UPPER (BYTE)):";
   b := SUCC (bu) ON (rangefail): success (); GOTO pred_lower_int; END;
   failed ();
pred_lower_int:
   what_test := "6.20.3 PRED (LOWER (INT)):";
   i := PRED (il) ON (rangefail): success(); GOTO succ_upper_int; END;
   failed ();
succ_upper_int:
   what_test := "6.20.3 SUCC (UPPER (INT)):";
   i := SUCC (iu) ON (rangefail): success (); GOTO pred_lower_long; END;
   failed ();
pred_lower_long:
   what_test := "6.20.3 PRED (LOWER (LONG)):";
   l := PRED (ll)
	ON (overflow,rangefail): success(); GOTO succ_upper_long; END;
   failed ();
succ_upper_long:
   what_test := "6.20.3 SUCC (UPPER (LONG)):";
   l := SUCC (lu)
	ON (overflow, rangefail): success (); GOTO pred_lower_ubyte; END;
   failed ();
pred_lower_ubyte:
   what_test := "6.20.3 PRED (LOWER (UBYTE)):";
   ub := PRED (ubl) ON (rangefail): success(); GOTO succ_upper_ubyte; END;
   failed ();
succ_upper_ubyte:
   what_test := "6.20.3 SUCC (UPPER (UBYTE)):";
   ub := SUCC (ubu) ON (rangefail): success (); GOTO pred_lower_uint; END;
   failed ();
pred_lower_uint:
   what_test := "6.20.3 PRED (LOWER (UINT)):";
   ui := PRED (uil) ON (rangefail): success(); GOTO succ_upper_uint; END;
   failed ();
succ_upper_uint:
   what_test := "6.20.3 SUCC (UPPER (UINT)):";
   ui := SUCC (uiu) ON (rangefail): success (); GOTO pred_lower_ulong; END;
   failed ();
pred_lower_ulong:
   what_test := "6.20.3 PRED (LOWER (ULONG)):";
   ul := PRED (ull)
	ON (overflow,rangefail): success(); GOTO succ_upper_ulong; END;
   failed ();
succ_upper_ulong:
   what_test := "6.20.3 SUCC (UPPER (ULONG)):";
   ul := SUCC (ulu)
	ON (overflow,rangefail): success (); GOTO pred_lower_intrange; END;
   failed ();
pred_lower_intrange:
   what_test := "6.20.3 PRED (LOWER (INT (10:200))):";
   ri := PRED (ril) ON (rangefail): success (); GOTO succ_upper_intrange; END;
   failed ();
succ_upper_intrange:
   what_test := "6.20.3 SUCC (UPPER (INT (10:200))):";
   ri := SUCC (riu) ON (rangefail): success (); GOTO pred_lower_enum; END;
   failed ();
pred_lower_enum:
   what_test := "6.20.3 PRED (LOWER (SET (ax, bx, cx))):";
   enum := PRED (enuml) ON (overflow): success (); GOTO succ_upper_enum; END;
   failed ();
succ_upper_enum:
   what_test := "6.20.3 SUCC (UPPER (SET (ax, bx, cx))):";
   enum := SUCC (enumu) ON (overflow): success (); GOTO pred_lower_char_range; END;
   failed ();
pred_lower_char_range:
   what_test := "6.20.3 PRED (LOWER (CHAR ('A':'Z'))):";
   cr := PRED (crl) ON (rangefail): success (); GOTO succ_upper_char_range; END;
   failed ();
succ_upper_char_range:
   what_test := "6.20.3 SUCC (UPPER (CHAR ('A':'Z'))):";
   cr := SUCC (cru) ON (rangefail): success (); GOTO pred_lower_bool; END;
   failed ();
pred_lower_bool:
   what_test := "6.20.3 PRED (LOWER (BOOL)):";
   bo := PRED (bol) ON (overflow): success (); GOTO succ_upper_bool; END;
   failed ();
succ_upper_bool:
   what_test := "6.20.3 SUCC (UPPER (BOOL)):";
   bo := SUCC (bou) ON (overflow): success (); GOTO min_empty_powerset; END;
   failed ();
min_empty_powerset:
   what_test := "6.20.3 MIN on empty powerset:";
   ri := MIN (ps) ON (empty): success (); GOTO max_empty_powerset; END;
   failed ();
max_empty_powerset:
   what_test := "6.20.3 MAX on empty powerset:";
   ri := MAX (ps) ON (empty): success (); RETURN; END;
   failed ();

END z200_6_20_3;

/*
integer to integer assignment:
There are three types of assignment where rangefail check is neccessary:

 a) assign longer INT to shorter INT
 b) assign signed to unsigned
 c) assign unsigned to signed at equal size

                       RHS
         +-------------------------------------------+
LHS      | UBYTE | BYTE  | UINT | INT | ULONG | LONG |
---------+-------+-------+------+-----+-------+------+
UBYTE    |       |  Xb   |  Xa  |  Xa |  Xa   |  Xa  |
---------+-------+-------+------+-----+-------+------+
BYTE     |   Xc  |       |  Xa  |  Xa |  Xa   |  Xa  |
---------+-------+-------+------+-----+-------+------+
UINT     |       |  Xb   |      |  Xb |  Xa   |  Xa  |
---------+-------+-------+------+-----+-------+------+
INT      |       |       |  Xc  |     |  Xa   |  Xa  |
---------+-------+-------+------+-----+-------+------+
ULONG    |       |  Xb   |      |  Xb |       |  Xb  |
---------+-------+-------+------+-----+-------+------+
LONG     |       |       |      |     |  Xc   |      |
---------+-------+-------+------+-----+-------+------+

FIXME: Need SET type test(s) here, tests of parameter passing,
       passing to LOC/INOUT parameters.
*/
int_int_assgn: PROC ();
   DCL b BYTE, ub UBYTE, i INT, ui UINT, l LONG, ul ULONG;

   what_test := "UBYTE := BYTE:";
   b := -10;
   ub := b ON (rangefail): success (); GOTO uint_ubyte; END;
   failed ();
uint_ubyte:
   what_test := "UBYTE := UINT:";
   ui := 300;
   ub := ui ON (rangefail): success (); GOTO int_ubyte; END;
   failed ();
int_ubyte:
   what_test := "UBYTE := INT:";
   i := -300;
   ub := i ON (rangefail): success (); GOTO ulong_ubyte; END;
   failed ();
ulong_ubyte:
   what_test := "UBYTE := ULONG:";
   ul := 100000;
   ub := ul ON (rangefail): success (); GOTO long_ubyte; END;
   failed ();
long_ubyte:
   what_test := "UBYTE := LONG:";
   l := -100000;
   ub := l ON (rangefail): success (); GOTO byte_ubyte; END;
   failed ();
byte_ubyte:
   what_test := "BYTE := UBYTE:";
   ub := 255;
   b := ub ON (rangefail): success (); GOTO uint_byte; END;
   failed ();
uint_byte:
   what_test := "BYTE := UINT:";
   ui := 10000;
   b := ui ON (rangefail): success (); GOTO int_byte; END;
   failed ();
int_byte:
   what_test := "BYTE := INT:";
   i := -10000;
   b := i ON (rangefail): success (); GOTO ulong_byte; END;
   failed ();
ulong_byte:
   what_test := "BYTE := ULONG:";
   ul := 100000;
   b := ul ON (rangefail): success (); GOTO long_byte; END;
   failed ();
long_byte:
   what_test := "BYTE := LONG:";
   l := -100000;
   b := l ON (rangefail): success (); GOTO byte_uint; END;
   failed ();
byte_uint:
   what_test := "UINT := BYTE:";
   b := -100;
   ui := b ON (rangefail): success (); GOTO int_uint; END;
   failed ();
int_uint:
   what_test := "UINT := INT:";
   i := -100;
   ui := i ON (rangefail): success (); GOTO ulong_uint; END;
   failed ();
ulong_uint:
   what_test := "UINT := ULONG:";
   ul := 100000;
   ui := ul ON (rangefail): success (); GOTO long_uint; END;
   failed ();
long_uint:
   what_test := "UINT := LONG:";
   l := -100000;
   ui := l ON (rangefail): success (); GOTO uint_int; END;
   failed ();
uint_int:
   what_test := "INT := UINT:";
   ui := 65535;
   i := ui ON (rangefail): success (); GOTO ulong_int; END;
   failed ();
ulong_int:
   what_test := "INT := ULONG:";
   ul := 100000;
   i := ul ON (rangefail): success (); GOTO long_int; END;
   failed ();
long_int:
   what_test := "INT := LONG:";
   l := -100000;
   i := l ON (rangefail): success (); GOTO byte_ulong; END;
   failed ();
byte_ulong:
   what_test := "ULONG := BYTE:";
   b := -100;
   ul := b ON (rangefail): success (); GOTO int_ulong; END;
   failed ();
int_ulong:
   what_test := "ULONG := INT:";
   i := -10000;
   ul := i ON (rangefail): success (); GOTO long_ulong; END;
   failed ();
long_ulong:
   what_test := "ULONG := LONG:";
   l := -100000;
   ul := l ON (rangefail): success (); GOTO ulong_long; END;
   failed ();
ulong_long:
   what_test := "LONG := ULONG:";
   ul := H'88880000;
   l := ul ON (rangefail): success (); RETURN; END;
   failed ();
END int_int_assgn;

-- test some assignments of ranges
range_assgn: PROC ();
   DCL i INT := 100;
   DCL ir INT(0:10);
   DCL c CHAR := 'a';
   DCL cr CHAR('A':'Z');
   DCL boo BOOL := FALSE;
   DCL boor BOOL(TRUE:TRUE);
   SYNMODE aset = SET (aa, bb, xx, yy, zz);
   SYNMODE setrange = aset(xx:yy);
   DCL s aset := aa;
   DCL sr setrange;

   what_test := "INT(0:10) := INT:";
   ir := i ON (rangefail): success (); GOTO char_range; END;
   failed ();
char_range:
   what_test := "CHAR('A':'Z') := CHAR:";
   cr := c ON (rangefail): success (); GOTO bool_range; END;
   failed ();
bool_range:
   what_test := "BOOL(TRUE:TRUE) := BOOL:";
   boor := boo ON (rangefail): success (); GOTO set_range; END;
   failed ();
set_range:
   what_test := "setrange(xx:yy) := set(aa, bb, xx, yy, zz):";
   sr := s ON (rangefail): success (); RETURN; END;
   failed ();
END range_assgn;

-- test some assignments of ranges
range_assgn_p: PROC ();
   DCL i  INT := 100;
   DCL c  CHAR('A':'Z');
   DCL boo BOOL := FALSE;
   SYNMODE aset = SET (aa, bb, xx, yy, zz);
   SYNMODE setrange = aset(xx:yy);
   DCL s aset := aa;

   pi: PROC (a INT(0:10));
   END pi;

   pc: PROC (a CHAR ('A':'Z') LOC);
   END pc;

   pb: PROC (a BOOL(TRUE:TRUE) INOUT);
   END pb;

   ps: PROC (a setrange);
   END ps;

   what_test := "pi(INT(0:10)) passing INT:";
   pi (i) ON (rangefail): success (); GOTO char_range; END;
   failed ();
char_range:
   what_test := "pc(CHAR('A':'Z') LOC) passing CHAR:";
   pc('a') ON (rangefail): success (); GOTO bool_range; END;
   failed ();
bool_range:
   what_test := "pb(BOOL(TRUE:TRUE) INOUT) passing BOOL:";
   pb (boo) ON (rangefail): success (); GOTO set_range; END;
   failed ();
set_range:
   what_test := "ps (setrange(xx:yy)) passing set(aa, bb, xx, yy, zz):";
   ps (s) ON (rangefail): success (); RETURN; END;
   failed ();
END range_assgn_p;

z200_4_2_6 ();
z200_4_2_7 ();
z200_4_2_7_bit ();
z200_4_2_8 ();
z200_4_2_9 ();
z200_6_5_2 ();
z200_6_20_3 ();
int_int_assgn ();
range_assgn ();
range_assgn_p ();

END rngfail;
