/* CHILL compiler regression test file
 Copyright (C) 1992, 1993 Free Software Foundation, Inc.
 
 This file is part of GNU CC.

 GNU CC is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 GNU CC is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GNU CC; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

sortsearch: MODULE

  SYN LenTab = 90;

  NEWMODE m_item  = CHARS (40);
  NEWMODE m_table = ARRAY ( 1 : LenTab ) m_item;

  DCL a m_table;
  DCL key m_item;
  DCL i INT;

-- this is necessary because string comparisons don't work
isless: PROC( a, b READ m_item LOC ) RETURNS ( BOOL );
  DCL i INT;
  DCL ca, cb CHAR;
  DO FOR i := 0 TO LENGTH(a) - 1;
-- we need this because we can't compare sublocations
    ca := a(i); cb := b(i);
    IF ca < cb THEN RETURN TRUE; FI;
    IF ca > cb THEN RETURN FALSE; FI;
  OD;
  RETURN FALSE;
END isless;

dump: PROC( a READ m_table LOC, lo, hi INT );
  DCL i INT;
  DO FOR i := lo TO hi;
-- no string formating
    WRITETEXT( stdout, "%C2: " // a(i) // "%/", i );
  OD;
END dump;

--
-- binary search
--
binsearch: PROC ( a READ m_table LOC, 
		  x READ m_item LOC ) RETURNS ( INT );
  DCL i, l, m, r INT;
  -- Note: compiler doesn't like UPPER(a)
  l := 0; r := UPPER(m_table) + 1;
  DO WHILE l + 1 /= r;
    m := (l + r) / 2;  -- 1 <= m <= N
    IF /* a(m) < x */ isless( a(m), x ) THEN l := m;
    ELSE             r := m;  
    FI; 
  OD;
  --  a[l] <= x < a[r] mit 0 <= l <= N
  RETURN IF r <= UPPER(m_table) ANDIF a(r) = x THEN r ELSE 0 FI;
END binsearch;

--
-- quicksort
--
quicksort: PROC ( a m_table LOC );

-- compiler can't handle access to global LOC parameter, so we need 1st par.
  sort: PROC (a m_table LOC, low, upp INT IN) RECURSIVE;
    DCL i, j INT;
    DCL x, h m_item;
    --
    -- determine mid-position x
    x := a((low+upp)/2);
    --
    -- partition
    i := low;  j := upp;
    DO WHILE i <= j;
      DO WHILE /* a(i) < x */ isless( a(i), x ); i +:= 1; OD;
      DO WHILE /* x < a(j) */ isless( x, a(j) ); j -:= 1; OD;
        IF i <= j THEN
          h := a(i);  a(i) := a(j);  a(j) := h;
          i +:= 1;  j -:= 1;
        FI;
    OD;
    --
    -- sort partitions
    IF low < j THEN sort( a, low, j ); FI;
    IF i < upp THEN sort( a, i, upp ); FI;
  END sort;

  sort( a, 1, UPPER(m_table) );
END quicksort;

a := [
"Altermann Robert, Dipl.-Ing.            ",
"Biro Jozsef                             ",
"Dolhay Mark                             ",
"Grasegger Siegfried                     ",
"Gruber Thomas                           ",
"Gyenes Karoly                           ",
"Nagy Jenoe                              ",
"Sauerwein Kurt                          ",
"Steinbrecher Helmut, Dir. Ing.          ",
"Szekely Bela                            ",
"Vajda Sandor                            ",
"Jost Karin                              ",
"Akhavan Pirouz                          ",
"Astl Cathrin, Dipl.-Ing.                ",
"Boyer Hannes, Ing.                      ",
"Damianitsch Gerhard, Ing.               ",
"Di Mattio Gerhard, Mag.                 ",
"Eber Gerald, Dipl.-Ing.                 ",
"Fritzl Brigitte                         ",
"Fuss Walter, Dipl.-Ing.                 ",
"Hoerandl August, Dipl.-Ing.             ",
"Hoeslinger Raimund, Dipl.-Ing.          ",
"Huber Ernst, Dipl.-Ing.                 ",
"Kohler Armin, Ing.                      ",
"Kral Dietrich, Ing.                     ",
"Luethi Stefan                           ",
"Messinger Anita, Dipl.-Ing.             ",
"Michaelides Marios                      ",
"Nussmayr Harald, Dipl.-Ing.             ",
"Padourek Karl, Dipl.-Ing.               ",
"Papadaki Maria                          ",
"Preisinger Kurt, Dipl.-Ing.             ",
"Puchleitner Gerald, Dipl.-Ing.          ",
"Reh Wolfgang                            ",
"Samimi Reha, Dipl.-Ing.                 ",
"Schabauer Harald, Dipl.-Ing.            ",
"Schuerer-Waldh. Thomas, Dipl.-Ing.      ",
"Schuetz Werner, Dipl.-Ing. Dr.          ",
"Staffel Gerhard, Dr.                    ",
"Tinlayan Ali, Ing.                      ",
"Wey Friedrich, Ing.                     ",
"Zeiler Wolfgang, Ing.                   ",
"Zirps Hannes                            ",
"Eckstein Hannes, Ing.                   ",
"Fuss Gerhard, Ing.                      ",
"Pichlbaumgartne Peter                   ",
"Sacher Curt, Ing.                       ",
"Wenighofer Franz                        ",
"Berger Johann                           ",
"Boehm Petr, Dipl.-Ing.                  ",
"Dworzak Franz, Dipl.-Ing.               ",
"Erb Andreas, Dipl.-Ing.                 ",
"Gratcl Paul                             ",
"Hammer Johannes, Ing.                   ",
"Kitzler Eva, Dipl.-Ing.                 ",
"Koegl Johann, Ing.                      ",
"Krasny Sascha                           ",
"Krotz Werner, Dr.                       ",
"Kucera Alfred, Ing.                     ",
"Radatz Gerhard                          ",
"Schneider Martin, Dipl.-Ing.            ",
"Schoen Heimo, Ing.                      ",
"Schott Karl                             ",
"Selami Gholam-Hos                       ",
"Jahoda Reinhard                         ",
"Rigerl Herbert, Ing.                    ",
"Fritz Peter, Ing.                       ",
"Glaser Erwin, Ing.                      ",
"Kristen Erwin, Ing.                     ",
"Matiz Walter, Ing.                      ",
"Vinazzer Claudius, Dipl.-Ing.           ",
"Aigner Reinhold                         ",
"Baumruck Manfred                        ",
"Eiweck Julius                           ",
"Frece Peter, Ing.                       ",
"Trimmel Franz, Ing.                     ",
"Artner Gerald                           ",
"Baumgartner Dieter, Dipl.-Ing.          ",
"Benisch Guenter, Ing.                   ",
"Boehm Arthur                            ",
"Leitgeb Christian, Dipl.-Ing.           ",
"Puhr Georg                              ",
"Rauch Herbert, Ing.                     ",
"Rykr Otto                               ",
"Staringer Helmut                        ",
"Telefont Heinz, Ing.                    ",
"Urban Friedrich, Ing.                   ",
"Weithofer Walter, Dipl.-Ing.            ",
"Boehm Walter, Dipl.-Ing.                ",
"Laun Wolfgang, Dipl.-Ing. Dr.           "
];

quicksort( a );
dump( a, 1, UPPER(m_table) );

key := "Laun Wolfgang, Dipl.-Ing. Dr.           ";
i := binsearch( a, key );
IF i /= 0 THEN
  WRITETEXT( stdout, key // "%/found at position %C%/", i );
ELSE
  WRITETEXT( stdout, key // "%/not found...%/" );
FI;

END sortsearch;
