vector: MODULE

SYNMODE m_index = RANGE(1:10);
NEWMODE vector = ARRAY (m_index) INT;

DCL a, b, c vector;

-- sum computes the sum of the elements of a vector. 
sum: PROC (a READ vector LOC) RETURNS (INT);
  DCL el, s INT;
  s := 0;
  DO FOR el IN a;
    s +:= el;
  OD;
  RESULT s;
END sum;

-- count returns the number of positive array elements of a vector.
count: PROC (a READ vector LOC) RETURNS (INT);
   DCL el, c INT;
   c := 0;
   DO FOR el IN a;
     IF el > 0 THEN c +:= 1; FI;
   OD;
   RESULT c;
END count;


-- compare vector element by element, returning the number of inequalities.
compare: PROC (a, b READ vector LOC) RETURNS (INT);
   DCL i m_index, 
       count INT;
   count:= 0;
   DO FOR i IN m_index;
      IF a(i)/= b(i) THEN count +:= 1; FI;
   OD;
   RESULT count;
END compare;


-- tally zero elements in a vector the index of which is a multiple of 4. 

tally: PROC (a READ vector LOC) RETURNS (INT);
   DCL i, c, b INT;
   c := 0;
   IF LOWER(a) MOD 4 = 0 THEN 
     b := LOWER(a); 
   ELSE 
     b := LOWER(a) + 4 - LOWER(a) MOD 4;
   FI;
   DO FOR i := b BY 4 TO UPPER(a);
     IF a(i) = 0 THEN c +:= 1; FI;
   OD;
   RESULT c;
END tally;


-- imin returns smallest IMIN in vector with A(IMIN)=0

imin: PROC (a /*READ*/ vector LOC) RETURNS (INT) EXCEPTIONS (NotFound);
   DCL j, i INT;
   i := LOWER(a);
/***** Workaround ***/
   DO FOR j IN a WHILE j /= 0;
     i := SUCC( i );
   OD;
   IF i = UPPER(a) + 1 THEN CAUSE NotFound; ELSE RESULT i; FI;
/***** for: ****
   DO FOR i := LOWER(a) TO UPPER(a);
     IF a(i) = 0 THEN RETURN i; FI;
   OD;
   CAUSE NotFound;
/*** End of Workaround ***/
END imin;


-- insert positive value into array maintaining ascending order.

insert: PROC (a vector LOC, x INT) EXCEPTIONS(NotFree);
   DCL i RANGE (PRED(LOWER(vector)) : UPPER(vector)),
       n RANGE (LOWER(vector) : UPPER(vector)),
       j INT;

   /* i := maximum { i | a(i) > 0 } */
   i:= LOWER(i);
/******** Workaround ***/
   DO FOR j IN a WHILE j > 0;
     i := SUCC(i);
   OD;
/********* for: ******
   loop: DO FOR j IN a;
     IF j = 0 THEN EXIT loop; FI;
     i := SUCC(i);
   OD;
/*** End of Workaraound ***/
   IF i = UPPER(a) THEN CAUSE NotFree; FI;
   /* if not full, insert value of x */
   n:= SUCC(i);
   DO FOR i:= i DOWN TO 1 WHILE a(i) > x;
     a(SUCC(i)) := a(i); n:= i;
   OD;
   a(n):= x;
END insert;

dump: PROC( a vector LOC, c CHAR );
  DCL i m_index;
  DO FOR i IN m_index;
    WRITETEXT( stdout, "%C(%C)=%C ", c, i, a(i) );
  OD;
  WRITETEXT( stdout, "%/" );
END dump;

a := vector [ 1, -1, 2, -2, 3, -3, 4, -4, 5, -5 ];
b := a;
b(4) := 4;
b(7) := 7;
c := vector [(*): 0];

dump(a,'a');
WRITETEXT( stdout, "sum %C%/", sum( a ));
WRITETEXT( stdout, "positive elements %C%/", count( a ));
dump(b,'b');
WRITETEXT( stdout, "differences %C%/", compare( a, b ));

dump(c,'c');
WRITETEXT( stdout, "zero elements at (4*k): %C%/", tally( c ));

insert( c, 13 );
insert( c, 2  );
insert( c, 17 );
insert( c, 11 );
insert( c, 3  );
insert( c, 7  );
insert( c, 5  );
dump(c,'c');
WRITETEXT( stdout, "smallest index with element 0: %C%/", imin( c ));

END vector;
