/* CHILL compiler regression test file
 Copyright (C) 1992, 1993 Free Software Foundation, Inc.
 
 This file is part of GNU CC.

 GNU CC is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 GNU CC is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GNU CC; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

rngfail: MODULE

-- this module is supposed to check all possibilities for
-- raising a rangefail exception during compile time
-- It's a derivative of the execute/rangefail.ch runtime test.
-- Let's keep them parallel as much as possible.

-- global used variables and functions 
DCL what_test CHAR (100) VARYING;

success: PROC ();
   WRITETEXT (stdout, what_test // " PASSED%/");
END success;

failed: PROC ();
   WRITETEXT (stdout, what_test // " FAILED%/");
END failed;

/*
integer to integer assignment:
There are three types of assignment where rangefail check is neccessary:

 a) assign longer INT to shorter INT
 b) assign signed to unsigned
 c) assign unsigned to signed at equal size

                       RHS
         +-------------------------------------------+
LHS      | UBYTE | BYTE  | UINT | INT | ULONG | LONG |
---------+-------+-------+------+-----+-------+------+
UBYTE    |       |  Xb   |  Xa  |  Xa |  Xa   |  Xa  |
---------+-------+-------+------+-----+-------+------+
BYTE     |   Xc  |       |  Xa  |  Xa |  Xa   |  Xa  |
---------+-------+-------+------+-----+-------+------+
UINT     |       |  Xb   |      |  Xb |  Xa   |  Xa  |
---------+-------+-------+------+-----+-------+------+
INT      |       |       |  Xc  |     |  Xa   |  Xa  |
---------+-------+-------+------+-----+-------+------+
ULONG    |       |  Xb   |      |  Xb |       |  Xb  |
---------+-------+-------+------+-----+-------+------+
LONG     |       |       |      |     |  Xc   |      |
---------+-------+-------+------+-----+-------+------+

FIXME: Need SET type test(s) here, tests of parameter passing,
       passing to LOC/INOUT parameters.
*/
int_int_assgn: PROC ();
   DCL b BYTE, ub UBYTE, i INT, ui UINT, l LONG, ul ULONG;

   what_test := "UBYTE := BYTE:";
--   b := -10;
   ub := -10 ON (rangefail): success (); GOTO uint_ubyte; END;
   failed ();
uint_ubyte:
   what_test := "UBYTE := UINT:";
--   ui := 300;
   ub := 300 ON (rangefail): success (); GOTO int_ubyte; END;
   failed ();
int_ubyte:
   what_test := "UBYTE := INT:";
--   i := -300;
   ub := -300 ON (rangefail): success (); GOTO ulong_ubyte; END;
   failed ();
ulong_ubyte:
   what_test := "UBYTE := ULONG:";
--   ul := 100000;
   ub := 100000 ON (rangefail): success (); GOTO long_ubyte; END;
   failed ();
long_ubyte:
   what_test := "UBYTE := LONG:";
--   l := -100000;
   ub := -100000 ON (rangefail): success (); GOTO byte_ubyte; END;
   failed ();
byte_ubyte:
   what_test := "BYTE := UBYTE:";
--   ub := 255;
   b := 255 ON (rangefail): success (); GOTO uint_byte; END;
   failed ();
uint_byte:
   what_test := "BYTE := UINT:";
--   ui := 10000;
   b := 10000 ON (rangefail): success (); GOTO int_byte; END;
   failed ();
int_byte:
   what_test := "BYTE := INT:";
--   i := -10000;
   b := -10000 ON (rangefail): success (); GOTO ulong_byte; END;
   failed ();
ulong_byte:
   what_test := "BYTE := ULONG:";
--   ul := 100000;
   b := 100000 ON (rangefail): success (); GOTO long_byte; END;
   failed ();
long_byte:
   what_test := "BYTE := LONG:";
--   l := -100000;
   b := -100000 ON (rangefail): success (); GOTO byte_uint; END;
   failed ();
byte_uint:
   what_test := "UINT := BYTE:";
--   b := -100;
   ui := -100 ON (rangefail): success (); GOTO int_uint; END;
   failed ();
int_uint:
   what_test := "UINT := INT:";
--   i := -100;
   ui := -100 ON (rangefail): success (); GOTO ulong_uint; END;
   failed ();
ulong_uint:
   what_test := "UINT := ULONG:";
--   ul := 100000;
   ui := 100000 ON (rangefail): success (); GOTO long_uint; END;
   failed ();
long_uint:
   what_test := "UINT := LONG:";
--   l := -100000;
   ui := -100000 ON (rangefail): success (); GOTO uint_int; END;
   failed ();
uint_int:
   what_test := "INT := UINT:";
--   ui := 65535;
   i := 65535 ON (rangefail): success (); GOTO ulong_int; END;
   failed ();
ulong_int:
   what_test := "INT := ULONG:";
--   ul := 100000;
   i := 100000 ON (rangefail): success (); GOTO long_int; END;
   failed ();
long_int:
   what_test := "INT := LONG:";
--   l := -100000;
   i := -100000 ON (rangefail): success (); GOTO byte_ulong; END;
   failed ();
byte_ulong:
   what_test := "ULONG := BYTE:";
--   b := -100;
   ul := -100 ON (rangefail): success (); GOTO int_ulong; END;
   failed ();
int_ulong:
   what_test := "ULONG := INT:";
--   i := -10000;
   ul := -10000 ON (rangefail): success (); GOTO long_ulong; END;
   failed ();
long_ulong:
   what_test := "ULONG := LONG:";
--   l := -100000;
   ul := -100000 ON (rangefail): success (); GOTO ulong_long; END;
   failed ();
ulong_long:
   what_test := "LONG := ULONG:";
--   ul := H'88880000;
   l := H'88880000 ON (rangefail): success (); RETURN; END;
   failed ();
END int_int_assgn;

int_int_assgn ();

END rngfail;
