%{
/*
 * Remove comments from a CHILL source file.
 * Ignores comment start/end sequences inside
 * string literals.
 */
int lineno = 1;
int startline;
%}
%x COM1
%x COM2
%x LIT1
%x LIT2
%%
<INITIAL>"/*"  { startline = lineno; BEGIN (COM1); }
<COM1>[^\n]    { ; }
<COM1>\n       { lineno++; }
<COM1>"*/"     { BEGIN (INITIAL); }

<INITIAL>"--"  { startline = lineno; BEGIN (COM2); }
<COM2>[^\n]    { ; }
<COM2>\n       { lineno++; ECHO; BEGIN (INITIAL); }

<INITIAL>\"    { startline = lineno; ECHO; BEGIN (LIT1); }
<LIT1>[^"]     { ECHO; }
<LIT1>\"       { ECHO; BEGIN (INITIAL); }

<INITIAL>\'    { startline = lineno; ECHO; BEGIN (LIT2); }
<LIT2>[^']     { ECHO; }
<LIT2>\'       { ECHO; BEGIN (INITIAL); }

<INITIAL>.     { ECHO; }
<INITIAL>\n    { lineno++; ECHO; }

<COM1><<EOF>>  { fprintf (stderr, "error EOF looking for `*/' "); 
                 fprintf (stderr, "to match `/*' seen on line %d\n", startline); 
                 exit (1);
               }
<LIT1><<EOF>>  { fprintf (stderr, "error: EOF looking for `\"' ");
                 fprintf (stderr, "to match `\"' seen on line %d\n", startline); 
                 exit (1);
               }
<LIT2><<EOF>>  { fprintf (stderr, "error: EOF looking for `\'' "); 
                 fprintf (stderr, "to match `'' seen on line %d\n", startline); 
                 exit (1);
               }
%%
int main (argc, argv)
	int argc;
	char *argv[];
{
  yylex ();
  exit (0);
}


