#ifndef lint
static char rcsid[]= "$Header: /rel/cvsfiles/devo/gcc/ch/utils/printf.c,v 1.1 1994/03/07 19:34:00 bill Exp $";
#endif

/*
 * printf - duplicate the C library routine of the same name, but from
 * the shell command level.
 *
 * This version by Chris Torek, based on an earlier version by Fred Blonder.
 */

#include <stdio.h>
#include <ctype.h>
#include <sysexits.h>

char	*progname;

char	*ctor(), **doit();
double	atof();
int	atoi();
long	atol();

main(argc, argv)
     int argc;
     char **argv;
{
  register char *cp, *convp, **ap, **ep;
  register int ch, ndyn, flags;
  char cbuf[BUFSIZ];	/* separates each conversion */
  static char hasmod[] = "has integer length modifier";
  
  /* flags */
#define	LONG	1
#define	SHORT	2
  
  ap = argv;
  ep = &ap[argc];
  progname = *ap++;
  if (argc < 2) {
    (void) fprintf(stderr,
		   "%s: Usage: %s <format-string> [ arg1 . . . ]\n",
		   progname, progname);
    exit(EX_USAGE);
  }
  
  ctrl(cp = *ap++);	/* backslash interpretation of fmt string */
  
  /*
   * Scan format string for conversion specifications.
   * (The labels would be loops, but then everything falls
   * off the right.)
   */
 scan:
  while ((ch = *cp++) != '%') {
    if (ch == 0)
      exit(EX_OK);
    (void) putchar(ch);
  }
  
  ndyn = 0;
  flags = 0;
  convp = cbuf;
  *convp++ = ch;
  
  /* scan for conversion character */
 cvt:
  switch (ch = *cp++) {
    
  case '\0':	/* unterminated conversion */
    exit(EX_OK);
    
    /* string or character format */
  case 'c': case 's':
    if (flags)
      illfmt(cbuf, convp, ch, hasmod);
    ap = doit(cbuf, convp, ap, ep, ndyn, ch, ch);
    goto scan;
    
    /* integer formats */
  case 'd': case 'i': case 'o': case 'u': case 'x': case 'X':
    if ((flags & (LONG|SHORT)) == (LONG|SHORT))
      illfmt(cbuf, convp, ch, "is both long and short");
    ap = doit(cbuf, convp, ap, ep, ndyn, ch,
	      flags & LONG ? 'l' : flags & SHORT ? 'h' : 'i');
    goto scan;
    
    /* floating point formats */
  case 'e': case 'E': case 'f': case 'g': case 'G':
    if (flags)
      illfmt(cbuf, convp, ch, hasmod);
    ap = doit(cbuf, convp, ap, ep, ndyn, ch, 'f');
    goto scan;
    
    /* Roman (well, why not?) */
  case 'r': case 'R':
    if (flags)
      illfmt(cbuf, convp, ch, hasmod);
    ap = doit(cbuf, convp, ap, ep, ndyn, 's', ch);
    goto scan;
    
  case '%':	/* boring */
    (void) putchar('%');
    goto scan;
    
    /* short integers */
  case 'h':
    flags |= SHORT;
    break;
    
    /* long integers */
  case 'l':
    flags |= LONG;
    break;
    
    /* field-width or precision specifier, or flag: keep scanning */
  case '.': case '#': case '-': case '+': case ' ':
  case '0': case '1': case '2': case '3': case '4':
  case '5': case '6': case '7': case '8': case '9':
    break;
    
    /* dynamic field width or precision: count it */
  case '*':
    ndyn++;
    break;
    
  default:	/* something we cannot handle */
    if (isascii(ch) && isprint(ch))
      cbuf[0] = ch, cbuf[1] = 0;
    else
      (void) sprintf(cbuf, "\\%03o", (unsigned char)ch);
    (void) fprintf(stderr,
		   "%s: illegal conversion character `%s'\n",
		   progname, cbuf);
    exit(EX_USAGE);
    /* NOTREACHED */
  }
  
  /* 2 leaves room for ultimate conversion char and for \0 */
  if (convp >= &cbuf[sizeof(cbuf) - 2]) {
    (void) fprintf(stderr, "%s: conversion string too long\n",
		   progname);
    exit(EX_USAGE);
  }
  *convp++ = ch;
  goto cvt;
}

illfmt(cbuf, convp, ch, why)
     char *cbuf, *convp;
     int ch;
     char *why;
{
  
  *convp++ = ch;
  *convp = 0;
  (void) fprintf(stderr, "%s: format `%s' illegal: %s\n",
		 progname, cbuf, why);
  exit(EX_USAGE);
}

/*
 * Emit a conversion.  cch holds the printf format character for
 * this conversion; cty holds a simplified version (all integer
 * conversions, e.g., are represented as 'i').
 */
char **
doit(cbuf, convp, ap, ep, ndyn, cch, cty)
     char *cbuf, *convp;
     register char **ap;
     char **ep;
     register int ndyn;
     int cch, cty;
{
  register char *s;
  union {		/* four basic conversion types */
    int i;
    long l;
    double d;
    char *str;
  } arg;
  int a1, a2;	/* dynamic width and/or precision */
  
  /* finish off the conversion string */
  s = convp;
  *s++ = cch;
  *s = 0;
  s = cbuf;
  
  /* verify number of arguments */
  if (&ap[ndyn] >= ep) {
    (void) fprintf(stderr,
		   "%s: not enough args for format `%s'\n",
		   progname, s);
    exit(EX_USAGE);
  }
  
  /* pick up dynamic specifiers */
  if (ndyn) {
    a1 = atoi(*ap++);
    if (ndyn > 1)
      a2 = atoi(*ap++);
    if (ndyn > 2) {
      (void) fprintf(stderr,
		     "%s: too many `*'s in `%s'\n",
		     progname, s);
      exit(EX_USAGE);
    }
  }
  
#define	PRINTF(what) \
  if (ndyn == 0) \
    (void) printf(s, what); \
  else if (ndyn == 1) \
    (void) printf(s, a1, what); \
  else \
    (void) printf(s, a1, a2, what);
  
  /* emit the appropriate conversion */
  switch (cty) {
    
    /* string */
  case 's':
    ctrl(arg.str = *ap++);
    goto string;
    
    /* roman (much like string) */
  case 'r': case 'R':
    arg.str = ctor(atoi(*ap++), cty == 'R');
  string:
    PRINTF(arg.str);
    break;
    
    /* floating point */
  case 'f':
    arg.d = atof(*ap++);
    PRINTF(arg.d);
    break;
    
    /* character */
  case 'c':
    ctrl(*ap);
    arg.i = *(*ap++);
    goto integer;
    
    /* short integer */
  case 'h':
    arg.i = (short) atoi(*ap++);
    goto integer;
    
    /* integer */
  case 'i':
    arg.i = atoi(*ap++);
  integer:
    PRINTF(arg.i);
    break;
    
    /* long integer */
  case 'l':
    arg.l = atol(*ap++);
    PRINTF(arg.l);
    break;
  }
  return (ap);
}

/*
 * Return the index of the character c in the string s; character 0
 * is NOT considered part of the string (unlike index() or strchr()).
 * If c is not found (or is 0), return -1.
 *
 * This is used for hex and octal digit conversions in ctrl().
 */
int
digit(s, c)
     char *s;
     register int c;
{
  register char *p;
  
  for (p = s; *p; p++)
    if (*p == c)
      return (p - s);
  return (-1);
}

/*
 * Convert backslash notation to control characters, in place.
 */
ctrl(s)
     register char *s;
{
  register char *op = s;
  register int v, c;
  static char oct[] = "01234567";
  static char hex[] = "0123456789abcdefABCDEF";
  
  while ((c = *s++) != 0) {
    if (c != '\\') {
      *op++ = c;
      continue;
    }
    switch (*s++) {
    case '\0':	/* end-of-string: user goofed */
      s--;
      break;
      
    case '\\':	/* backslash */
      *op++ = '\\';
      break;
      
    case 'n':	/* newline */
      *op++ = '\n';
      break;
      
    case 't':	/* horizontal tab */
      *op++ = '\t';
      break;
      
    case 'r':	/* carriage-return */
      *op++ = '\r';
      break;
      
    case 'f':	/* form-feed */
      *op++ = '\f';
      break;
      
    case 'b':	/* backspace */
      *op++ = '\b';
      break;
      
    case 'v':	/* vertical tab */
      *op++ = '\13';
      break;
      
    case 'a':	/* WARNING! DANGER! DANGER! DANGER! */
      *op++ = '\7';
      break;
      
    case '0': case '1': case '2': case '3':
    case '4': case '5': case '6': case '7':
      /* octal constant, 3 digits maximum */
      v = digit(oct, s[-1]);
      if ((c = digit(oct, *s)) >= 0) {
	v = (v << 3) + c;
	if ((c = digit(oct, *++s)) >= 0) {
	  v = (v << 3) + c;
	  s++;
	}
      }
      *op++ = v;
      break;
      
    case 'x':	/* hex constant */
      v = 0;
      while ((c = digit(hex, *s)) >= 0) {
	if (c >= 16)
	  c -= 6;
	v = (v << 4) + c;
	s++;
      }
      *op++ = v;
      break;
      
      /*
       * The name of this object is taken from troff:
       * \z might be better, but this has a precedent.
       * It exists solely so that we can end a hex constant
       * which must be followed by a legal hex character.
       */
    case '&':	/* special zero-width `character' */
      break;
      
    default:
      *op++ = s[-1];
    }
  }
  *op = '\0';
}

/*
 * Convert integer to Roman Numerals. (How have you survived without it?)
 */
char *
ctor(x, caps)
     int x, caps;
{
  static char buf[BUFSIZ];
  register char *outp = buf;
  register unsigned n = x;
  register int u, v;
  register char *p, *q;
  
  if ((int)n < 0) {
    *outp++ = '-';
    n = -n;
  }
  p = caps ? "M\2D\5C\2L\5X\2V\5I" : "m\2d\5c\2l\5x\2v\5i";
  v = 1000;
  if (n >= v * BUFSIZ / 2)	/* conservative */
    return ("[abortive Roman numeral]");
  for (;;) {
    while (n >= v)
      *outp++ = *p, n -= v;
    if (n == 0)
      break;
    q = p + 1;
    u = v / *q;
    if (*q == 2)		/* magic */
      u /= *(q += 2);
    if (n + u >= v) {
      *outp++ = *++q;
      n += u;
    } else {
      p++;
      v /= *p++;
    }
  }
  *outp = 0;
  return (buf);
}
