(defsubst mb (num)
  (buffer-substring (match-beginning num) (match-end num)))

(defmacro += (sym &rest args)
  (list 'setq sym (append '(funcall '+) (cons sym args))))

(defsubst tcov-fnfinish ()
  (save-excursion
    (goto-char old-fnstart)
    (search-forward "\n}")
    (+= lines (count-lines p (point)))
    (and (> count 0) (+= elines (count-lines p (point)))))
  (tcov-printout func lines elines outbuf))

(defun tcov-fnstart ()
  (save-excursion
    (if (eq (following-char) ?{)
	t
      (search-backward "\n{")
      (forward-char 1))
    (point)))
  
(defun tcov-fnparse (cb db outbuf)
  (setq db (get-buffer db))
  (set-buffer db)
  (goto-char (point-min))
  (setq cb (get-buffer cb))
  (set-buffer cb)
  (let (f (old-fnstart -1) line ncount oline p count func (lines 0) (elines 0))
    (while (not (save-excursion (set-buffer db) (looking-at "\n\\'")))
      (setq oline line
	    line (read db)
	    count ncount
	    ncount (read db))
      (goto-line line)
      (if (and p (< (point) p))		; The increment part of a for loop
	  (setq line oline		; ignore it, it doesn't matter
		ncount count)
	(cond ((/= (setq f (tcov-fnstart)) old-fnstart)
	       (if func (tcov-fnfinish))
	       (setq old-fnstart f)
	       (let ((o (point-min)))
		 (if func
		     (save-excursion
		       (re-search-backward "^}")
		       (setq o (point))))
		 (if (save-excursion
		       (or (re-search-backward
			    "^ *\\([_a-z$][_a-z$0-9]*\\) *(" o t)
			   ;; int foo (bar)
			   (re-search-backward
			    "^[a-z]+ +\\([_a-z$][_a-z$0-9]*\\) *(" o t)))
		       (setq func (mb 1))
		     (setq func "*UNPARSED*")))
	       (setq lines 0
		     elines 0)
	       (message func))
	      (t
	       (+= lines (count-lines p (point)))
	       (and (> count 0) (+= elines (count-lines p (point))))))
	(setq p (point))))
    (tcov-fnfinish)))

(defun basename (filename)
  (let ((s 0))
    (while (string-match "/" filename s)
      (setq s (match-end 0)))
    (substring filename s)))

(defvar cf nil "Current file")
(defvar cump 0 "Cumulative coverage percentage")
(defvar cumf 0 "Cumulative function count")

(defun tcov-printout (func lines elines outbuf)
  (save-excursion
    (set-buffer outbuf)
    (goto-char (point-max))
    (insert (format "%-12s " (basename cf))
	    (format "%-40s " func)
	    (format "%3d    " lines)
	    (format "%3d    " elines)
	    (format "%3d%%" (/ (* 100 elines) lines))
	    "\n")
    (+= cump (/ (* 100 elines) lines))
    (+= cumf 1)))

(defun tcov ()
  (interactive)
  (let (fs df cb db of)
    (if noninteractive
	(setq of (find-file-noselect (car command-line-args-left))
	      fs (mapcar 'expand-file-name (cdr command-line-args-left)))
      (setq fs (directory-files "~/b/chill/devo/gcc" 'full-name "^ch-.*\.c$")
	    of (get-buffer-create "*tcov by function*")))
    (setq fs (cons 'ignore fs)
	  cump 0
	  cumf 0)
    (set-buffer of)
    (goto-char (point-max))
    (insert "
Filename     Function                                Lines  Lines  Percent
                                                     Total Covered Coverage
------------ --------------------------------------   ---    ---    ----
")
    (while (setq fs (cdr fs))
      (setq cf (car fs))
      (if (or (string-match "ch-parse.c" cf)
	      (string-match "ch-lex.c" cf))
	  t
	(setq df (copy-sequence cf))
	(aset df (1- (length df)) ?d)
	(setq db (find-file-noselect df))
	(setq cb (find-file-noselect cf))
	(set-buffer of)
	(goto-char (point-max))
	(message (format "Doing %s" cf))
	(tcov-fnparse cb db of)
	(message (format "Done with %s" cf))))
    (set-buffer of)
    (goto-char (point-max))
    (insert (format "\n%d functions, %d%% av. coverage\n" cumf (/ cump cumf)))
    (if noninteractive
	(save-buffer)
      (switch-to-buffer of))))
